/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.filter;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.archam.core.exception.ResourceNotFoundException;
import es.xunta.amtega.transversal.archam.core.web.context.ExecutionContext;
import es.xunta.amtega.transversal.configuration.environment.EnvironmentVariables;
import es.xunta.amtega.transversal.utils.BaseUtils.Configuration;

/**
 * <code>ExecutionContextFilter</code> es un filtro que añade
 * información del entorno al contexto de la aplicación.
 */
public class ExecutionContextFilter implements Filter {

    private static final Logger LOGGER = LogManager.getLogger(ExecutionContextFilter.class);

    private static final String NA = "NA";

    private static final String PATH_ARQUETIPO_PACKAGES_PROPERTIES = "META-INF/archetype.version";
    private static final String PROPERTY_ARQUETIPO_ARTIFACTID = "archetype.artifactId";
    private static final String PROPERTY_ARQUETIPO_VERSION = "archetype.version";

    private ExecutionContext executionContext = null;

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        executionContext = createExecutionContext();
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {

        request.setAttribute("executionContext", executionContext);

        chain.doFilter(request, response);
    }

    @Override
    public void destroy() {
        // Nothing to do
    }

    /**
     * Carga en un objeto <code>ExecutionContext</code> información del entorno.
     *
     * <ul>
     * <li>Versión. En caso de no constar en las propiedades devuelve el valor
     * <code>NA</code>.
     * <li>Código de la aplicación, obtenido de las propiedades.
     * <li>Entorno, obtenido de las variables de entorno.
     * <li>Nombre del servidor, obtenido de las variables de entorno.
     * </ul>
     *
     * @return el contexto de ejecución.
     */
    private ExecutionContext createExecutionContext() {

        String version = Configuration.getProperty("app.version", NA);
        String appCode = Configuration.getApplicationCode();
        String environment = EnvironmentVariables.getEnvironment();
        String serverName = EnvironmentVariables.getServerName();
        Long startup = System.currentTimeMillis();

        return new ExecutionContextImpl(version, environment, serverName, appCode, startup);
    }

    /**
     * <code>ExecutionContextImpl</code> implementa la información del contexto
     * de la aplicación.
     */
    private class ExecutionContextImpl implements ExecutionContext {

        private final String version;
        private final String environment;
        private final String serverName;
        private final String appCode;
        private final long startUp;

        /**
         * Constructor con la parámetros del contexto.
         * @param version la versión de la aplicación.
         * @param environment el entorno de ejecución.
         * @param serverName el nombre del servidor.
         * @param appCode el código de la aplicación.
         * @param starUp momento de inicio de la aplicación
         */
        public ExecutionContextImpl(String version, String environment, String serverName, String appCode, long startUp) {
            this.version = version;
            this.environment = environment;
            this.serverName = serverName;
            this.appCode = appCode;
            this.startUp = startUp;
        }

        @Override
        public String getVersion() {
            return version;
        }

        @Override
        public String getArchetypeId() {
            return readArchetypeProperty(PROPERTY_ARQUETIPO_ARTIFACTID);
        }

        @Override
        public String getArchetypeVersion() {
            return readArchetypeProperty(PROPERTY_ARQUETIPO_VERSION);
        }

        /**
         * Recupera la versión del arquetipo utilizada en la generación de la aplicación
         * @return Versión
         */
        private String readArchetypeProperty(String property) {
            InputStream is = this.getClass().getClassLoader()
                    .getResourceAsStream(PATH_ARQUETIPO_PACKAGES_PROPERTIES);
            Properties props = new Properties();

            try {
                props.load(is);
            } catch (Exception e) {
                ResourceNotFoundException rnfe = new ResourceNotFoundException(
                        "Error al leer fichero archetype.version", e);
                LOGGER.error("No se ha podido obtener la versión del arquetipo", rnfe);
                throw rnfe;
            } finally {
                if(is!=null){
                    try {
                        is.close();
                    } catch (IOException e) {
                        LOGGER.error("Error cerrando recurso", e);
                    }
                }
            }

            return props.getProperty(property, NA);
        }

        @Override
        public String getEnvironment() {
            return environment;
        }

        @Override
        public String getServerName() {
            return serverName;
        }

        @Override
        public String getAppCode() {
            return appCode;
        }

        @Override
        public long getStartUp() {
            return startUp;
        }

    }
}
