/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.mvc;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.web.servlet.support.RequestDataValueProcessor;

/**
 * <code>ChainedRequestDataValueProcessor</code> permite trabajar
 * simultáneamente con varias implementaciones del tipo
 * <code>RequestDataValueProcessor</code>.
 */
public class ChainedRequestDataValueProcessor implements RequestDataValueProcessor {

    private final Set<RequestDataValueProcessor> processors = new HashSet<>();

    /**
     * Añade un <code>RequestDataValueProcessor</code> a una colección de
     * procesadores de ese mismo tipo.
     *
     * @param processor el procesador a añadir.
     */
    public void add(RequestDataValueProcessor processor) {
        this.processors.add(processor);
    }

    /**
     * Establece un conjunto de procesadores
     * <code>RequestDataValueProcessor</code>.
     *
     * @param processors los procesadores a establecer
     */
    public void setProcessors(Set<RequestDataValueProcessor> processors) {
        this.processors.clear();
        this.processors.addAll(processors);
    }

    @Override
    public String processAction(HttpServletRequest request, String action, String httpMethod) {
        for (RequestDataValueProcessor processor : processors) {
            String result = processor.processAction(request, action, httpMethod);
            if (!String.valueOf(result).equals(action)) {
                return result;
            }
        }

        return action;
    }

    @Override
    public String processFormFieldValue(HttpServletRequest request, String name, String value, String type) {
        for (RequestDataValueProcessor processor : processors) {
            String result = processor.processFormFieldValue(request, name, value, type);
            if (!String.valueOf(result).equals(value)) {
                return result;
            }
        }

        return value;
    }

    @Override
    public Map<String, String> getExtraHiddenFields(HttpServletRequest request) {
        Map<String, String> result = new HashMap<>();
        for (RequestDataValueProcessor processor : processors) {
            result.putAll(processor.getExtraHiddenFields(request));
        }

        return result;
    }

    @Override
    public String processUrl(HttpServletRequest request, String url) {
        for (RequestDataValueProcessor processor : processors) {
            String result = processor.processUrl(request, url);
            if (!String.valueOf(result).equals(url)) {
                return result;
            }
        }

        return url;
    }

}
