/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.mvc;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.UUID;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.util.Assert;
import org.springframework.web.bind.support.SessionAttributeStore;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;

/**
 * <code>ConversationalSessionAttributeStore</code> es la clase que controla el
 * almacenamiento y recuperación de los atributos en la sesión HTTP.
 */
public class ConversationalSessionAttributeStore implements SessionAttributeStore, BeanPostProcessor {
    private static final Logger LOGGER = LogManager.getLogger(ConversationalSessionAttributeStore.class);
    public static final String CID_FIELD = "_cid";
    public static final String SESSION_MAP = "sessionConversationMap";
    public static final String WEBREQUEST_NULL_MESSAGE = "WebRequest must not be null";
    public static final String ATTRIBUTE_NAME_NULL_MESSAGE = "Attribute name must not be null";
    public static final int CONVERSATIONS_INIT_VALUE = 10;
    private int keepAliveConversations = CONVERSATIONS_INIT_VALUE;

    @Override
    public void cleanupAttribute(WebRequest request, String attributeName) {
        Assert.notNull(request, WEBREQUEST_NULL_MESSAGE);
        Assert.notNull(attributeName, ATTRIBUTE_NAME_NULL_MESSAGE);

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("cleanupAttribute - removing bean reference for (" + attributeName + ") from conversation ("
                    + getConversationId(request) + ").");
        }

        Map<String, Object> conversationStore = getConversationStore(request, getConversationId(request));
        if( conversationStore!=null ){
            conversationStore.remove(attributeName);
            
            // Eliminación del almacén de diálogos de la sesión, cuando este esté vacío.
            if (conversationStore.isEmpty()) {
                getSessionConversationsMap(request).remove(getConversationId(request));
            }
        }
    }

    @Override
    public Object retrieveAttribute(WebRequest request, String attributeName) {
        Assert.notNull(request, WEBREQUEST_NULL_MESSAGE);
        Assert.notNull(attributeName, ATTRIBUTE_NAME_NULL_MESSAGE);

        if (getConversationId(request) != null) {
            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("retrieveAttribute - retrieving bean reference for (" + attributeName
                        + ") for conversation (" + getConversationId(request) + ").");
            }

            return getConversationStore(request, getConversationId(request)).get(attributeName);
        } else {
            return null;
        }
    }

    @Override
    public void storeAttribute(WebRequest request, String attributeName, Object attributeValue) {
        Assert.notNull(request, WEBREQUEST_NULL_MESSAGE);
        Assert.notNull(attributeName, ATTRIBUTE_NAME_NULL_MESSAGE);
        Assert.notNull(attributeValue, "Attribute value must not be null");

        String cId = getConversationId(request);
        if (cId == null || cId.trim().length() == 0) {
            cId = UUID.randomUUID().toString();
        }
        request.setAttribute(CID_FIELD, cId, WebRequest.SCOPE_REQUEST);
        LOGGER.debug("storeAttribute - storing bean reference for (" + attributeName + ").");
        store(request, attributeName, attributeValue, cId);
    }

    /**
     * Obtiene el mapa de objetos de un determinado diálogo de la sesión,
     * creándolo si este no existe.
     * <p>
     * El mapa del diálogo se almacena a su vez dentro de un mapa de sesión, que
     * también se crea en el caso de que no exista.
     *
     * @param request la petición recibida.
     * @param conversationId el identificador del diálogo.
     * @return el mapa del diálogo.
     */
    private Map<String, Object> getConversationStore(WebRequest request, String conversationId) {

        Map<String, Object> conversationMap = getSessionConversationsMap(request).get(conversationId);
        if (conversationId != null && conversationMap == null) {
            conversationMap = new HashMap<String, Object>();
            getSessionConversationsMap(request).put(conversationId, conversationMap);
        }

        return conversationMap;
    }

    /**
     * Obtiene los los diálogos de la sesión mapeados.
     *
     * @param request la petición recibida.
     * @return el <code>LinkedHashMap</code> de todos los diálogos y sus mapas.
     */
    private Map<String, Map<String, Object>> getSessionConversationsMap(WebRequest request) {
        @SuppressWarnings("unchecked")
        Map<String, Map<String, Object>> sessionMap = (LinkedHashMap<String, Map<String, Object>>) request
                .getAttribute(SESSION_MAP, WebRequest.SCOPE_SESSION);
        if (sessionMap == null) {
            sessionMap = new LinkedHashMap<String, Map<String, Object>>();
            request.setAttribute(SESSION_MAP, sessionMap, WebRequest.SCOPE_SESSION);
        }

        return sessionMap;
    }

    /**
     * Almacena un objeto en sesión.
     * <p>
     * Si se alcanza el número máximo de diálogos activos eliminará los más
     * antiguos. Cuando este tope sea 0, no efectúan estas eliminaciones.
     *
     * @param request la petición recibida.
     * @param attributeName el nombre del atributo (asignado con la anotación
     * @SessionAttributes del controlador).
     * @param attributeValue el valor del atributo a almacenar.
     */
    private void store(WebRequest request, String attributeName, Object attributeValue, String cId) {

        Map<String, Map<String, Object>> sessionConversationsMap = getSessionConversationsMap(request);

        if (keepAliveConversations > 0 && sessionConversationsMap.size() >= keepAliveConversations
                && !sessionConversationsMap.containsKey(cId)) {

            // Eliminación de los diálogos antiguos.
            String key = sessionConversationsMap.keySet().iterator().next();
            sessionConversationsMap.remove(key);
        }
        getConversationStore(request, cId).put(attributeName, attributeValue);
    }

    public int getKeepAliveConversations() {
        return keepAliveConversations;
    }

    public void setKeepAliveConversations(int numConversationsToKeep) {
        keepAliveConversations = numConversationsToKeep;
    }

    /**
     * Obtiene el parámetro que identifica el diálogo de una petición web.
     *
     * @param request la petición recibida.
     * @return el identificador del diálogo.
     */
    private String getConversationId(WebRequest request) {
        String cid = request.getParameter(CID_FIELD);
        if (cid == null) {
            cid = (String) request.getAttribute(CID_FIELD, WebRequest.SCOPE_REQUEST);
        }

        return cid;
    }

    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) {
        return bean;
    }

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) {
        if( bean instanceof RequestMappingHandlerAdapter ){
            ((RequestMappingHandlerAdapter) bean).setSessionAttributeStore(this);
        }
        
        return bean;
    }

}
