/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.mvc;

import static es.xunta.amtega.transversal.archam.core.web.mvc.Message.MESSAGE_ATTRIBUTE;

import org.springframework.ui.Model;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

/**
 * <code>MessageHelper</code> contiene algunos métodos que, en escenarios de
 * redirección, facilitan la inclusión de mensajes en las vistas.
 */
public final class MessageHelper {

    private MessageHelper() {
        // void
    }

    /**
     * Añade un mensaje de <b>éxito</b> a los atributos de una redirección.
     *
     * @param ra los parámetros de la redirección.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void success(RedirectAttributes ra, String message, Object... args) {
        addAttribute(ra, message, Message.Type.SUCCESS, args);
    }

    /**
     * Añade un mensaje de <b>error</b> a los atributos de una redirección.
     *
     * @param ra los parámetros de la redirección.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void error(RedirectAttributes ra, String message, Object... args) {
        addAttribute(ra, message, Message.Type.DANGER, args);
    }

    /**
     * Añade un mensaje <b>informativo</b> a los atributos de una redirección.
     *
     * @param ra los parámetros de la redirección.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void info(RedirectAttributes ra, String message, Object... args) {
        addAttribute(ra, message, Message.Type.INFO, args);
    }

    /**
     * Añade un mensaje de <b>advertencia</b> a los atributos de una
     * redirección.
     *
     * @param ra los parámetros de la redirección.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void warning(RedirectAttributes ra, String message, Object... args) {
        addAttribute(ra, message, Message.Type.WARNING, args);
    }

    /**
     * Añade un mensaje de <b>éxito</b> al modelo de una vista.
     *
     * @param model el modelo con las variables de la vista.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void success(Model model, String message, Object... args) {
        addAttribute(model, message, Message.Type.SUCCESS, args);
    }

    /**
     * Añade un mensaje de <b>error</b> al modelo de una vista.
     *
     * @param model el modelo con las variables de la vista.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void error(Model model, String message, Object... args) {
        addAttribute(model, message, Message.Type.DANGER, args);
    }

    /**
     * Añade un mensaje <b>informativo</b> al modelo de una vista.
     *
     * @param model el modelo con las variables de la vista.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void info(Model model, String message, Object... args) {
        addAttribute(model, message, Message.Type.INFO, args);
    }

    /**
     * Añade un mensaje de <b>advertencia</b> al modelo de una vista
     *
     * @param model el modelo con las variables de la vista.
     * @param message el texto del mensaje.
     * @param args los parámetros del mensaje.
     */
    public static void warning(Model model, String message, Object... args) {
        addAttribute(model, message, Message.Type.WARNING, args);
    }

    /**
     * Añade un mensaje de caracter temporal entre los atributos de una
     * redirección.
     *
     * @param ra los parámetros de la redirección.
     * @param message el texto del mensaje.
     * @param type el tipo de mensaje.
     * @param args los parámetros del mensaje.
     */
    private static void addAttribute(RedirectAttributes ra, String message, Message.Type type, Object... args) {
        ra.addFlashAttribute(MESSAGE_ATTRIBUTE, new Message(message, type, args));
    }

    /**
     * Añade un mensaje al modelo de una vista.
     *
     * @param model el modelo con los parámetros de la vista.
     * @param message el texto del mensaje.
     * @param type el tipo de mensaje.
     * @param args los parámetros del mensaje.
     */
    private static void addAttribute(Model model, String message, Message.Type type, Object... args) {
        model.addAttribute(MESSAGE_ATTRIBUTE, new Message(message, type, args));
    }
}
