/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;

import javax.servlet.http.HttpServletRequest;

/**
 * <code>HttpUtils</code> contiene utilidades HTTP.
 */
public final class HttpUtils {

    private static final String RESOURCE_REGEXP = "(?i)(.*).css$|(.*).js$|(.*).woff$|(.*).png$|(.*).jpg$|(.*).gif$";
    public static final String REFERER_PARAM_NAME = "_refererPramameterName";
    public static final String REFERER_PARAM = "_referer";
    public static final String REFERER = "referer";
    public static final String REDIRECT = "redirect:";
    public static final String ENCODING = "utf-8";
    public static final String EMPTY = "";
    public static final String Q_MARK = "?";
    public static final String AND = "&";
    public static final String EQ = "=";

    private HttpUtils() {

    }

    /**
     * Compone la URI que permite hacer la redirección HTTP hacia una ruta dada.
     *
     * @param path
     *            la ruta a utilizar.
     * @return la URI compuesta para la redirección.
     */
    public static String redirectTo(String path) {
        return new StringBuilder(REDIRECT).append(path).toString();
    }

    /**
     * Compone la ruta que permite redireccionar una petición a la página donde
     * se originó la solicitud HTTP previa.
     *
     * @param request
     *            la solicitud recibida.
     * @return la URI compuesta para la redirección.
     */
    public static String redirectToReferer(final HttpServletRequest request) {
        final String referer = (String) request.getAttribute(REFERER);
        return new StringBuilder(REDIRECT).append(referer).toString().replace(request.getContextPath(), EMPTY);
    }

    /**
     * Obtiene el parámetro <code>referer</code> incluído en una petición HTTP
     * dada, junto con los posibles parámetros que tenga.
     *
     * @param request
     *            la solicitud HTTP.
     * @return la URI correspondiente al Referer (y los posibles parámetros) de
     *         la solicitud.
     * @throws UnsupportedEncodingException
     *             Si la codificación de los caracteres de la solicitud no es
     *             compatible con la vigente en la aplicación.
     */
    public static String getReferer(final HttpServletRequest request) throws UnsupportedEncodingException {
        final String refererParameter = request.getParameter(REFERER_PARAM);

        if (refererParameter != null) {
            // Utiliza el parámetro en la URL
            return URLDecoder.decode(refererParameter, ENCODING);
        }

        return buildRequestUrl(request);
    }

    /**
     * Codifica una URL dada.
     *
     * @param url
     *            la URL recibida.
     * @return la URL codificada.
     * @throws UnsupportedEncodingException
     *             Si la codificación de la URL no es compatible con la vigente
     *             en la aplicación.
     */
    public static String encode(final String url) throws UnsupportedEncodingException {
        return URLEncoder.encode(url, ENCODING);
    }

    /**
     * Decodifica una URL.
     *
     * @param url
     *            la URL recibida.
     * @return la URL descodificada.
     * @throws UnsupportedEncodingException
     *             Si la codificación de la URL no es compatible con la vigente
     *             en la aplicación.
     */
    public static String decode(final String url) throws UnsupportedEncodingException {
        return URLDecoder.decode(url, ENCODING);
    }

    /**
     * Valida si una URL dada pertenece a un recurso estático (css, js, ...).
     *
     * @param request
     *            la URL recibida.
     * @return <code>true</code> si pertenece a un recurso estático y
     *         <code>false</code> en caso contrario.
     */
    public static boolean isResourceRequest(final HttpServletRequest request) {
        final String uri = request.getRequestURI();

        return uri.matches(RESOURCE_REGEXP);
    }

    /**
     * Construye la URL actual sin información de protocolo ni servidor.
     * 
     * @param request
     *            petición actual.
     * @return URL a la que se accede
     */
    public static String buildRequestUrl(HttpServletRequest request) {
        return buildRequestUrl(request, new String[0]);
    }

    /**
     * Construye la URL actual sin información de protocolo ni servidor.
     * 
     * @param request
     *            petición actual
     * @param params
     *            parámetros adicionales a añadir a la URL
     * @return URL actual con los parámetros adicionales
     */
    public static String buildRequestUrl(HttpServletRequest request, String... params) {
        StringBuilder url = new StringBuilder();

        String requestURI = request.getRequestURI();
        String queryString = request.getQueryString();

        url.append(requestURI);

        boolean hasParamters = false;
        if (queryString != null) {
            url.append(Q_MARK).append(queryString);
            hasParamters = true;
        }

        if (params != null) {
            for (String param : params) {
                url.append(hasParamters ? AND : Q_MARK);
                url.append(param);
                hasParamters = true;
            }
        }

        return url.toString();
    }
}
