/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.util;

import es.xunta.amtega.transversal.utils.BaseUtils;
import java.io.IOException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.tika.Tika;
import org.springframework.web.multipart.MultipartFile;

/**
 * <code>MultipartFileUtils</code> contiene utilidades para trabajar con subida de ficheros
 */
public final class MultipartFileUtils {

    private static final Logger LOGGER = LogManager.getLogger(MultipartFileUtils.class);
    
    /**
     * Etiqueta utilizada en el fichero de configuración de la aplicación para
     * especificar los tipos de ficheros que se permite asociar a los
     * documentos.
     */
    private static final String ALLOWED_FILE_TYPES = "multipart.allowed.content.type";
    private static final String DEFAULT_CONTENT_TYPE = "application/octet-stream";
    
    private MultipartFileUtils() {
        
    }

    /**
     * Obtiene, del fichero de configuración, los formatos permitidos para los
     * ficheros asociados a los documentos.
     *
     * @return la relación de tipos mime permitidos
     */
    public static String getValidContentTypes() {
        return BaseUtils.Configuration.getProperty(ALLOWED_FILE_TYPES);
    }

    /**
     * Detecta el tipo Mime real para un objeto <code>MultipartFile</code>
     *
     * @param multipart fichero de usuario.
     * @return el tipo mime detectado
     */
    public static String getMimeType(MultipartFile multipart) {

        try {
            Tika tikaMultipartFile = new Tika();
            String tipomime = tikaMultipartFile.detect(multipart.getInputStream());
            
            LOGGER.info("Detectado fichero asociado con contenido de tipo: {}", tipomime);
            
            return tipomime;
        } catch (IOException ex) {
            LOGGER.error("Error al intentar detectar el tipo de fichero asociado. En adelante se le considera por defecto del tipo: {}", DEFAULT_CONTENT_TYPE, ex);
            
            //Valor por defecto en caso de algun error en la deteccion.
            return DEFAULT_CONTENT_TYPE;
        }
    }
    
    /**
     * Comprueba si el tipo del fichero seleccionado para asociar a un documento
     * está permitido.
     *
     * Los tipos Mime permitidos se definen en el fichero de configuración de la
     * aplicación. En el caso de que no conste ninguno se permite cualquier tipo
     * de fichero.
     *
     * @param multipart fichero de usuario.
     * @return true si se seleccionó un fichero con un tipo válido.
     */
    public static boolean isAllowedContentType(MultipartFile multipart) {
        LOGGER.info("Validando tipo de contenido del fichero asociado: {}", multipart.getOriginalFilename() + " (presuntamente " + multipart.getContentType() + ")");

        final String validContentTypes = MultipartFileUtils.getValidContentTypes();
        if( validContentTypes==null ) {
            // Si no se define la propiedad no se filtra
            return true;
        }
        
        final String mimeType = MultipartFileUtils.getMimeType(multipart).toLowerCase();
        
        return validContentTypes.contains(mimeType);
    }

}
