/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.filter;

import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.EMPTY;
import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.EQ;
import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.Q_MARK;
import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.REFERER;
import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.REFERER_PARAM;
import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.REFERER_PARAM_NAME;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;
import static org.mockito.Mockito.mock;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Test;
import org.springframework.mock.web.MockFilterChain;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import es.xunta.amtega.transversal.archam.core.web.filter.RefererManagerFilter.GenericResponseWrapper;
import es.xunta.amtega.transversal.archam.core.web.util.HttpUtils;

/**
 * Tests unitarios para el filtro RefererManagerFilter 
 */
public class RefererManagerFilterTest {
    
    private static final Logger LOGGER = LogManager.getLogger(RefererManagerFilterTest.class);
    private static final String TEST_REFERER_URL = "/referer-url";
    private static final String LOCATION_URL = "/location";
    private static final String LOCATION_HEADER = "Location";
    private static final String REDIRECT_ERROR = "La redirección no es correcta";

    @Test
    public void testInit() throws ServletException {
        //given: Declaramos el filtro y la configuración
        RefererManagerFilter filter = new RefererManagerFilter();
        FilterConfig config = mock(FilterConfig.class);
        //when: Llamamos al método init
        filter.init(config);
        //then: nada que validar
    }
    
    @Test
    public void testDestroy() {
        //given: Declaramos el filtro
        RefererManagerFilter filter = new RefererManagerFilter();
        //when: Llamamos al método destroy
        filter.destroy();
        //then: nada que validar
    }
    
    @Test
    public void doFilterWithRefererShouldReturnSameReferer() {
        //given: inicializamos las variables y añadimos el referer a la request
        RefererManagerFilter filter = new RefererManagerFilter();
        HttpServletRequest request = new MockHttpServletRequest();
        String referer = TEST_REFERER_URL;
        request.setAttribute(REFERER, referer);
        //when: realizamos el filtrado
        this.assertDoFilter(filter, request);
        //then: validamos que esl referer no ha cambiado
        assertThat("El parámetro referer ha sido modificado", (String)request.getAttribute(REFERER), is(referer));
    }
    
    @Test
    public void doFilterWithRefererNullShouldReturnEmptyReferer() {
        //given: inicializamos las variables y añadimos el referer a la request
        RefererManagerFilter filter = new RefererManagerFilter();
        HttpServletRequest request = new MockHttpServletRequest();
        String referer = null;
        request.setAttribute(REFERER, referer);
        //when: realizamos el filtrado
        this.assertDoFilter(filter, request);
        //then: validamos que esl referer no ha cambiado
        assertThat("El parámetro referer ha sido modificado", (String)request.getAttribute(REFERER), is(EMPTY));
    }
    
    @Test
    public void doFilterWithoutRefererShouldReturnRefererParam() throws UnsupportedEncodingException {
        //given: inicializamos las variables y añadimos el referer a la request
        RefererManagerFilter filter = new RefererManagerFilter();
        MockHttpServletRequest request = new MockHttpServletRequest();
        String referer = TEST_REFERER_URL;
        request.addParameter(HttpUtils.REFERER_PARAM, referer);
        //when: realizamos el filtrado
        this.assertDoFilter(filter, request);
        //then: validamos que esl referer no ha cambiado
        assertThat("El referer no ha sido añadido", request.getAttribute(REFERER), is(notNullValue()));
        assertThat("El referer no ha sido añadido", (String)request.getAttribute(REFERER), is(referer));
        assertThat("El nombre del referer no ha sido añadido", request.getAttribute(REFERER_PARAM_NAME), is(notNullValue()));
        assertThat("El nombre del referer no ha sido añadido", (String)request.getAttribute(REFERER_PARAM_NAME), is(REFERER_PARAM));
        assertThat("El referer_param no tiene el valor esperado", (String)request.getAttribute(REFERER_PARAM), is(HttpUtils.encode(referer)));
    }
    
    @Test
    public void newGenericResponseWrapper() {
        //given: declaramos un filter y un referer
        HttpServletResponse response = new MockHttpServletResponse();
        //when: usamos el construtor con response y referer
        GenericResponseWrapper wrapper = this.getGenericResponseWrapper(response);
        //then: validamos el response
        assertThat("El response no tiene el valor esperado", wrapper.getResponse(), is((ServletResponse)response));
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void sendRedirectWithNullLocation() throws IOException {
        //given: declaramos un filter y un referer
        GenericResponseWrapper wrapper = this.getGenericResponseWrapper();
        //when: llamamos a sendRedirect con location null
        wrapper.sendRedirect(null);
        //then: validar que se ha producido la excepción
        fail("No se ha producido la excepción IllegalArgumentException");
    }
    
    @Test
    public void sendRedirectWithLocationContainsRefererParam() throws IOException {
        //given: declaramos un filter y un referer
        GenericResponseWrapper wrapper = this.getGenericResponseWrapper();
        String location = LOCATION_URL + REFERER_PARAM.concat(EQ);
        //when: llamamos a sendRedirect con location
        wrapper.sendRedirect(location);
        //then: valida que se ha realizado la redirección
        assertThat(REDIRECT_ERROR, wrapper.getHeader(LOCATION_HEADER), is(location));
    }
    
    @Test
    public void sendRedirectWithLocationContainsQMark() throws IOException {
        //given: declaramos un filter y un referer
        GenericResponseWrapper wrapper = this.getGenericResponseWrapper();
        String location = LOCATION_URL + Q_MARK;
        //when: llamamos a sendRedirect con location
        wrapper.sendRedirect(location);
        //then: valida que se ha realizado la redirección
        String expected = location.concat("&_referer=").concat(this.getLocationReferer());
        assertThat(REDIRECT_ERROR, wrapper.getHeader(LOCATION_HEADER), is(expected));
    }
    
    @Test
    public void sendRedirectWithLocationNotContainsQMark() throws IOException {
        //given: declaramos un filter y un referer
        GenericResponseWrapper wrapper = this.getGenericResponseWrapper();
        String location = LOCATION_URL;
        //when: llamamos a sendRedirect con location
        wrapper.sendRedirect(location);
        //then: valida que se ha realizado la redirección
        String expected = location.replace("?", "").concat("?_referer=").concat(this.getLocationReferer());
        assertThat(REDIRECT_ERROR, wrapper.getHeader(LOCATION_HEADER), is(expected));
    }   
    
    
    
    private void assertDoFilter(RefererManagerFilter filter, HttpServletRequest request) {
        MockFilterChain mockChain = new MockFilterChain();
        HttpServletResponse response = mock(HttpServletResponse.class);
        try {
            filter.doFilter(request, response, mockChain);
        } catch (IOException | ServletException e) {
            LOGGER.error(e);
            fail("Se produjo un error: "+e.getLocalizedMessage());
        }
    }
    
    private GenericResponseWrapper getGenericResponseWrapper() {
        return this.getGenericResponseWrapper(new MockHttpServletResponse());
    }
    
    private GenericResponseWrapper getGenericResponseWrapper(HttpServletResponse response) {
        RefererManagerFilter filter = new RefererManagerFilter();
        return filter.new GenericResponseWrapper(response, TEST_REFERER_URL);
    }
    
    private String getLocationReferer() throws UnsupportedEncodingException {
        return URLEncoder.encode(TEST_REFERER_URL, "UTF-8");
    }

}
