/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.mvc;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpSession;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;

import es.xunta.amtega.transversal.archam.core.web.mvc.ConversationalSessionAttributeStore;

/**
 * Tests unitarios de la clase ConversationalSessionAttributeStore 
 */
public class ConversationalSessionAttributeStoreTest {
    
    private static final String TEST_ATTR_NAME = "attributeName";
    private static final String TEST_ATTR_VAL = "attributeValue";
    private static final String NO_EXCPETION_MESSAGE = "No se ha lanzado la excepción esperada";
    
    private ConversationalSessionAttributeStore store;

    @Before
    public void init(){
        RequestMappingHandlerAdapter requestMappingHandlerAdapter = new RequestMappingHandlerAdapter();
        store = new ConversationalSessionAttributeStore();
        store.postProcessAfterInitialization(requestMappingHandlerAdapter, "");
    }

    @Test(expected = IllegalArgumentException.class)
    public void cleanupAttributeWithNullRequestShouldThrowIllegalArgumentException() {
        store.cleanupAttribute(null, TEST_ATTR_NAME);
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void cleanupAttributeWithNullAttributeNameShouldThrowIllegalArgumentException() {
        //given: declaramos un ConversationalSessionAttributeStore
        WebRequest webRequest = getWebRequest();
        //when: limpiamos el atributo
        store.cleanupAttribute(webRequest, null);
        //then: validamos si no se ha lanzado la excepción
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test
    public void cleanupAttributeWithNullSessionMap() {
        //given: declaramos un ConversationalSessionAttributeStore
        WebRequest webRequest = getWebRequest();
        //when: limpiamos el atributo
        store.cleanupAttribute(webRequest, TEST_ATTR_NAME);
        //then: nada que validar
    }
    
    @Test
    public void cleanupAttributeWithSessionMapAndAttribute() {
        //given: declaramos un ConversationalSessionAttributeStore con un atributo
        WebRequest webRequest = this.getWebRequestWithCidAndAttribute(TEST_ATTR_NAME, TEST_ATTR_VAL);
        //when: limpiamos el atributo
        store.cleanupAttribute(webRequest, TEST_ATTR_NAME);
        //then: validamos que el atributo se ha eliminado
        Assert.assertNull("El atributo attributeName no se ha elimiando correctamente", store.retrieveAttribute(webRequest, TEST_ATTR_NAME));
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void storeWithNullRequestShouldThrowIllegalArgumentException() {
        //when: almacenamos el atributo
        store.storeAttribute(null, TEST_ATTR_NAME, TEST_ATTR_VAL);
        //then: validamos si no se ha lanzado la excepción
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void storeWithNullAttributeNameShouldThrowIllegalArgumentException() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequest();
        //when: almacenamos el atributo
        store.storeAttribute(webRequest, null, TEST_ATTR_VAL);
        //then: validamos si no se ha lanzado la excepción
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void storeWithNullAttributeValueShouldThrowIllegalArgumentException() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequest();
        //when: almacenamos el atributo
        store.storeAttribute(webRequest, TEST_ATTR_NAME, null);
        //then: validamos si no se ha lanzado la excepción
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test
    public void storeWithNullCidShouldStoreWithNewCid() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequest();
        //when: almacenamos el atributo
        store.storeAttribute(webRequest, TEST_ATTR_NAME, TEST_ATTR_VAL);
        //then: validamos que se ha añadido el atributo y que el valor es correcto
        this.assertStore(webRequest);
    }
    
    @Test
    public void storeWithEmptyCidShouldStoreWithNewCid() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequestWithCid();
        //when: almacenamos el atributo
        store.storeAttribute(webRequest, TEST_ATTR_NAME, TEST_ATTR_VAL);
        //then: validamos que se ha añadido el atributo y que el valor es correcto
        this.assertStore(webRequest);
    }
    
    @Test
    public void storeWithCidShouldStoreWithSameCid() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequestWithCid();
        //when: almacenamos el atributo
        store.storeAttribute(webRequest, TEST_ATTR_NAME, TEST_ATTR_VAL);
        //then: validamos que se ha añadido el atributo y que el valor es correcto
        this.assertStore(webRequest);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void retrieveAttributeWithNullRequestShouldThrowIllegalArgumentException() {
        //when: recuperamos el atributo
        store.retrieveAttribute(null, TEST_ATTR_NAME);
        //then: validamos si no se ha lanzado la excepción
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void retrieveAttributeWithNullAttributeNameShouldThrowIllegalArgumentException() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequest();
        //when: recuperamos el atributo
        store.retrieveAttribute(webRequest, null);
        //then: validamos si no se ha lanzado la excepción
        Assert.fail(NO_EXCPETION_MESSAGE);
    }
    
    @Test
    public void retrieveAttributeWithNullConversationIdShouldReturnNull() {
        //given: inicializamos el store
        WebRequest webRequest = getWebRequest();
        //when: recuperamos el atributo
        Object result = store.retrieveAttribute(webRequest, TEST_ATTR_NAME);
        //then: validamos el valor del resultado
        Assert.assertNull("El attributo no es nulo", result);
    }
    
    @Test
    public void retrieveAttributeWithConversationIdShouldReturnAttribute() {
        //given: inicializamos el store
        WebRequest webRequest = this.getWebRequestWithCidAndAttribute(TEST_ATTR_NAME, TEST_ATTR_VAL);
        //when: recuperamos el atributo
        Object result = store.retrieveAttribute(webRequest, TEST_ATTR_NAME);
        //then: validamos el valor del resultado
        Assert.assertNotNull("El attributo es nulo", result);
    }
    
    @Test
    public void setKeepAliveConversationsShouldReturnSameKeepAliveConversations() {
        // given:
        int conversations = 20;
        // when:indicamos el valor
        store.setKeepAliveConversations(conversations);
        //then: recuperamos el mismo valor
        Assert.assertEquals("El valor de KeepAliveConversations no es el mismo", conversations, store.getKeepAliveConversations());
    }
    
    
    
    private WebRequest getWebRequest() {
        HttpServletRequest request = new MockHttpServletRequest();
        return new ServletWebRequest(request);
    }
    
    private WebRequest getWebRequestWithCid() {
        HttpServletRequest request = new MockHttpServletRequest();
        request.setAttribute(ConversationalSessionAttributeStore.CID_FIELD, "cid");
        return new ServletWebRequest(request);
    }
    
    private WebRequest getWebRequestWithCidAndAttribute(String attributeName, String attributeValue) {
        MockHttpServletRequest request = new MockHttpServletRequest();
        request.addParameter(ConversationalSessionAttributeStore.CID_FIELD, "cid");
        WebRequest webRequest = new ServletWebRequest(request);
        
        Map<String, Map<String, Object>> sessionMap = new LinkedHashMap<String, Map<String, Object>>();
        String cid = "cid";
        sessionMap.put(cid, new HashMap<String, Object>());
        sessionMap.get(cid).put(attributeName, attributeValue);
        request.setAttribute(ConversationalSessionAttributeStore.CID_FIELD, cid);
        MockHttpSession session = new MockHttpSession();
        session.setAttribute(ConversationalSessionAttributeStore.SESSION_MAP, sessionMap);
        request.setSession(session);
        
        return webRequest;
    }
    
    private void assertStore(WebRequest webRequest) {
        Assert.assertNotNull("No se ha añadido el atributo al store", store.retrieveAttribute(webRequest, TEST_ATTR_NAME));
        Assert.assertEquals("El valor del atributo no es el esperado", TEST_ATTR_VAL, store.retrieveAttribute(webRequest, TEST_ATTR_NAME));
    }
    
}
