/* amtega-archam-core-web: Amtega Archetype Core Web
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core-web".
*
* "amtega-archam-core-web" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "APP" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.web.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;

import javax.servlet.http.HttpServletRequest;

import org.junit.Assert;
import org.junit.Test;
import org.springframework.mock.web.MockHttpServletRequest;

/**
 * Tests unitarios para la clase de utilidades HttpUtils
 */
public class HttpUtilsTest {
    
    private static final String DOC_PATH = "/documentos/";

    @Test
    public void redirectToShouldReturnRedirectToPath() {
        //given: declaramos una ruta
        String path = DOC_PATH;
        //when: realizamos la llamada con la ruta
        String result = HttpUtils.redirectTo(path);
        //then: obtenemos la redirección a la ruta
        Assert.assertEquals(result, HttpUtils.REDIRECT + path);
    }
    
    @Test
    public void redirectToRefererShouldReturnRedirectToReferer() {
        //given: declaramos la request y añadimos un referer
        HttpServletRequest request = new MockHttpServletRequest();
        String referer = DOC_PATH;
        request.setAttribute(HttpUtils.REFERER, referer);
        //when: realiazmos la llamada con la request
        String result = HttpUtils.redirectToReferer(request);
        //then: validamos la url de redireccion
        Assert.assertEquals(result, HttpUtils.REDIRECT + referer);
    }
    
    @Test
    public void getRefererWithRefererShouldReturnDecodeRefererParameter() throws UnsupportedEncodingException {
        //given: declaramos la request y añadimos un referer
        MockHttpServletRequest request = new MockHttpServletRequest();
        String referer = DOC_PATH;
        request.addParameter(HttpUtils.REFERER_PARAM, referer);
        //when: realiazmos la llamada con la request
        String result = HttpUtils.getReferer(request);
        //then: validamos que devuelve en referer
        Assert.assertEquals(result, URLDecoder.decode(referer, HttpUtils.ENCODING));
    }
    
    @Test
    public void getRefererWithoutRefererShouldReturnRequestUri() throws UnsupportedEncodingException {
        //given: declaramos la request y añadimos la requestUri
        MockHttpServletRequest request = new MockHttpServletRequest();
        String requestUri = DOC_PATH;
        request.setRequestURI(requestUri);
        //when: realiazmos la llamada con la request
        String result = HttpUtils.getReferer(request);
        //then: validamos que devuelve en referer
        Assert.assertEquals(result, requestUri);
    }
    
    @Test
    public void encodeShoudReturnEncodeUrl() throws UnsupportedEncodingException {
        //given: declaramos la url
        String url = DOC_PATH;
        //when: realiazmos la llamada al método
        String result = HttpUtils.encode(url);
        //then: validamos que devuelve la url
        Assert.assertEquals(result, URLEncoder.encode(url, HttpUtils.ENCODING));
    }
    
    @Test
    public void decodeShoudReturnDecodeUrl() throws UnsupportedEncodingException {
        //given: declaramos la url
        String url = DOC_PATH;
        //when: realiazmos la llamada al método
        String result = HttpUtils.decode(url);
        //then: validamos que devuelve la url
        Assert.assertEquals(result, URLDecoder.decode(url, HttpUtils.ENCODING));
    }
    
    @Test
    public void isResourceRequestShouldReturnTrue(){
        //given: declaramos la request
        MockHttpServletRequest request = new MockHttpServletRequest();
        request.setRequestURI("documentos.js");
        //when: realiazmos la llamada al método
        Boolean result = HttpUtils.isResourceRequest(request);
        //then: validamos el resultado
        Assert.assertTrue(result);
    }
    
    @Test
    public void isResourceRequestShouldReturnFalse(){
        //given: declaramos la request
        MockHttpServletRequest request = new MockHttpServletRequest();
        request.setRequestURI(DOC_PATH);
        //when: realiazmos la llamada al método
        Boolean result = HttpUtils.isResourceRequest(request);
        //then: validamos el resultado
        Assert.assertFalse(result);
    }
    
    @Test
    public void buildRequestUrlShouldReturnRequestUri() {
        //given: declaramos la request
        MockHttpServletRequest request = new MockHttpServletRequest();
        request.setRequestURI(DOC_PATH);
        //when: realiazmos la llamada al método
        String result = HttpUtils.buildRequestUrl(request);
        //then: validamos el resultado
        Assert.assertEquals(result, request.getRequestURI());
    }
    
    @Test
    public void buildRequestUrlWithParamsShouldReturnRequestUri() {
        //given: declaramos la request
        MockHttpServletRequest request = new MockHttpServletRequest();
        request.setRequestURI(DOC_PATH);
        String param1 = "param1";
        String param2 = "param2";
        //when: realiazmos la llamada al método
        String result = HttpUtils.buildRequestUrl(request, param1, param2);
        //then: validamos el resultado
        Assert.assertEquals(result, request.getRequestURI() + HttpUtils.Q_MARK + param1 + HttpUtils.AND + param2);
    }
    
    @Test
    public void buildRequestUrlWithParamsAndQueryShouldReturnRequestUri() {
        //given: declaramos la request
        MockHttpServletRequest request = new MockHttpServletRequest();
        String query = "foo=foo&foo2=foo2";
        request.setRequestURI("/documentos");
        request.setQueryString(query);
        String param1 = "param1";
        String param2 = "param2";
        //when: realiazmos la llamada al método
        String result = HttpUtils.buildRequestUrl(request, param1, param2);
        //then: validamos el resultado
        Assert.assertEquals(result, request.getRequestURI() + HttpUtils.Q_MARK + query + HttpUtils.AND + param1 + HttpUtils.AND + param2);
    }
}
