/* amtega-archam-core: Amtega Archetype Core
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core”.
*
* "amtega-archam-core" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-archam-core" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.aop;

import java.sql.SQLException;
import java.util.Arrays;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;

import es.xunta.amtega.transversal.archam.core.exception.BaseDatosException;
import es.xunta.amtega.transversal.archam.core.exception.InternalErrorException;
import es.xunta.amtega.transversal.logging.AuditableException;

/**
 * <code>LoggingAspect</code> es la clase que se encarga de la gestión de los
 * logs AOP de las capas de Servicio y Repositorio.
 */
public class LoggingAspect {

    private static final String ILLEGAL_ARGUMENT = "Illegal argument: {} in {}.{}()";
    private static final String EXIT_METHOD = "Exit: {}.{}() with result = {}";
    private static final String ENTER_METHOD = "Enter: {}.{}() with argument[s] = {}";
    private static final String LOG_ERROR_PATTERN = "Error: {}.{}()";
    private static final Logger LOGGER = LogManager.getLogger(LoggingAspect.class);

    /**
     * Registra una traza correspondiente a un error en la capa de acceso a
     * datos de la aplicacion.
     *
     * @param joinPoint
     *            la representación del punto de unión
     *            ({@link ProceedingJoinPoint}) que se debe enlazar a
     *            continuación.
     * @param e
     *            la excepción correspondiente al error.
     */
    public void logAfterExceptionThrowing(JoinPoint joinPoint, Throwable e) {
        final String typeName = joinPoint.getSignature().getDeclaringTypeName();
        final String methodName = joinPoint.getSignature().getName();

        final AuditableException auditableException = translate(e);

        error(LOG_ERROR_PATTERN, typeName, methodName, auditableException);
    }

    /**
     * Registra trazas de log de entrada/salida/excepción para la capa de
     * servicio.
     *
     * @param joinPoint
     *            la representación del punto de unión
     *            ({@link ProceedingJoinPoint}) que se debe enlazar a
     *            continuación.
     * @return el objeto que sería devuelto originalmente.
     * @throws Throwable
     *             la {@code Exception} lanzada por este metodo.
     */
    public Object logAround(ProceedingJoinPoint joinPoint) throws Throwable {

        debug(ENTER_METHOD, joinPoint.getSignature().getDeclaringTypeName(), joinPoint.getSignature().getName(),
                Arrays.toString(joinPoint.getArgs()));
        try {
            Object result = joinPoint.proceed();
            debug(EXIT_METHOD, joinPoint.getSignature().getDeclaringTypeName(), joinPoint.getSignature().getName(),
                    result);
            return result;
        } catch (IllegalArgumentException e) {
            error(ILLEGAL_ARGUMENT, Arrays.toString(joinPoint.getArgs()),
                    joinPoint.getSignature().getDeclaringTypeName(), joinPoint.getSignature().getName(),
                    new InternalErrorException(e));
            throw e;
        }
    }

    /**
     * Convierte una excepción genérica a excepción de tipo AuditableException
     * 
     * @param e
     *            excepción original
     * @return wrapper de excepción
     */
    private AuditableException translate(Throwable e) {
        if (AuditableException.class.isAssignableFrom(e.getClass())) {
            return (AuditableException) e;
        }

        // Creamos AuditableException (BaseDatosException o
        // InternalErrorException)
        SQLException sqlException = null;
        if (e.getCause() instanceof SQLException) {
            sqlException = (SQLException) e.getCause();
        }

        if (e.getCause() != null && e.getCause().getCause() instanceof SQLException) {
            sqlException = (SQLException) e.getCause().getCause();
        }

        String msg = e.getMessage();
        boolean isDBException = sqlException != null;
        if (isDBException) {
            return new BaseDatosException(sqlException.getErrorCode(), msg, e);
        }

        return new InternalErrorException(msg, e);
    }

    private void debug(String msg, Object... objects) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug(msg, objects);
        }
    }

    private void error(String msg, Object... objects) {
        LOGGER.error(msg, objects);
    }

}
