/* amtega-archam-core: Amtega Archetype Core
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core”.
*
* "amtega-archam-core" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-archam-core" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.context;

import java.net.URL;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.context.ApplicationContextInitializer;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.env.PropertiesPropertySource;

import es.xunta.amtega.transversal.utils.BaseUtils.Configuration;

/**
 * <code>ExternalConfigurationConfigurer</code> es la clase que implementa la
 * inclusión de la configuración que se encuentra definida en recursos externos
 * a la aplicación.
 */
public class ExternalConfigurationInitializer implements ApplicationContextInitializer<ConfigurableApplicationContext> {

    private static final Logger LOGGER = LogManager.getLogger(ExternalConfigurationInitializer.class);
    private static final String[] ARCHETYPE_PROPERTIES_FILES = {"/META-INF/archetype.version", "/archetype.version"};
    private static final String ARCHETYPE_PROP_ARTIFACT = "archetype.artifactId";
    private static final String ARCHETYPE_PROP_VERSION = "archetype.version";
    private static final String APP_CODE_PROP = "application.code";

    /**
     * Inicializa un contexto de la aplicación dado.
     * <p>
     * Para la carga de los recursos externos hace uso del <i>Componente
     * Transversal de Configuración de la AMTEGA</i>.
     *
     * @param applicationContext el contexto de la aplicación a inicializar.
     */
    @Override
    public void initialize(ConfigurableApplicationContext applicationContext) {
        // Añade las propiedades externalizadas en la impl Environment de Spring
        Properties archProperties = createArchetypeProperties();
        applicationContext.getEnvironment().getPropertySources().addFirst(new PropertiesPropertySource("archetype", archProperties));
        applicationContext.getEnvironment().getPropertySources().addFirst(new PropertiesPropertySource("application", createApplicationProperties()));
        applicationContext.getEnvironment().getPropertySources().addFirst(new PropertiesPropertySource("external", Configuration.getProperties()));
        // Log de arranque
        // Al acceder directamente al objeto 'archProperties' evitamos que se sobreescriba sin querer esta info desde la config de la app
        String appCode = Configuration.getApplicationCode();
        appCode = appCode==null ? null : appCode.toUpperCase();
        String archetypeArtifact = archProperties.getProperty(ARCHETYPE_PROP_ARTIFACT);
        String archetypeVersion = archProperties.getProperty(ARCHETYPE_PROP_VERSION);
        if(archetypeVersion!=null) {
            LOGGER.info("Iniciando aplicación {} (arquetipo: '{}', versión: '{}')", appCode, archetypeArtifact, archetypeVersion);
        } else {
            LOGGER.warn("Iniciando aplicación {} (no se encuentra información de uso de arquetipo)", appCode);
        }
    }
    
    /**
     * Propiedades derivadas del arquetipo
     * @return properties
     */
    private Properties createArchetypeProperties() {
        // Buscamos el properties, ya que en versiones antiguas (<=1.3.0) puede estar en varias rutas
        String pathArchetypeFile = null;
        for(String path : ARCHETYPE_PROPERTIES_FILES) {
            URL archetypeFile = ExternalConfigurationInitializer.class.getResource(path);
            if(archetypeFile!=null) {
                pathArchetypeFile = path;
                break;
            }
        }
        // No existe el archivo, devolvemos Properties vacío
        if(pathArchetypeFile==null) {
            return new Properties();
        }
        // Cargamos la config
        String appCode = Configuration.getApplicationCode();
        es.xunta.amtega.transversal.configuration.Configuration archConfig = es.xunta.amtega.transversal.configuration.Configuration.build(appCode, "classpath:"+pathArchetypeFile);
        return archConfig.getProperties();
    }
    
    /**
     * Propiedades de uso general
     * @return properties
     */
    private Properties createApplicationProperties() {
        final Properties props = new Properties();
        props.setProperty(APP_CODE_PROP, Configuration.getApplicationCode());
        return props;
    }

}
