/* amtega-archam-core: Amtega Archetype Core
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core”.
*
* "amtega-archam-core" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-archam-core" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.exception;

import static es.xunta.amtega.transversal.logging.StringPool.EMPTY;

import es.xunta.amtega.transversal.logging.AuditableException;
import es.xunta.amtega.transversal.logging.Criticidad;
import es.xunta.amtega.transversal.logging.TipoError;

/**
 * <code>GenericAuditableRuntimeException</code> Clase genérica para el registro
 * de excepciones en tiempo de ejecución con datos de categorización del error.
 *
 */
public class InternalErrorRuntimeException extends RuntimeException implements AuditableException {

    private static final long serialVersionUID = 1L;
    private final String sistema;
    private final TipoError tipoError;
    private final Criticidad criticidad;
    private final String codigoError;
    private final String observaciones;

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error, mensaje,
     * observaciones y causa.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param codigoError
     *            <code>String</code> código del error
     * @param msg
     *            <code>String</code> mensaje del error
     * @param observaciones
     *            <code>String</code> observaciones del error
     * @param cause
     *            <code>Throwable</code> la causa de la excepción
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad,
            String codigoError, String msg, String observaciones, Throwable cause) {
        super(msg, cause);
        this.sistema = sistema;
        this.tipoError = tipoError;
        this.criticidad = criticidad;
        this.codigoError = codigoError;
        this.observaciones = observaciones;
    }

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error y mensaje.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param codigoError
     *            <code>String</code> código del error
     * @param msg
     *            <code>String</code> mensaje del error
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad,
            String codigoError, String msg) {
        this(sistema, tipoError, criticidad, codigoError, msg, EMPTY, null);
    }

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error y mensaje.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param codigoError
     *            <code>int</code> código del error
     * @param msg
     *            <code>String</code> mensaje del error
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad, int codigoError,
            String msg) {
        this(sistema, tipoError, criticidad, String.valueOf(codigoError), msg, EMPTY, null);
    }

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error, mensaje y causa.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param codigoError
     *            <code>String</code> código del error
     * @param msg
     *            <code>String</code> mensaje del error
     * @param cause
     *            <code>Throwable</code> causa del error
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad,
            String codigoError, String msg, Throwable cause) {
        this(sistema, tipoError, criticidad, codigoError, msg, EMPTY, cause);
    }

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error, mensaje y causa.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param codigoError
     *            <code>int</code> código del error
     * @param msg
     *            <code>String</code> mensaje del error
     * @param cause
     *            <code>Throwable</code> causa del error
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad, int codigoError,
            String msg, Throwable cause) {
        this(sistema, tipoError, criticidad, String.valueOf(codigoError), msg, EMPTY, cause);
    }

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error, mensaje y causa.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param msg
     *            <code>String</code> mensaje del error
     * @param causa
     *            <code>Throwable</code> causa del error
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad, String msg,
            Throwable causa) {
        this(sistema, tipoError, criticidad, EMPTY, msg, EMPTY, causa);
    }

    /**
     * Constructor de los mensajes de error en tiempo de ejecución a partir del
     * sistema, tipo de error, criticidad, código de error, mensaje y causa.
     *
     * @param sistema
     *            <code>String</code> sistema de la aplicación afectado
     * @param tipoError
     *            <code>TipoError</code> tipo de error
     * @param criticidad
     *            <code>Criticidad</code> criticidad del error
     * @param msg
     *            <code>String</code> mensaje del error
     */
    public InternalErrorRuntimeException(String sistema, TipoError tipoError, Criticidad criticidad, String msg) {
        this(sistema, tipoError, criticidad, EMPTY, msg, EMPTY, null);
    }

    /**
     * Obtiene el sistema del error.
     */
    @Override
    public String getSistema() {
        return sistema;
    }

    /**
     * Obtiene el tipo de error.
     */
    @Override
    public TipoError getTipoError() {
        return tipoError;
    }

    /**
     * Obtiene la criticidad del error.
     */
    @Override
    public Criticidad getCriticidad() {
        return criticidad;
    }

    /**
     * Obtiene el código del error.
     */
    @Override
    public String getCodigoError() {
        return codigoError;
    }

    /**
     * Obtiene las observaciones del error.
     */
    @Override
    public String getObservaciones() {
        return observaciones;
    }

    /**
     * Obtiene el mensaje del error.
     */
    @Override
    public String getMensaje() {
        return getMessage();
    }

}
