/* amtega-archam-core: Amtega Archetype Core
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core”.
*
* "amtega-archam-core" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-archam-core" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.archam.core.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Calendar;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.archam.core.io.exception.FileStorageException;
import es.xunta.amtega.transversal.utils.BaseUtils;

/**
 * <code>StorageManagerImpl</code> es el componente que implementa el servicio
 * de almacenamiento de ficheros.
 */
class StorageManagerImpl implements StorageManager {
    private static final Logger LOGGER = LogManager.getLogger(StorageManagerImpl.class);
    private static final String SLASH = "/";
    private static final String ERROR_SAVING_FILE = "Error guardando el fichero";
    private static final String ERROR_SAVING_PATH = "Error creando la ruta del fichero";
    private static final String BASE_PATH = "path.documentos";

    @Override
    public String store(InputStream in) {
        assertNotNull(in);

        final String rootPath = getRootPath();
        final String relativePath = createRelativePath();
        final File dir = new File(rootPath + relativePath);
        OutputStream out = null;
        if (!dir.exists()) {
            try {
                dir.mkdirs();
            } catch (SecurityException e) {
                throw new FileStorageException(ERROR_SAVING_PATH, e);
            }
        }

        // Avoid file name caracter errors
        final String fileName = String.valueOf(System.currentTimeMillis());

        try {
            String path = dir.getAbsolutePath() + SLASH + fileName;
            out = new FileOutputStream(path);
            IOUtils.copy(in, out);

            LOGGER.info("Se ha creado el fichero: {}", path);
        } catch (Exception e) {
            throw new FileStorageException(ERROR_SAVING_FILE, e);
        } finally {
            IOUtils.closeQuietly(out);
            IOUtils.closeQuietly(in);
        }

        return relativePath + fileName;
    }

    @Override
    public String store(File file) {
        assertNotNull(file);

        try {
            return store(new FileInputStream(file));
        } catch (FileNotFoundException e) {
            throw new FileStorageException(ERROR_SAVING_FILE, e);
        }
    }

    @Override
    public File load(String path) {
        return new File(getRootPath() + path);
    }

    @Override
    public void delete(String path) {
        if( path!=null ){
            LOGGER.info("Eliminando fichero: {}", path);
            delete(load(path));
        }
    }

    @Override
    public void delete(File file) {
        LOGGER.info("Eliminando fichero: {}", file);
        FileUtils.deleteQuietly(file);
    }

    private String createRelativePath() {
        final String basePath = getBasePath();
        Calendar cal = Calendar.getInstance();

        StringBuilder builder = new StringBuilder();
        if (basePath != null && !basePath.isEmpty()) {
            builder.append(SLASH);
            builder.append(basePath);
        }
        builder.append(SLASH);
        builder.append(cal.get(Calendar.YEAR));
        builder.append(SLASH);
        builder.append(cal.get(Calendar.MONTH) + 1);
        builder.append(SLASH);
        builder.append(cal.get(Calendar.DAY_OF_MONTH));
        builder.append(SLASH);

        return builder.toString();
    }

    private String getRootPath() {
        return BaseUtils.Configuration.getDataPath();
    }

    private String getBasePath() {
        return BaseUtils.Configuration.getProperty(BASE_PATH);
    }

    /**
     * Comprueba que el objeto no es null
     * 
     * @param param
     *            objeto a validar
     */
    private void assertNotNull(Object param) {
        if (param == null) {
            throw new FileStorageException("El fichero o stream no puede ser null");
        }
    }

}
