/* amtega-archam-core: Amtega Archetype Core
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core”.
*
* "amtega-archam-core" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-archam-core" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.archam.core.aop;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.sql.SQLException;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.reflect.MethodSignature;
import org.junit.Before;
import org.junit.Test;
import org.springframework.dao.DataIntegrityViolationException;

import es.xunta.amtega.transversal.archam.core.exception.SistemaAutenticacionException;

/**
 * Tests unitarios del <code>LoggingAspect</code>
 */
public class LoggingAspectTest {

    private LoggingAspect aspect = new LoggingAspect();
    private JoinPoint joinPoint;
    private MethodSignature signature;

    private static final String TEXT_MESSAGE = "msg";
    private static final String TYPE_NAME = "typeName";
    private static final String METHOD_NAME = "methodName";

    @Before
    public void setup() {
        joinPoint = mock(JoinPoint.class);
        signature = mock(MethodSignature.class);
    }

    /**
     * Valida el método con una excepción no auditable
     */
    @Test
    public void logAfterDataAccessThrowingWithoutAuditableException() {

        //given: declaramos la excepción
        Throwable e = new Throwable();

        //configuramos el mock
        when(joinPoint.getSignature()).thenReturn(signature);
        when(signature.getDeclaringTypeName()).thenReturn(TYPE_NAME);
        when(signature.getName()).thenReturn(METHOD_NAME);

        //when: llamamos al método
        aspect.logAfterExceptionThrowing(joinPoint, e);

        //then: nada que validar, devuelve null
    }

    /**
     * Valida el método con una excepción auditable
     */
    @Test
    public void logAfterDataAccessThrowingWithAuditableExcepction() {
        //given: declaramos la excepción
        SistemaAutenticacionException e = new SistemaAutenticacionException(TEXT_MESSAGE);

        //configuramos el mock
        when(joinPoint.getSignature()).thenReturn(signature);
        when(signature.getDeclaringTypeName()).thenReturn(TYPE_NAME);
        when(signature.getName()).thenReturn(METHOD_NAME);

        //when: llamamos al método
        aspect.logAfterExceptionThrowing(joinPoint, e);

        //then: nada que validar, devuelve null
    }

    /**
     * Valida el log con una excepción de base de datos
     */
    @Test
    public void logAfterDataAccessThrowingWithDataAccessException() {
        //given: declaramos la excepción
        DataIntegrityViolationException e = new DataIntegrityViolationException(TEXT_MESSAGE);

        //configuramos el mock
        when(joinPoint.getSignature()).thenReturn(signature);
        when(signature.getDeclaringTypeName()).thenReturn(TYPE_NAME);
        when(signature.getName()).thenReturn(METHOD_NAME);

        //when: llamamos al método
        aspect.logAfterExceptionThrowing(joinPoint, e);

        //then: nada que validar, devuelve null
    }

    /**
     * Valida el log con una excepción de base de datos con causa de tipo
     * JDBCException
     */
    @Test
    public void logAfterDataAccessThrowingWithDataAccessExceptionAndCauseJDBCException() {
        //given: declaramos la excepción
        SQLException cause = new SQLException(TEXT_MESSAGE);
        DataIntegrityViolationException e = new DataIntegrityViolationException(TEXT_MESSAGE, cause);

        //configuramos el mock
        when(joinPoint.getSignature()).thenReturn(signature);
        when(signature.getDeclaringTypeName()).thenReturn(TYPE_NAME);
        when(signature.getName()).thenReturn(METHOD_NAME);

        //when: llamamos al método
        aspect.logAfterExceptionThrowing(joinPoint, e);

        //then: nada que validar, devuelve null
    }

}
