/* amtega-archam-core: Amtega Archetype Core
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-archam-core”.
*
* "amtega-archam-core" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-archam-core" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.archam.core.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import org.apache.commons.io.FileUtils;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Test;

import es.xunta.amtega.transversal.archam.core.io.exception.FileStorageException;
import es.xunta.amtega.transversal.utils.BaseUtils.Configuration;

/**
 * Test unitario para validar las funcionalidades del componente
 * StorageManagerImpl
 */
public class StorageManagerImplTest {

    private static final String MOCKUP_FILE = "/mockup.file";
    private static final String DOC_BIBLIO_PATH = "path.biblioteca.documentos";
    private static final String TEXT_EXPECTED_EXCEPTION = "Se esperaba una excepción FileStorageException";
    private static final String TEXT_FILE_NOT_SAVED = "El fichero no se guardó correctamente";
    private static final String TEXT_FILE_EXPECTED = "Se esperaba un fichero";
    private static final String TEXT_FILE_NOT_EXPECTED = "Se espera que el fichero no exista";

    StorageManager storage = new StorageManagerImpl();

    @Test(expected = FileStorageException.class)
    public void nullInputStreamShouldThrowException() {
        // given: el InputStream es null
        InputStream in = null;

        // when: intentamos guardar un InputStream
        storage.store(in);

        // then: el método debe lanzar una FileStorageException
        Assert.fail(TEXT_EXPECTED_EXCEPTION);
    }

    @Test(expected = FileStorageException.class)
    public void nullFileShouldThrowException() {
        // given: el File es null
        File in = null;

        // when: intentamos guardar un File
        storage.store(in);

        // then: el método debe lanzar una FileStorageException
        Assert.fail(TEXT_EXPECTED_EXCEPTION);
    }

    @Test
    public void inputStreamShouldBeSaved() throws FileNotFoundException {
        // given: el InputStream es válido
        String path = Configuration.getResourcesPath() + MOCKUP_FILE;
        InputStream in = new FileInputStream(new File(path));

        // when: Intentamos guardar un InputStream
        String filePath = storage.store(in);

        // then: el fichero se debe crear en el directorio de datos
        boolean fileCreated = new File(Configuration.getDataPath() + filePath).exists();
        Assert.assertTrue(TEXT_FILE_NOT_SAVED, fileCreated);
    }

    @Test
    public void existingFileShouldBeSaved() {
        // given: el fichero existe
        String path = Configuration.getResourcesPath() + MOCKUP_FILE;
        File file = new File(path);

        // when: intentamos guardar un fichero
        String filePath = storage.store(file);

        // then: el fichero se debe crear en el directorio de datos
        boolean fileCreated = new File(Configuration.getDataPath() + filePath).exists();
        Assert.assertTrue(TEXT_FILE_NOT_SAVED, fileCreated);
    }

    @Test
    public void existingFileShouldBeLoaded() {
        // given: el fichero a cargar existe en el directorio de documentos
        String filePath = storage.store(new File(Configuration.getResourcesPath() + MOCKUP_FILE));

        // when: cargamos el fichero
        File file = storage.load(filePath);

        // then: el fichero existe
        Assert.assertTrue(TEXT_FILE_EXPECTED, file.exists());
    }

    @Test
    public void existingFileShouldBeDeleted() {
        // given: el fichero a eliminar existe en el directorio de documentos
        String filePath = storage.store(new File(Configuration.getResourcesPath() + MOCKUP_FILE));
        File file = new File(getDocsPath() + filePath);

        // when: borramos el documento por ruta
        storage.delete(file);

        // then: el fichero no existe
        Assert.assertTrue(TEXT_FILE_NOT_EXPECTED, !file.exists());
    }

    @Test
    public void deleteWithNullPathDoNothing() {
        // given: declaramos en path con valor nulo
        String filePath = null;

        // when: borramos el documento por ruta
        storage.delete(filePath);

        // then: nada que validar
    }

    @Test
    public void existingFileByPathShouldBeDeleted() {
        // given: el fichero a eliminar existe en el directorio de documentos
        String filePath = storage.store(new File(Configuration.getResourcesPath() + MOCKUP_FILE));

        // when: borramos el documento por ruta
        storage.delete(filePath);

        // then: el fichero no existe
        File file = new File(getDocsPath() + filePath);
        Assert.assertTrue(TEXT_FILE_NOT_EXPECTED, !file.exists());
    }

    @AfterClass
    public static void cleanup() {
        // Borramos directorios y ficheros creados durantes los tests
        String path = getDocsPath();
        FileUtils.deleteQuietly(new File(path));
    }

    private static String getDocsPath() {
        return Configuration.getDataPath() + "/" + Configuration.getProperty(DOC_BIBLIO_PATH);
    }

}
