/* amtega-base-plugin-spring-impl: Módulo implementación Spring del proyecto Amtega Plugins
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-base-plugin”.
*
* "amtega-base-plugin" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-base-plugin" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.plugin.registry;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;

import es.xunta.amtega.transversal.plugin.DefaultModule;
import es.xunta.amtega.transversal.plugin.PluginInstance;
import es.xunta.amtega.transversal.plugin.PluginType;
import es.xunta.amtega.transversal.plugin.type.Component;

/**
 * Implementación de la interfaz PluginRegistry que se encarga del registro de
 * plugins. Contiene un Map dónde organiza los plugins por Namespace e Id
 */
class PluginRegistryImpl implements PluginRegistry {

    private Map<String, Map<String, PluginInstance>> plugins = new HashMap<String, Map<String, PluginInstance>>();
    private Map<Class<? extends PluginType>, List<PluginInstance>> pluginsByType = new HashMap<>();

    @Override
    public void add(PluginInstance plugin) {
        String namespace = plugin.getModule().getNamespace();
        if (plugins.get(namespace) == null) {
            plugins.put(namespace, new HashMap<String, PluginInstance>());
        }

        if (plugins.get(namespace).containsKey(plugin.getId())) {
            throw new IllegalArgumentException("Ya existe un plugin con el mismo namespace e id");
        }

        plugins.get(namespace).put(plugin.getId(), plugin);

        // Añadimos plugin a mapa de plugins por tipo
        Class<? extends PluginType> pluginType = plugin.getType();
        if (!pluginsByType.containsKey(pluginType)) {
            pluginsByType.put(pluginType, new ArrayList<PluginInstance>());
        }
        pluginsByType.get(pluginType).add(plugin);

        // Si el tipo de plugin es Sortable se ordenan las instancias
        if (Component.class.isAssignableFrom(pluginType)) {
            sortPluginInstances(pluginsByType.get(pluginType));
        }
    }

    @Override
    public Collection<PluginInstance> getAvailablePlugins() {
        Collection<PluginInstance> list = new ArrayList<PluginInstance>();
        for (Entry<String, Map<String, PluginInstance>> map : plugins.entrySet()) {
            list.addAll(new ArrayList<PluginInstance>(map.getValue().values()));
        }
        return list;
    }

    @Autowired(required = false)
    public void setPlugins(Set<PluginInstance> plugins) {
        for (PluginInstance plugin : plugins) {
            this.add(plugin);
        }
    }

    @Override
    public List<PluginInstance> getPluginsByType(Class<? extends PluginType> type) {

        if (!pluginsByType.containsKey(type)) {
            return Collections.emptyList();
        }

        return pluginsByType.get(type);
    }

    @Override
    public PluginInstance getPluginById(String id) {
        return getPluginById(id, DefaultModule.NAMESPACE);
    }

    @Override
    public PluginInstance getPluginById(String id, String namespace) {

        if (!plugins.containsKey(namespace)) {
            throw new IllegalArgumentException("No existen plugins en el namespace: " + namespace);
        }

        if (plugins.get(namespace).get(id) == null) {
            throw new IllegalArgumentException("No existen plugin con id: " + id);
        }

        return plugins.get(namespace).get(id);
    }

    /**
     * Ordena los plugins empleando el valor de Sortable.getSort()
     * 
     * @param plugins
     *            listado de plugins a ordenar
     */
    private void sortPluginInstances(List<PluginInstance> plugins) {

        Collections.sort(plugins, new Comparator<PluginInstance>() {

            @Override
            public int compare(PluginInstance o1, PluginInstance o2) {
                int o1Sort = ((Component) o1.getInstance()).getSort();
                int o2Sort = ((Component) o2.getInstance()).getSort();

                return o1Sort - o2Sort;
            }
        });
    }

}
