/* amtega-base-plugin-spring-impl: Módulo implementación Spring del proyecto Amtega Plugins
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-base-plugin”.
*
* "amtega-base-plugin" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-base-plugin" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.plugin.registry;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.aop.support.AopUtils;
import org.springframework.beans.factory.config.AbstractFactoryBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.core.env.Environment;

import es.xunta.amtega.transversal.plugin.ConfigurationAware;
import es.xunta.amtega.transversal.plugin.ConfigurationSource;
import es.xunta.amtega.transversal.plugin.Module;
import es.xunta.amtega.transversal.plugin.PluginType;
import es.xunta.amtega.transversal.plugin.annotation.Plugin;

/**
 * Recupera del contexto los beans anotados con @Plugin Crea una nueva instancia
 * que es añadida el registro de plugins Valida que la versión mínima del
 * arquetipo del plugin sea menor o igual que la utilizada en la aplicación
 */
public class PluginRegistryInstanceFactoryBean extends AbstractFactoryBean<PluginRegistry>
        implements ApplicationContextAware {

    private static final int VERSION_LENGTH = 3;
    private static final String VERSION_FILE = "/META-INF/archetype.version";
    private static final String VERSION_KEY = "archetype.version";

    private Properties properties = new Properties();

    private ApplicationContext applicationContext;

    private static final Logger LOGGER = LogManager.getLogger(PluginRegistryInstanceFactoryBean.class);

    private Map<Class<? extends Module>, Module> cacheModule = new HashMap<Class<? extends Module>, Module>();
    private String availableVersion;

    @Override
    public Class<?> getObjectType() {
        return PluginRegistry.class;
    }

    @SuppressWarnings("unchecked")
    @Override
    protected PluginRegistry createInstance() throws Exception {

        // Sacamos la versión usada del arquetipo en generar la aplicación
        try (InputStream inputStream = getClass().getResourceAsStream(VERSION_FILE)) {
            properties.load(inputStream);
            availableVersion = properties.getProperty(VERSION_KEY, null);
        } catch (Exception e) {
            LOGGER.warn("No se ha podido cargar el fichero de info. de versión: {}", VERSION_FILE, e);
        }

        final PluginRegistry registry = new PluginRegistryImpl();

        Map<String, Object> plugins = applicationContext.getBeansWithAnnotation(Plugin.class);
        Set<Entry<String, Object>> entries = plugins.entrySet();
        for (final Entry<String, Object> entry : entries) {
            final Object plugin = entry.getValue();
            
            // Si la clase está dentro de un Proxy se extrae la clase original
            final boolean isProxiedClass = AopUtils.isAopProxy(plugin);
            final Class<?> targetClass = isProxiedClass ? AopUtils.getTargetClass(plugin) : plugin.getClass();
            final Plugin annotation = targetClass.getAnnotation(Plugin.class);
            final String id = annotation.id();
            final Class<? extends Module> metadata = annotation.module();

            if (!cacheModule.containsKey(metadata)) {
                Module module = metadata.newInstance();
                checkMinVersion(module);
                cacheModule.put(metadata, module);
            }

            Module module = cacheModule.get(metadata);
            Class<?>[] interfaces = getPluginInterfaces(entry.getValue());
            for (final Class<?> interfaceClass : interfaces) {
                if (PluginType.class.isAssignableFrom(interfaceClass)) {
                    Class<? extends PluginType> type = (Class<? extends PluginType>) interfaceClass;
                    PluginType instance = (PluginType) entry.getValue();
                    PluginInstanceImpl pluginInstance = PluginInstanceImpl.with(id, module, type, instance);
                    registry.add(pluginInstance);

                    // Pasamos configuración a plugins configurables
                    configurePlugin(instance);
                }
            }
        }

        return registry;
    }

    private Class<?>[] getPluginInterfaces(Object plugin) {
        List<Class<?>> allInterfaces = new ArrayList<>();
        
        Class<?>[] interfaces = plugin.getClass().getInterfaces();
        if (interfaces.length > 0) {
            allInterfaces.addAll( Arrays.asList(interfaces) );
        }

        Class<?> parentClass = plugin.getClass().getSuperclass();
        if (parentClass != null && parentClass.getInterfaces().length > 0) {
            allInterfaces.addAll( Arrays.asList(parentClass.getInterfaces()) );
        }

        if( allInterfaces.isEmpty() ){
            throw new IllegalArgumentException("La clase no define el tipo de plugin con una interfaz");
        }
        
        return allInterfaces.toArray(new Class<?>[]{});
    }

    private void checkMinVersion(Module module) {

        // Si no hay versión del arquetipo no se valida
        if (availableVersion != null) {

            String moduleMinVersion = module.getMinVersion();
            // La versión mínima en los plugins es obligatoria
            checkNotNullVersion(moduleMinVersion);

            String[] tokensModuleMinVersion = tokenizeVersion(moduleMinVersion);
            String[] tokensAvailableVersion = tokenizeVersion(availableVersion);

            if (firstMayorThanSecond(tokensModuleMinVersion[0], tokensAvailableVersion[0])) {
                throw new IllegalArgumentException("Versión de plugin no soportada, versión mínima (major) no disponible");
            }
        }
    }
    
    private void configurePlugin(PluginType instance) {
        if (instance instanceof ConfigurationAware) {
            //Creamos un wrapper independiente por plugin
            final ConfigurationSource config = new EnvironmentConfigurationSourceWrapper(applicationContext.getEnvironment());
            ((ConfigurationAware) instance).configure(config);
        }
    }

    /**
     * Valida el formato de la versión
     * 
     * @param versionValue
     *            Versión a validar
     * @param message
     *            Mensaje para la excepción
     * @return valores numéricos de la versión validada
     */
    private String[] tokenizeVersion(String versionValue) {

        String[] tokensVersion = versionValue.split("\\.");

        if (tokensVersion.length != VERSION_LENGTH) {
            throw new IllegalArgumentException(
                    String.format("La información de versión [%s] no tiene un formato válido", versionValue));
        }

        return tokensVersion;
    }

    /**
     * Valida que el valor de la versión no sea nulo
     * 
     * @param versionValue
     *            Valor de la versión
     * @param message
     *            Mensaje
     */
    private void checkNotNullVersion(String versionValue) {
        if (versionValue == null) {
            throw new IllegalArgumentException("El plugin debe contar no versión mínima del arquetipo");
        }
    }

    /**
     * Compara dos valores y devuelve true si el primero es mayor que el segundo
     * 
     * @param first
     *            Primer entero en formato cadena
     * @param second
     *            Segundo entero en formato cadena
     * @return Resultado de la operación
     */
    private boolean firstMayorThanSecond(String first, String second) {
        return Integer.valueOf(first) > Integer.valueOf(second);
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    /**
     * Wrapper de implementación Environment
     */
    private class EnvironmentConfigurationSourceWrapper implements ConfigurationSource {
        private final Environment env;

        public EnvironmentConfigurationSourceWrapper(Environment env) {
            this.env = env;
        }

        @Override
        public boolean containsProperty(String key) {
            return env.containsProperty(key);
        }

        @Override
        public String getProperty(String key) {
            return env.getProperty(key);
        }

        @Override
        public String getProperty(String key, String defaultValue) {
            return env.getProperty(key, defaultValue);
        }

        @Override
        public <T> T getProperty(String key, Class<T> targetType) {
            return env.getProperty(key, targetType);
        }

        @Override
        public <T> T getProperty(String key, Class<T> targetType, T defaultValue) {
            return env.getProperty(key, targetType, defaultValue);
        }

    }

}
