/* amtega-base-plugin-spring-impl: Módulo implementación Spring del proyecto Amtega Plugins
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-base-plugin”.
*
* "amtega-base-plugin" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-base-plugin" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.plugin.support;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.GenericBeanDefinition;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.ClassPathBeanDefinitionScanner;
import org.springframework.context.support.ResourceBundleMessageSource;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.web.servlet.handler.MappedInterceptor;
import org.springframework.web.servlet.handler.SimpleUrlHandlerMapping;
import org.springframework.web.servlet.mvc.ParameterizableViewController;

import es.xunta.amtega.transversal.plugin.PluginInstance;
import es.xunta.amtega.transversal.plugin.annotation.Plugin;
import es.xunta.amtega.transversal.plugin.registry.PluginRegistry;
import es.xunta.amtega.transversal.plugin.registry.PluginRegistryInstanceFactoryBean;
import es.xunta.amtega.transversal.plugin.type.MessageSourceResource;
import es.xunta.amtega.transversal.plugin.type.ViewTemplate;
import es.xunta.amtega.transversal.plugin.web.interceptor.PluginRegistryAdviceInterceptor;

public class PluginSupportConfigurer implements InitializingBean, BeanPostProcessor {
    
    private static final int PARAM_0 = 0;
    private static final int PARAM_1 = 1;
    private static final int PARAM_2 = 2;
    private static final String DEFAULT_BASE_PACKAGE = "es.xunta.amtega";
    private static final String HANDLER_MAPPING_BEAN = "es.xunta.amtega.transversal.plugin.web.UrlHandlerMapping";
    private final ApplicationContext applicationContext;
    private Collection<String> basePackages;

    public PluginSupportConfigurer(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    public void setBasePackages(String... strings) {
        if (strings == null || strings.length == 0) {
            basePackages = Collections.emptyList();
        } else {
            basePackages = new HashSet<>(Arrays.asList(strings));
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        BeanDefinitionRegistry registry = (BeanDefinitionRegistry) applicationContext.getAutowireCapableBeanFactory();

        // Buscamos clases anotadas con @Plugin
        ClassPathBeanDefinitionScanner scanner = new ClassPathBeanDefinitionScanner(registry, false);
        scanner.addIncludeFilter(new AnnotationTypeFilter(Plugin.class));

        Collection<String> basePackagesToScann = getBasePackagesToScan();
        for (String basePackage : basePackagesToScann) {
            Set<BeanDefinition> candidates = scanner.findCandidateComponents(basePackage);
            for (BeanDefinition bd : candidates) {
                registry.registerBeanDefinition(bd.getBeanClassName(), bd);
            }
        }

        registerPluginRegistryFactory(registry);
        configureWebMvcSupport(registry);
    }

    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) {
        return bean;
    }

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) {

        if (bean instanceof ResourceBundleMessageSource) {
            // Modificamos configuración de ResourceBundleMessageSource
            // definidos
            PluginRegistry pluginRegistry = getPluginRegistry();
            List<PluginInstance> sources = pluginRegistry.getPluginsByType(MessageSourceResource.class);
            for (PluginInstance plugin : sources) {
                String baseName = plugin.getInstance(MessageSourceResource.class).getBaseName();
                ((ResourceBundleMessageSource) bean).addBasenames(baseName);
            }
        }

        return bean;
    }

    private Collection<String> getBasePackagesToScan() {
        if (basePackages == null) {
            return Collections.singleton(DEFAULT_BASE_PACKAGE);
        }

        return basePackages;
    }

    private void registerPluginRegistryFactory(BeanDefinitionRegistry registry) {
        RootBeanDefinition beanDef = new RootBeanDefinition(PluginRegistryInstanceFactoryBean.class);
        beanDef.setLazyInit(false);
        beanDef.setAutowireCandidate(true);
        registry.registerBeanDefinition(beanDef.getBeanClassName(), beanDef);
    }

    private void configureWebMvcSupport(BeanDefinitionRegistry registry) {
        addViewControllers(registry);
        addInterceptors(registry);
    }

    @SuppressWarnings("unchecked")
    private void addViewControllers(BeanDefinitionRegistry registry) {
        List<PluginInstance> views = getPluginRegistry().getPluginsByType(ViewTemplate.class);

        if (!views.isEmpty()) {
            // Register SimpleUrlHandlerMapping for view controllers
            BeanDefinition hm = registerHandlerMapping(registry);
            for (PluginInstance plugin : views) {
                // Create view controller bean definition
                RootBeanDefinition controller = new RootBeanDefinition(ParameterizableViewController.class);

                String url = createPluginViewUrl(plugin);
                Map<String, BeanDefinition> urlMap = (Map<String, BeanDefinition>) hm.getPropertyValues()
                        .getPropertyValue("urlMap").getValue();
                urlMap.put(url, controller);
            }
        }
    }

    private BeanDefinition registerHandlerMapping(BeanDefinitionRegistry registry) {

        if (registry.containsBeanDefinition(HANDLER_MAPPING_BEAN)) {
            return registry.getBeanDefinition(HANDLER_MAPPING_BEAN);
        }

        RootBeanDefinition beanDef = new RootBeanDefinition(SimpleUrlHandlerMapping.class);
        beanDef.setRole(BeanDefinition.ROLE_INFRASTRUCTURE);
        beanDef.getPropertyValues().add("order", "1");
        beanDef.getPropertyValues().add("urlMap", new ManagedMap<String, BeanDefinition>());

        registry.registerBeanDefinition(HANDLER_MAPPING_BEAN, beanDef);

        return beanDef;
    }

    private String createPluginViewUrl(PluginInstance plugin) {
        String namespace = plugin.getModule().getNamespace();
        String id = plugin.getId();
        String template = plugin.getInstance(ViewTemplate.class).getTemplate();

        StringBuilder builder = new StringBuilder("/plugins/");
        if (namespace != null && !"".equals(namespace)) {
            builder.append(namespace).append("/");
        }
        builder.append(id).append("/").append(template);

        return builder.toString();
    }

    private void addInterceptors(BeanDefinitionRegistry registry) {
        // interceptor
        RootBeanDefinition mappedInterceptorDef = new RootBeanDefinition(MappedInterceptor.class);
        mappedInterceptorDef.setRole(BeanDefinition.ROLE_INFRASTRUCTURE);

        GenericBeanDefinition interceptorBean = new GenericBeanDefinition();
        interceptorBean.setBeanClass(PluginRegistryAdviceInterceptor.class);
        ManagedList<String> includePatterns = null;
        ManagedList<String> excludePatterns = null;
        mappedInterceptorDef.getConstructorArgumentValues().addIndexedArgumentValue(PARAM_0, includePatterns);
        mappedInterceptorDef.getConstructorArgumentValues().addIndexedArgumentValue(PARAM_1, excludePatterns);
        mappedInterceptorDef.getConstructorArgumentValues().addIndexedArgumentValue(PARAM_2, interceptorBean);

        registry.registerBeanDefinition(interceptorBean.getBeanClassName(), mappedInterceptorDef);
    }

    private PluginRegistry getPluginRegistry() {
        return applicationContext.getBean(PluginRegistry.class);
    }

}
