/* amtega-base-plugin-spring-impl: Módulo implementación Spring del proyecto Amtega Plugins
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-base-plugin”.
*
* "amtega-base-plugin" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-base-plugin" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.plugin.web.interceptor;

import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import es.xunta.amtega.transversal.plugin.PluginInstance;
import es.xunta.amtega.transversal.plugin.PluginType;
import es.xunta.amtega.transversal.plugin.registry.PluginRegistry;
import es.xunta.amtega.transversal.plugin.type.Component;
import es.xunta.amtega.transversal.plugin.type.ViewFragment;

/**
 * Clase que da acceso al registro de plugins desde la vista
 */
class PluginRegistryViewHelper {

    private static final String DEFAULT_PACKAGE = Component.class.getPackage().getName() + ".";
    private final PluginRegistry registry;

    /**
     * Constructor con el registro de plugins
     *
     * @param registry Registro de plugins
     */
    public PluginRegistryViewHelper(PluginRegistry registry) {
        this.registry = registry;
    }

    /**
     * Recupera un plugin por id del namespace por defecto
     *
     * @param id Id
     * @return Plugin
     */
    public PluginInstance getPlugin(String id) {
        return registry.getPluginById(id);
    }

    /**
     * Recupera un plugin por id y por namespace
     *
     * @param id Id
     * @param namespace Namespace
     * @return Plugin
     */
    public PluginInstance getPlugin(String id, String namespace) {
        return registry.getPluginById(id, namespace);
    }

    public Collection<PluginInstance> getPlugins() {
        return registry.getAvailablePlugins();
    }

    /**
     * Recupera una colección de plugins por tipo
     *
     * @param type Tipo de plugin
     * @return Colección de plugins
     */
    public Collection<PluginInstance> getPlugins(String type) {
        return getPluginsByType(type);
    }

    /**
     * Busca el plugin identificado por su id y devuelve su representación HTML
     *
     * @param id identificador de plugin
     * @param request petición HTTP actual
     * @return contenido HTML generado por el plugin
     */
    public String render(String id, HttpServletRequest request) {
        PluginInstance plugin = getPlugin(id);
        if (plugin != null && ViewFragment.class.isAssignableFrom(plugin.getType())) {
            return plugin.getInstance(ViewFragment.class).getHtml(request);
        }

        return null;
    }

    /**
     * Valida que el tipo de plugin sea válido
     *
     * @param className Nombre del la clase del plugin
     * @return Lista de plugins
     */
    @SuppressWarnings("unchecked")
    private List<PluginInstance> getPluginsByType(String className) {
        try {
            String qualifiedName = hasPackage(className) ? className : DEFAULT_PACKAGE + className;
            Class<PluginType> cls = (Class<PluginType>) Class.forName(qualifiedName);
            return registry.getPluginsByType(cls);
        } catch (Exception e) {
            throw new IllegalArgumentException("Tipo de plugin no soportado", e);
        }
    }

    private boolean hasPackage(String className) {
        return String.valueOf(className).contains(".");
    }

}
