/* amtega-common-authentication: Amtega Common Authentication
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-authentication".
*
* "amtega-common-authentication" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-authentication" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.authentication;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.authentication.exception.AccountNotFoundException;
import es.xunta.amtega.transversal.authentication.exception.CommunicationException;
import es.xunta.amtega.transversal.authentication.exception.InvalidLoginCredentialsException;
import es.xunta.amtega.transversal.authentication.exception.MethodResponseException;
import es.xunta.amtega.transversal.authentication.exception.MultipleAccountsMatchException;
import es.xunta.amtega.transversal.authentication.provider.AuthenticationProvider;
import es.xunta.amtega.transversal.authentication.provider.AuthenticationProviderRegistry;

/**
 * Clase de utilidad que gestiona la creación del proveedor de autenticación y
 * permite acceder a las funcionalidades de autenticación
 */
public final class AuthenticationManager {

    private static final Logger LOGGER = LogManager.getLogger(AuthenticationManager.class);

    /**
     * Clave de propiedad empleada para indicar el usuario de cuenta de servicio
     */
    public static final String APPLICATION_USERNAME = "authentication.app.username";
    /**
     * Clave de propidad empleada para definir la contraseña de la cuenta de
     * servicio
     */
    public static final String APPLICATION_KEY_WORD = "authentication.app.password";

    private Properties config;

    /**
     * Constructor por defecto del mecanismo de autenticación
     */
    public AuthenticationManager() {
        this.config = new Properties();
    }

    /**
     * Constructor de mecanismo de autenticación indicando parámetros de
     * configuración
     *
     * @param config parámetros de configuración
     */
    public AuthenticationManager(Properties config) {
        this();
        setConfiguration(config);
    }

    /**
     * Establece la configuración del mecanismo de autenticación
     *
     * @param config parámetros de configuración
     */
    public void setConfiguration(Properties config) {
        if (config != null) {
            this.config = config;
        }
    }

    /**
     * Realiza la autenticación de un usuario mediante nombre de usuario y
     * contraseña
     *
     * @param username nombre de usuario
     * @param password contraseña
     * @throws InvalidLoginCredentialsException en caso de error debido a
     * credenciales inválidas
     * @throws CommunicationException en caso de error de comunicación
     */
    public void login(String username, String password)
            throws InvalidLoginCredentialsException, CommunicationException {
        // Realizamos login sobre los proveedores disponibles
        final StringBuilder builder = new StringBuilder();
        final Collection<AuthenticationProvider> providers = getProviders();
        for (AuthenticationProvider provider : providers) {
            try {
                provider.login(username, password);

                // Se finaliza la búsqueda si alguno de los proveedores responde
                // OK
                return;
            } catch (InvalidLoginCredentialsException e) {
                builder.append(provider.getIdentifier()).append(": ").append(e.getMessage());
                if (e.getCause() != null) {
                    builder.append(" - ").append(e.getCause().getMessage());
                }
                LOGGER.warn("Error de autenticación {}", builder.toString(), e);
            }
        }

        throw new InvalidLoginCredentialsException(builder.toString());
    }

    /**
     * Obtiene los datos de una cuenta identificada por el nombre de usuario
     *
     * @param username nombre de usuario a consultar
     * @return datos de cuenta de usuario
     * @throws AccountNotFoundException en caso de no encontrar la cuenta del
     * usuario
     * @throws MultipleAccountsMatchException en caso de encontrar más de una
     * cuenta para el usuario
     * @throws CommunicationException en caso de error de comunicación
     */
    public Account findAccount(String username)
            throws AccountNotFoundException, MultipleAccountsMatchException, CommunicationException {
        // Buscamos la cuenta en los proveedores disponibles y devuelve la
        // primera ocurrencia
        final Collection<AuthenticationProvider> providers = getProviders();
        for (AuthenticationProvider provider : providers) {
            try {
                return provider.findAccount(username);
            } catch (AccountNotFoundException e) {
                LOGGER.warn("Error recuperando cuenta de usuario: {} ", username, e);
            } catch (MultipleAccountsMatchException e) {
                LOGGER.warn("Múltiples cuentas de usuario: {} ", username, e);
            }
        }

        throw new AccountNotFoundException("No se ha encontrado la cuenta de usuario");
    }

    /**
     * Permite realizar búsquedas sobre el repositorio de identidades
     *
     * @param query consulta a realizar (query de ldap)
     * @return lista de cuentas coincidentes
     * @throws CommunicationException en caso de error de comunicación
     */
    public List<Account> findAccounts(String query) throws CommunicationException {
        // Busca todas la cuenta en todos los proveedores disponibles
        final List<Account> accounts = new ArrayList<Account>();
        final Collection<AuthenticationProvider> providers = getProviders();
        for (AuthenticationProvider provider : providers) {
            try {
                accounts.addAll(provider.findAccounts(query));
            } catch (MethodResponseException e) {
                LOGGER.warn("El proveedor {} no acepta la consulta: {}", provider.getIdentifier(), query, e);
            }
        }

        return Collections.unmodifiableList(accounts);
    }

    /**
     * Crea la lista de proveedores disponibles (activos)
     *
     * @return lista de proveedores
     */
    private Collection<AuthenticationProvider> getProviders() {
        Collection<AuthenticationProvider> availableProviders = AuthenticationProviderRegistry.getInstance()
                .getProviders();
        Collection<AuthenticationProvider> providers = new ArrayList<AuthenticationProvider>();
        for (AuthenticationProvider provider : availableProviders) {
            provider.setup(config);
            if (provider.isActive()) {
                providers.add(provider);
            }
        }

        return providers;
    }

}
