/* amtega-common-authentication: Amtega Common Authentication
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-authentication".
*
* "amtega-common-authentication" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-authentication" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.authentication.provider;

import java.util.List;
import java.util.Properties;

import es.xunta.amtega.transversal.authentication.Account;
import es.xunta.amtega.transversal.authentication.exception.AccountNotFoundException;
import es.xunta.amtega.transversal.authentication.exception.CommunicationException;
import es.xunta.amtega.transversal.authentication.exception.InvalidLoginCredentialsException;
import es.xunta.amtega.transversal.authentication.exception.MultipleAccountsMatchException;

/**
 * Definición de los métodos que debe implementar un proveedor de autenticación
 */
public interface AuthenticationProvider {

    /**
     * Identificador del proveedor de autenticación
     *
     * @return nombre descriptivo del proveedor de autenticación
     */
    String getIdentifier();

    /**
     * Realiza la autenticación de un usuario mediante nombre de usuario y
     * contraseña
     *
     * @param username nombre de usuario
     * @param password contraseña
     * @throws InvalidLoginCredentialsException en caso de error debido a
     * credenciales inválidas
     * @throws CommunicationException en caso de error de comunicación
     */
    void login(String username, String password) throws InvalidLoginCredentialsException, CommunicationException;

    /**
     * Obtiene los datos de una cuenta de usuario por el nombre de usuario
     *
     * @param username nombre de usuario a consultar
     * @return datos de cuenta de usuario
     * @throws AccountNotFoundException en caso de no encontrar la cuenta del
     * usuario
     * @throws MultipleAccountsMatchException en caso de encontrar más de una
     * cuenta para el usuario
     * @throws CommunicationException en caso de error de comunicación
     */
    Account findAccount(String username)
            throws AccountNotFoundException, MultipleAccountsMatchException, CommunicationException;

    /**
     * Consulta en el repositorio de identidades si existen cuentas para el
     * usuario indicado
     *
     * @param query consulta a realizar sobre el repositorio de identidades
     * @return lista de cuentas encontradas
     * @throws CommunicationException en caso de error de comunicación
     */
    List<Account> findAccounts(String query) throws CommunicationException;

    /**
     * Se indica la configuración a usar por el provedor de autenticación
     *
     * @param config parámetros de configuración
     */
    void setup(Properties config);

    /**
     * Indica si el proveedor está activo, se permite el uso de este proveedor
     *
     * @return true si se realizará intento de validación con este proveedor
     */
    boolean isActive();

    /**
     * Prioridad del proveedor (menor prioridad se ejecuta primero)
     *
     * @return valor de prioridad
     */
    int getPriority();

}
