/* amtega-common-authentication: Amtega Common Authentication
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-authentication".
*
* "amtega-common-authentication" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-authentication" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.authentication.provider.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.OperationNotSupportedException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.authentication.Account;
import es.xunta.amtega.transversal.authentication.AuthenticationManager;
import es.xunta.amtega.transversal.authentication.exception.AccountNotFoundException;
import es.xunta.amtega.transversal.authentication.exception.CommunicationException;
import es.xunta.amtega.transversal.authentication.exception.InvalidLoginCredentialsException;
import es.xunta.amtega.transversal.authentication.exception.MethodResponseException;
import es.xunta.amtega.transversal.authentication.exception.MultipleAccountsMatchException;
import es.xunta.amtega.transversal.authentication.provider.AuthenticationProvider;

/**
 * Implementación de proveedor de autenticación sobre LDAP
 */
public final class LDAPAuthenticationProviderImpl implements AuthenticationProvider {

    private static final Logger LOGGER = LogManager.getLogger(LDAPAuthenticationProviderImpl.class);

    /**
     * Constante vacía
     */
    private static final String EMPTY = "";
    /**
     * Clave de propiedad empleada para definir los atributos extra a buscar en
     * el LDAP
     */
    private static final String KEY_ATRIBUTOS = "authentication.attributes";
    /**
     * Clave de propiedad empleada para definir la URL del LDAP
     */
    private static final String KEY_LDAP_URL = "ldap.url";
    /**
     * Clave de propiedad empleada para definir el prefijo de la búsqueda en el
     * LDAP
     */
    private static final String KEY_PRINCIPAL_PREFIX = "ldap.principal.prefix";
    /**
     * Atributos mínimos a buscar en el LDAP
     */
    private static final String[] DEFAULT_ATTRIBUTES = {"sAMAccountName", "givenName", "sn", "mail", "employeeID"};
    /**
     * Contexto de la búsqueda en el LDAP
     */
    private static final String KEY_SEARCH_CONTEXT = "ldap.search.context";
    /**
     * Clave de propiedad empleada para definir si el LDAP está activo o no
     */
    private static final String LDAP_ACTIVE = "ldap.active";
    /**
     * Prefijo de nombre de usuario por defecto
     */
    private static final String DEFAULT_PRINCIPAL_PREFIX = "XUNTA\\";
    /**
     * Prioridad por defecto
     */
    private static final int DEFAULT_PRIORITY = 100;

    // Usuario y contraseña de aplicación para consulta sobre LDAP
    private String appUsername;
    private String appPassword;
    private String[] searchAttributes = DEFAULT_ATTRIBUTES;
    private String providerUrl;
    private String principalPrefix = DEFAULT_PRINCIPAL_PREFIX;
    private String searchContext = EMPTY;
    private boolean active = true;

    public LDAPAuthenticationProviderImpl() {

    }

    @Override
    public void login(String username, String password)
            throws InvalidLoginCredentialsException, CommunicationException {
        DirContext ctx = null;
        try {
            ctx = connect(username, password);
        } catch (AuthenticationException e) {
            LOGGER.error("Error de autenticación en LDAP", e);
            throw new InvalidLoginCredentialsException("Error de autenticación", e);
        } catch (OperationNotSupportedException e) {
            LOGGER.error("Error de operación no soportada en LDAP", e);
            throw new InvalidLoginCredentialsException("Operación no soportada", e);
        } catch (NamingException e) {
            LOGGER.error("Error de conexión a LDAP con usuario: {}", username, e);
            throw new CommunicationException("Error de LDAP", e);
        } finally {
            close(ctx);
        }
    }

    @Override
    public Account findAccount(String username)
            throws AccountNotFoundException, MultipleAccountsMatchException, CommunicationException {
        List<Account> accounts = findAccounts("sAMAccountName=" + username);
        if (accounts.isEmpty()) {
            throw new AccountNotFoundException("Cuenta no encontrada para usuario: " + username);
        }
        if (accounts.size() > 1) {
            throw new MultipleAccountsMatchException("Se han encontrado varias cuentas para usuario: " + username);
        }
        return accounts.get(0);
    }

    @Override
    public List<Account> findAccounts(String query) throws CommunicationException {
        DirContext ctx = null;
        try {
            ctx = connect(appUsername, appPassword);

            // Definimos búsqueda
            final SearchControls constraints = new SearchControls();
            constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
            constraints.setReturningAttributes(searchAttributes);

            // Ejecutamos y recuperamos información de consulta
            final NamingEnumeration<SearchResult> answer = ctx.search(searchContext, query, constraints);
            final List<Account> accounts = getAccountList(answer);

            return Collections.unmodifiableList(accounts);
        } catch (NamingException e) {
            LOGGER.warn("Error recuperando de cuentas: {}", query, e);
            throw new MethodResponseException("Error recuperando información de cuentas: " + query, e);
        } finally {
            close(ctx);
        }
    }

    private List<Account> getAccountList(NamingEnumeration<SearchResult> searchResults) throws NamingException {
        List<Account> accountsList = new ArrayList<Account>();
        while (searchResults.hasMoreElements()) {
            final Map<String, Object> accountAttrs = new HashMap<String, Object>();
            Attributes attrs = searchResults.next().getAttributes();
            NamingEnumeration<? extends Attribute> values = attrs.getAll();
            while (values.hasMore()) {
                Attribute attr = values.next();
                String key = attr.getID();

                if (attr.size() == 1) {
                    accountAttrs.put(key, attr.get());
                } else {
                    // Añadimos atributos multivaluados como Set
                    Collection<String> attrValues = getAttributeCollection(attr);
                    accountAttrs.put(key, Collections.unmodifiableCollection(attrValues));
                }
            }
            accountsList.add(AccountImpl.build(accountAttrs));
        }
        return accountsList;
    }

    private Collection<String> getAttributeCollection(Attribute attribute) throws NamingException {
        Collection<String> attrValues = new HashSet<String>();
        for (int i = 0; i < attribute.size(); i++) {
            attrValues.add((String) attribute.get(i));
        }
        return attrValues;
    }

    /**
     * Conecta con el DAP
     *
     * @param username Usuario del LDAP
     * @param password Contraseña del LDAP
     * @return Contexto
     * @throws NamingException En caso de error
     */
    private DirContext connect(String username, String password) throws NamingException {
        // Validamos datos de usuario y contraseña, LDAP no lanza error con
        // contraseña vacía
        if (username == null || username.trim().isEmpty()) {
            throw new AuthenticationException("No se ha indicado un nombre de usuario");
        }

        if (password == null || password.trim().isEmpty()) {
            throw new AuthenticationException("No se ha indicado una contraseña válida");
        }

        // Datos de conexión a LDAP
        Map<String, String> env = new HashMap<String, String>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, providerUrl);
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, principalPrefix + username);
        env.put(Context.SECURITY_CREDENTIALS, password);

        // Establece conexión con LDAP
        return new InitialDirContext(new Hashtable<String, String>(env));
    }

    /**
     * Cierra el contexto
     *
     * @param ctx Contexto
     */
    private void close(DirContext ctx) {
        if (ctx != null) {
            try {
                ctx.close();
            } catch (NamingException e) {
                LOGGER.error("Error cerrando conexión a LDAP", e);
            }
        }
    }

    @Override
    public void setup(Properties config) {
        this.appUsername = config.getProperty(AuthenticationManager.APPLICATION_USERNAME, EMPTY);
        this.appPassword = config.getProperty(AuthenticationManager.APPLICATION_KEY_WORD, EMPTY);
        this.searchContext = config.getProperty(KEY_SEARCH_CONTEXT, EMPTY);
        this.active = Boolean.valueOf(config.getProperty(LDAP_ACTIVE, Boolean.TRUE.toString()));

        if (config.containsKey(KEY_ATRIBUTOS) && !config.getProperty(KEY_ATRIBUTOS).isEmpty()) {
            // Atributos de consulta adicionales en LDAP
            final String[] attrs = config.getProperty(KEY_ATRIBUTOS).replaceAll(" ", EMPTY).split(",");
            final List<String> extraAttributes = new ArrayList<String>(Arrays.asList(DEFAULT_ATTRIBUTES));
            extraAttributes.addAll(Arrays.asList(attrs));
            searchAttributes = extraAttributes.toArray(new String[]{});
        }

        if (config.containsKey(KEY_LDAP_URL)) {
            this.providerUrl = config.getProperty(KEY_LDAP_URL);
        }

        if (config.containsKey(KEY_PRINCIPAL_PREFIX)) {
            this.principalPrefix = config.getProperty(KEY_PRINCIPAL_PREFIX);
        }
    }

    @Override
    public boolean isActive() {
        return active;
    }

    @Override
    public int getPriority() {
        return DEFAULT_PRIORITY;
    }

    @Override
    public String getIdentifier() {
        return "LDAP";
    }

}
