/* amtega-common-authentication: Amtega Common Authentication
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-authentication".
*
* "amtega-common-authentication" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-authentication" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.test;

import java.util.Collection;
import java.util.Properties;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import es.xunta.amtega.transversal.authentication.Account;
import es.xunta.amtega.transversal.authentication.AuthenticationManager;
import es.xunta.amtega.transversal.authentication.exception.AccountNotFoundException;
import es.xunta.amtega.transversal.authentication.exception.CommunicationException;
import es.xunta.amtega.transversal.authentication.exception.InvalidLoginCredentialsException;
import es.xunta.amtega.transversal.authentication.exception.MultipleAccountsMatchException;
import es.xunta.amtega.transversal.authentication.provider.AuthenticationProvider;
import es.xunta.amtega.transversal.authentication.provider.AuthenticationProviderRegistry;

public class AuthenticationManagerTest {

    private Properties config = new Properties();
    private AuthenticationManager auth;

    private static final String TEST_LDAP_URL = "ldap://test:389/";
    private static final int NUM_PROVIDERS = 3;

    @Before
    public void init() {
        config.put("ldap.url", TEST_LDAP_URL);
        auth = new AuthenticationManager(config);
    }

    @Test
    public void availableServicesTest() {
        Collection<AuthenticationProvider> providers = AuthenticationProviderRegistry.getInstance().getProviders();
        Assert.assertTrue("El número de proveedores no es correcto", providers.size() == NUM_PROVIDERS);
    }

    @Test
    public void activarProveedorTest() {
        Properties conf = new Properties();
        conf.setProperty("ldap.active", Boolean.FALSE.toString());
        conf.setProperty("authentication.dummy", Boolean.TRUE.toString());
        conf.setProperty("test.active", Boolean.TRUE.toString());
        Collection<AuthenticationProvider> providers = AuthenticationProviderRegistry.getInstance().getProviders();
        for (AuthenticationProvider provider : providers) {
            provider.setup(conf);
            if ("LDAP".equals(provider.getIdentifier())) {
                if (provider.isActive()) {
                    Assert.fail("El proveedor de LDAP está activo");
                }
            } else if (!provider.isActive()) {
                Assert.fail("El proveedor no está activo");
            }
        }
    }

    @Test
    public void testAuthentionProviderTest() throws InvalidLoginCredentialsException, CommunicationException,
            AccountNotFoundException, MultipleAccountsMatchException {
        config.setProperty("test.active", Boolean.TRUE.toString());
        auth.login("test", "test");

        Account account = auth.findAccount("test");
        Assert.assertEquals("El usuario no se corresponde (test!=test)", "test", account.getUsuario());
    }

    @Test @Ignore
    public void invalidCredentialsTest() throws CommunicationException {

        try {
            auth.login("usuario.novalido", "contrasena.novalida1");
            Assert.fail("El método debe fallar al validar usuario incorrecto");
        } catch (InvalidLoginCredentialsException e) {
            // Validamos que falla el método de login
            Assert.assertNotNull(e);
        }

        try {
            auth.login("ue00294", "contrasena.novalida2");
            Assert.fail("El método debe fallar al validar contraseña incorrecta");
        } catch (InvalidLoginCredentialsException e) {
            // Validamos que falla el método de login
            Assert.assertNotNull(e);
        }
    }

    @Test
    public void emptyCredentialsTest() throws CommunicationException {
        String errorMessage = "El método debe fallar al validar contraseña incorrecta";

        try {
            auth.login(null, "");
            Assert.fail(errorMessage);
        } catch (InvalidLoginCredentialsException e) {
            // Validamos que falla el método de login
            Assert.assertNotNull(e);
        }

        try {
            auth.login("", "contrasena.novalida3");
            Assert.fail(errorMessage);
        } catch (InvalidLoginCredentialsException e) {
            // Validamos que falla el método de login
            Assert.assertNotNull(e);
        }

        try {
            auth.login("usuario", null);
            Assert.fail(errorMessage);
        } catch (InvalidLoginCredentialsException e) {
            // Validamos que falla el método de login
            Assert.assertNotNull(e);
        }

        try {
            auth.login("usuario", "");
            Assert.fail(errorMessage);
        } catch (InvalidLoginCredentialsException e) {
            // Validamos que falla el método de login
            Assert.assertNotNull(e);
        }
    }

}
