/* amtega-common-configuration: Amtega Common Configuration
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-configuration”.
*
* "amtega-common-configuration" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-configuration" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.configuration;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.configuration.environment.EnvironmentVariables;
import es.xunta.amtega.transversal.configuration.exception.UndefinedConfigurationParametersException;

/**
 * Clase de utilidad para leer ficheros de configuración según el estándar de
 * ubicación de ficheros Amtega
 */
public final class Configuration {
    private static final Logger LOGGER = LogManager.getLogger(Configuration.class);

    private static final String KEY_APP_USERNAME = "app.user";
    private static final String KEY_APP_KEYWORD = "app.pass";
    private final String applicationCode;
    private final Properties properties;

    /**
     * Constructor privado
     * 
     * @param appCode
     *            Código de aplicación
     * @param resources
     *            Recusos (ficheros de configuración)
     */
    private Configuration(String appCode, String... resources) {
        this.applicationCode = appCode;
        final PropertiesLoader loader = new PropertiesLoader(getConfigPath());

        // Fuentes de propiedades de configuración
        final Collection<PropertiesSource> sources = new ArrayList<PropertiesSource>();
        for (String resource : resources) {
            Properties props = loader.load(resource);
            sources.add(new SimplePropertiesSource(props));
        }

        // Añadimos almacenes de propiedades personalizados
        sources.addAll(PropertiesSourceRegistry.getInstance().getPropertiesSources());

        this.properties = new UnmodifiableProperties(sources);
    }

    /**
     * Devuelve el código de aplicación indicado para la configuración
     * 
     * @return Código de aplicación
     */
    public String getApplicationCode() {
        return applicationCode;
    }

    /**
     * Obtiene todas las propiedades definidas para la
     * aplicación
     * 
     * @return Propiedades
     */
    public Properties getProperties() {
        return properties;
    }

    /**
     * Lee una propidad existente en alguno de los ficheros de configuración
     * definidos en configuration.file. En caso de estar prefijada por {cipher}
     * se descifrará automáticamente su contenido.
     * 
     * @param key
     *            Clave de configuración
     * @return Valor de configuración
     */
    public String getProperty(String key) {
        return getProperties().getProperty(key);
    }

    /**
     * Devuelve el valor de la propiedad o el valor por defecto si no se
     * encuentra
     * 
     * @see #getProperty
     * 
     * @param key
     *            Clave de configuración
     * @param defaultValue
     *            Valor por defecto
     * @return Valor de configuración
     */
    public String getProperty(String key, String defaultValue) {
        return getProperties().getProperty(key, defaultValue);
    }

    /**
     * Devuelve la propiedad como boolean o el valor por defecto si no se
     * encuentra
     * 
     * @see #getProperty
     * 
     * @param key
     *            Clave de configuración
     * @param defaultValue
     *            Valor por defecto
     * @return Valor de configuración
     */
    public boolean getProperty(String key, boolean defaultValue) {
        return Boolean.parseBoolean(getProperty(key, String.valueOf(defaultValue)));
    }

    /**
     * Devuelve la propiedad como int o el valor por defecto si no se encuentra
     * 
     * @see #getProperty
     * 
     * @param key
     *            Clave de configuración
     * @param defaultValue
     *            Valor por defecto
     * @return Valor de configuración
     */
    public int getProperty(String key, int defaultValue) {
        return Integer.parseInt(getProperty(key, String.valueOf(defaultValue)));
    }

    /**
     * Devuelve la propiedad como long o el valor por defecto si no se encuentra
     * 
     * @see #getProperty
     * 
     * @param key
     *            Clave de configuración
     * @param defaultValue
     *            Valor por defecto
     * @return Valor de configuración
     */
    public long getProperty(String key, long defaultValue) {
        return Long.parseLong(getProperty(key, String.valueOf(defaultValue)));
    }

    /**
     * Devuelve la propiedad como double o el valor por defecto si no se
     * encuentra
     * 
     * @see #getProperty
     * 
     * @param key
     *            Clave de configuración
     * @param defaultValue
     *            Valor por defecto
     * @return Valor de configuración
     */
    public double getProperty(String key, double defaultValue) {
        return Double.parseDouble(getProperty(key, String.valueOf(defaultValue)));
    }

    /**
     * Devuelve el usuario de la cuenta de servicio de la aplicación
     * 
     * @return Usuario
     */
    public String getUsername() {
        return getProperty(KEY_APP_USERNAME);
    }

    /**
     * Devuelve la contraseña de la cuenta de servicio de la aplicación
     * 
     * @return Contraseña
     */
    public String getPassword() {
        return getProperty(KEY_APP_KEYWORD);
    }

    /**
     * Devuelve la ruta al directorio de ficheros de configuración de la
     * aplicación
     * 
     * @return Ruta de configuración
     */
    public String getConfigPath() {
        return buildPath(EnvironmentVariables.getConfigPath());
    }

    /**
     * Devuelve la ruta al directorio de logs de la aplicación
     * 
     * @return Ruta de logs
     */
    public String getLogPath() {
        return buildPath(EnvironmentVariables.getLogPath());
    }

    /**
     * Devuelve la ruta al directorio de ficheros temporales de la aplicación
     * 
     * @return Ruta de ficheros temporales
     */
    public String getTempPath() {
        return buildPath(EnvironmentVariables.getTempPath());
    }

    /**
     * Devuelve la ruta al directorio de ficheros permanentes de la aplicación
     * 
     * @return Ruta de datos
     */
    public String getDataPath() {
        return buildPath(EnvironmentVariables.getDataPath());
    }

    /**
     * Devuelve la ruta al directorio de recursos de la aplicación
     * 
     * @return Ruta de recursos
     */
    public String getResourcesPath() {
        return buildPath(EnvironmentVariables.getResourcesPath());
    }

    /**
     * Devuelve la ruta al directorio de certificados de la aplicación
     * 
     * @return Ruta de certificados
     */
    public String getCertPath() {
        return buildPath(EnvironmentVariables.getCertPath());
    }

    @Override
    public String toString() {
        return new StringBuilder(applicationCode).append(":").append(properties.toString()).toString();
    }

    /**
     * Constructor de instancias de configuración, valida y crea la instancia
     * 
     * @param appCode
     *            código de aplicación (se utilizará para construir las rutas a
     *            los recursos)
     * @param resources
     *            conjunto de recursos de propiedades (.properties)
     * @return Instancia de configuración con todas las propiedades disponibles
     */
    public static Configuration build(String appCode, String... resources) {

        if (appCode == null || StringPool.EMPTY.equals(appCode)) {
            LOGGER.error("No se ha definido el código de aplicación");
            throw new UndefinedConfigurationParametersException("applicationCode");
        }

        String[] validResources = cleanResources(resources);
        if (validResources.length == 0) {
            String defaultConfigFile = appCode.concat(".properties");
            LOGGER.debug("No se han indicado recursos de configuración, se utiliza el fichero por defecto: {}",
                    defaultConfigFile);

            return new Configuration(appCode, defaultConfigFile);
        }

        return new Configuration(appCode, validResources);
    }

    /**
     * Convierte la lista dinámica de recusos e un array, limpiando los valores
     * vacíos
     * 
     * @param resources
     *            Recursos
     * @return Arary de recursos
     */
    private static String[] cleanResources(String... resources) {
        Set<String> validResources = new HashSet<String>();
        if (resources != null) {
            for (String string : resources) {
                if (string != null && !string.isEmpty()) {
                    validResources.add(string);
                }
            }
        }

        return validResources.toArray(new String[] {});
    }
    
    /**
     * Crea la ruta a un directorio de aplicación concatenando $apps_[key]_path/
     * + codigo_aplicacion. Si la propiedad app.code no está definida devuelve
     * la ruta a rootPath
     * 
     * @param appCode Código de aplicación
     * @param rootPath Directorio raíz
     * @return Ruta de la aplicación
     */
    private String buildPath(String rootPath) {
        boolean isDirectory = rootPath.endsWith(File.separator);
        boolean endsWithSlash = applicationCode.endsWith(File.separator);

        // Añadimos directorio de aplicación y /
        String dir = isDirectory ? StringPool.EMPTY : File.separator;
        String slash = endsWithSlash ? StringPool.EMPTY : File.separator;

        return rootPath.concat(dir).concat(applicationCode).concat(slash);
    }

}
