/* amtega-common-configuration: Amtega Common Configuration
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-configuration”.
*
* "amtega-common-configuration" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-configuration" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.configuration;

import static es.xunta.amtega.transversal.configuration.StringPool.BLANK;
import static es.xunta.amtega.transversal.configuration.StringPool.COMMA;
import static es.xunta.amtega.transversal.configuration.StringPool.DOT;
import static es.xunta.amtega.transversal.configuration.StringPool.EMPTY;
import static es.xunta.amtega.transversal.configuration.StringPool.SLASH;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;
import java.util.StringTokenizer;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.configuration.exception.PropertiesSourceNotAvailableException;

/**
 * Clase que gestiona la lectura de ficheros de propiedades en las posibles
 * localizaciones: classpath, directorio de configuración o ruta en el sistema
 */
final class PropertiesLoader {

    public static final String CLASSPATH = "classpath:";
    
    private static final Logger LOGGER = LogManager.getLogger(PropertiesLoader.class);
    private static final String DOT_SLASH = DOT.concat(File.separator);
    private final String rootPath;

    public PropertiesLoader(final String rootPath) {
        this.rootPath = rootPath;
    }

    /**
     * Carga los ficheros properties indicados (es posible indicar varios
     * ficheros de configuración: classpath:fichero.properties, mail.properties)
     * 
     * @param resourceName Nombre del recurso
     * @return Propiedades contenidas en los recursos
     */
    public Properties load(String resourceName) {
        final Properties properties = new Properties();
        final String cleanResourceName = String.valueOf(resourceName).replace(BLANK, EMPTY);
        StringTokenizer tk = new StringTokenizer(cleanResourceName, COMMA);
        while (tk.hasMoreElements()) {
            String resource = (String) tk.nextElement();
            try {
                final Properties props = loadPropertiesFile(resource);
                properties.putAll(props);
            } catch (PropertiesSourceNotAvailableException e) {
                LOGGER.error("Error cargando el fichero '{}'", resource, e);
                throw e;
            }
        }
        return properties;
    }

    /**
     * Carga las propiedades contenidas en un fichero
     * 
     * @param resourceName Recurso
     * @return Propiedades contenidas en el recurso
     */
    private Properties loadPropertiesFile(String resourceName) {
        URL url = this.locateResource(resourceName);
        final Properties properties = new Properties();
        InputStream in = null;
        try {
            in = url.openStream();
            properties.load(in);
        } catch (Exception e) {
            throw new PropertiesSourceNotAvailableException("Error leyendo fichero de configuración '"+resourceName+"'", e);
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException e) {
                LOGGER.error("Error cerrando recurso '{}'", resourceName, e);
            }
        }
        return properties;
    }

    /**
     * Busca el fichero de propiedades en el classpath, si se indica, en el
     * directorio de configuración o en la ruta indicada
     * 
     * @param resourceName Recurso
     * @return Propiedades contenidas en el recurso
     */
    private URL locateResource(String resourceName) {
        LOGGER.debug("Cargando fichero de configuración '{}'", resourceName);
        URL url = null;
        if (resourceName.trim().startsWith(CLASSPATH)) {
        // Si se especifica el prefijo del classpath, buscamos sólo allí
            url = this.locateInClasspath(resourceName.replace(CLASSPATH, EMPTY));
        } else {
            // Sino, intentamos localizarlo dentro del directorio raiz indicado
            url = this.locateInRootPath(resourceName);
        if (url == null) {
                url = this.locateInAbsolutePath(resourceName);
        }
        }
        // Si no encontramos, error
        if (url == null) {
            throw new PropertiesSourceNotAvailableException("No se ha encontrado el fichero de configuración '" + resourceName + "'");
        }
        return url;
    }

    /**
     * Busca el fichero de propiedades en el classpath
     * 
     * @param resourceName Recurso
     * @return URL al recurso físico
     */
    private URL locateInClasspath(String resourceName) {
        LOGGER.debug("Buscando fichero en classpath: '{}'", resourceName);
        // Para el classloader las rutas son absolutas
        String fileName = resourceName.startsWith(SLASH) ? resourceName.substring(1) : resourceName;
        // Intentamos cargar el recurso con el classloader
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        if (loader == null) {
            throw new PropertiesSourceNotAvailableException("No se puede acceder al classloader");
        }
        return loader.getResource(fileName);
        }

    /**
     * Busca el fichero de propiedades dentro de rootPath
     * 
     * @param resourceName Recurso
     * @return URL al recurso físico
     */
    private URL locateInRootPath(String resourceName) {
        String path = this.buildPathFromRoot(resourceName);
        LOGGER.debug("Buscando fichero de configuración en '{}'", path);
        File file = new File(path);
        try {
            if (file.exists()) {
                LOGGER.debug("Cargando fichero de configuración de ruta '{}'", file.getAbsolutePath());
                return file.toURI().toURL();
            } else {
                LOGGER.warn("No existe el fichero de configuración '{}' en el directorio de configuración '{}'", resourceName, rootPath);
            }
        } catch (MalformedURLException e) {
            LOGGER.error("Error inesperado cargando fichero de configuración '{}'", file.getAbsolutePath(), e);
        }
        return null;
    }

    /**
     * Busca el fichero de propiedades asumiendo que es una ruta absoluta
     * 
     * @param resourceName Recurso
     * @return URL al recurso físico
     */
    private URL locateInAbsolutePath(String resourceName) {
        // En este método no imprimimos tanto log para no confundir al usuario final en caso de no existir el fichero en ruta absoluta
        File file = new File(resourceName);
        try {
            if (file.exists()) {
                LOGGER.warn("Encontrado fichero de configuración en '{}', cargándolo a modo de alternativa", file.getAbsolutePath());
                return file.toURI().toURL();
            }
        } catch (MalformedURLException e) {
            LOGGER.error("Error inesperado cargando fichero de configuración alternativo '{}'", file.getAbsolutePath(), e);
        }
        return null;
    }

    /**
     * Crea la ruta al fichero de forma relativa a 'rootPath'
     * 
     * @param fileName Nombre del fichero
     * @return Ruta normalizada
     */
    private String buildPathFromRoot(String fileName) {
        StringBuilder fName = new StringBuilder(rootPath);
        // Aseguramos ruta correcta
        if (!rootPath.endsWith(File.separator)) {
            fName.append(File.separator);
        }
        // Si el fichero tiene un "./" lo eliminamos
        if (fileName.startsWith(DOT_SLASH)) {
            fName.append(fileName.substring(DOT_SLASH.length()));
        } else {
            fName.append(fileName);
        }
        return fName.toString();
    }

}
