/* amtega-common-configuration: Amtega Common Configuration
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-configuration”.
*
* "amtega-common-configuration" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-configuration" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.configuration;

import static java.util.Collections.unmodifiableCollection;
import static java.util.Collections.unmodifiableSet;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

/**
 * Implementación de Properties de solo lectura y posibles valores encriptados
 */
final class UnmodifiableProperties extends Properties {
    private static final long serialVersionUID = 1L;
    private final Collection<ReloadablePropertiesSource> reloadableSources = new ArrayList<ReloadablePropertiesSource>();
    private transient Set<Object> keySet = null;
    private transient Set<Map.Entry<Object, Object>> entrySet = null;
    private transient Collection<Object> values = null;

    /**
     * Crea una instancia con los componentes necesarios para la gestión de
     * propiedades
     * 
     * @param sources
     *            fuentes que proporcionan properties de configuración
     */
    public UnmodifiableProperties(final Collection<PropertiesSource> sources) {

        if (sources == null || sources.isEmpty()) {
            throw new IllegalArgumentException("No se han indicado fuentes de properties");
        }

        for (PropertiesSource source : sources) {
            final Properties properties = source.getProperties();
            if (properties != null) {
                for (Map.Entry<Object, Object> entry : properties.entrySet()) {
                    super.put(entry.getKey(), entry.getValue());
                }

                if (source instanceof ReloadablePropertiesSource) {
                    reloadableSources.add((ReloadablePropertiesSource) source);
                }
            }
        }
    }

    @Override
    public void load(InputStream par1) throws IOException {
        throw new UnsupportedOperationException();
    }

    @Override
    public Object setProperty(String par1, String par2) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String getProperty(String key) {
        reloadIfNeeded();
        return super.getProperty(key);
    }

    @Override
    public String getProperty(String key, String defaultValue) {
        reloadIfNeeded();
        return super.getProperty(key, defaultValue);
    }

    @Override
    public void clear() {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean containsKey(Object key) {
        reloadIfNeeded();
        return super.containsKey(key);
    }

    @Override
    public boolean containsValue(Object key) {
        reloadIfNeeded();
        return super.containsValue(key);
    }

    @Override
    public Set<Map.Entry<Object, Object>> entrySet() {
        reloadIfNeeded();
        if (entrySet == null) {
            entrySet = unmodifiableSet(super.entrySet());
        }
        return entrySet;
    }

    @Override
    public Object get(Object par1) {
        reloadIfNeeded();
        return super.get(par1);
    }

    @Override
    public Enumeration<Object> keys() {
        reloadIfNeeded();
        return super.keys();
    }

    @Override
    public boolean isEmpty() {
        reloadIfNeeded();
        return super.isEmpty();
    }

    @Override
    public Set<Object> keySet() {
        reloadIfNeeded();
        if (keySet == null) {
            keySet = unmodifiableSet(super.keySet());
        }

        return keySet;
    }

    @Override
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }

    @Override
    public synchronized Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public synchronized void putAll(Map<? extends Object, ? extends Object> t) {
        throw new UnsupportedOperationException();
    }

    @Override
    public int size() {
        reloadIfNeeded();
        return super.size();
    }

    @Override
    public Collection<Object> values() {
        reloadIfNeeded();
        if (values == null) {
            values = unmodifiableCollection(super.values());
        }

        return values;
    }

    /**
     * Realiza la recarga de los recursos de tipo "recargable" si están
     * definidos
     */
    private void reloadIfNeeded() {
        for (ReloadablePropertiesSource source : reloadableSources) {
            Properties properties = source.getProperties();
            for (Map.Entry<Object, Object> entry : properties.entrySet()) {
                super.put(entry.getKey(), entry.getValue());
            }
        }
    }

}