/* amtega-common-configuration: Amtega Common Configuration
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-configuration”.
*
* "amtega-common-configuration" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-configuration" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.configuration;

import static org.junit.Assert.*;

import java.io.File;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import es.xunta.amtega.transversal.configuration.Configuration;
import es.xunta.amtega.transversal.configuration.exception.PropertiesSourceNotAvailableException;
import es.xunta.amtega.transversal.configuration.exception.UndefinedSystemPropertyException;
import es.xunta.amtega.transversal.configuration.test.CustomReloadablePropertiesSource;

public class ConfigurationTest {

    private static final String CODAPP = "cod_aplicacion";
    private static final String OTRAAPP = "otraaplicacion";

    private static final String CLASSPATH_PROPS = "classpath:classpath-configuration.properties";
    
    private static final String CODAPP_PROPERTY_KEY = "codapp.key";
    private static final String CODAPP_PROPERTY_VALUE = "codapp.value";
    private static final String RELOADABLE_PROPERTY_KEY = "my-reloadable-property-key";
    private static final String USER_PROPERTY_VALUE = "user";
    private static final String PASS_PROPERTY_VALUE = "hello world";

    @Rule
    public ExpectedException thrown = ExpectedException.none();

    @Test
    public void readSystemPropertiesTest() {
        Configuration config = Configuration.build(CODAPP);
        String expected = CODAPP.concat(File.separator);
        assertTrue(config.getConfigPath().endsWith(expected));
        assertTrue(config.getDataPath().endsWith("data-mock".concat(File.separator).concat(expected)));
        assertTrue(config.getCertPath().endsWith(expected));
        assertTrue(config.getTempPath().endsWith("temp-mock".concat(File.separator).concat(expected)));
        assertTrue(config.getResourcesPath().endsWith("resources-mock".concat(File.separator).concat(expected)));
    }

    @Test
    public void readSystemPropertiesErrorTest() {
        Configuration config = Configuration.build(CODAPP);
        thrown.expect(UndefinedSystemPropertyException.class);
        config.getLogPath();
        fail("está definida");
    }

    @Test
    public void readConfigurationDefaultTest() {
        Configuration config = Configuration.build(CODAPP);
        assertEquals(CODAPP_PROPERTY_VALUE, config.getProperty(CODAPP_PROPERTY_KEY));
    }

    @Test
    public void readConfigurationFromPropertiesStorageTest() {
        Configuration config = Configuration.build(CODAPP);
        assertEquals("custom.value", config.getProperty("custom.key"));
    }

    @Test
    public void readConfigurationFromClassPathTest() {
        Configuration config = Configuration.build(CODAPP, CLASSPATH_PROPS);
        assertNotNull(config.getProperties());
        assertEquals("classpath.value", config.getProperty("classpath.key"));
    }

    @Test
    public void readConfigurationFromConfigPathTest() {
        Configuration config = Configuration.build(CODAPP, "cod_aplicacion.properties", "myconfiguration.properties");
        assertNotNull(config.getProperties());
        assertEquals(CODAPP_PROPERTY_VALUE, config.getProperty(CODAPP_PROPERTY_KEY)); // cod_aplicacion.properties
        assertEquals("my.value", config.getProperty("my.key"));                       // myconfiguration.properties
    }

    @Test
    public void readConfigurationFromSubdirConfigPathTest() {
        Configuration config = Configuration.build(CODAPP, "subdir/insubdirectory.properties");
        assertNotNull(config.getProperties());
        assertEquals("insubdir.value", config.getProperty("insubdir.key"));
    }

    @Test
    public void readConfigurationFromConfigSubDirPathTest() {
        thrown.expect(PropertiesSourceNotAvailableException.class);
        Configuration.build(CODAPP, "otraaplicacion/otraaplicacion.properties");
        fail("No se puede acceder a recursos de otros directorios de aplicación");
    }

    @Test
    public void readNonExistingConfigurationFileTest() {
        thrown.expect(PropertiesSourceNotAvailableException.class);
        Configuration.build(CODAPP, "fichero-noexiste.properties");
        fail("El fichero no existe");
    }

    @SuppressWarnings("resource")
    @Test
    public void createBeanTest() {
        ApplicationContext context = new ClassPathXmlApplicationContext("/application-context.xml");
        Configuration config = context.getBean(Configuration.class);
        assertEquals("bean.value", config.getProperty("bean.key"));
    }

    @Test
    public void reloadablePropertiesSourceTest() {
        Configuration config = Configuration.build(CODAPP);
        CustomReloadablePropertiesSource.getInstance().setProperty(RELOADABLE_PROPERTY_KEY, "value1");
        assertEquals("Valor no esperado", config.getProperty(RELOADABLE_PROPERTY_KEY), "value1");
        //Emulamos un cambio de propiedades en la clase CustomReloadablePropertiesSource
        CustomReloadablePropertiesSource.getInstance().setProperty(RELOADABLE_PROPERTY_KEY, "value-reloaded");
        assertEquals("Valor no esperado", config.getProperty(RELOADABLE_PROPERTY_KEY), "value-reloaded");
    }

    @Test
    public void readAppUserDataTest() {
        Configuration config = Configuration.build(CODAPP);
        assertEquals(USER_PROPERTY_VALUE, config.getUsername());
        assertEquals(PASS_PROPERTY_VALUE, config.getPassword());
    }

    @Test
    public void multipleInstances() {
        Configuration config1 = Configuration.build(CODAPP);
        Configuration config2 = Configuration.build(OTRAAPP);

        assertTrue(config1.getConfigPath().endsWith(CODAPP+File.separator));
        assertTrue(config2.getConfigPath().endsWith(OTRAAPP+File.separator));
        
        assertEquals(CODAPP_PROPERTY_VALUE, config1.getProperty(CODAPP_PROPERTY_KEY));
        assertEquals("otraapp.value", config2.getProperty(CODAPP_PROPERTY_KEY));
    }
    
}
