/* amtega-common-logging-sql: Amtega Common Logging SQL
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-logging-sql".
*
* "amtega-common-logging-sql" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-logging-sql" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 */
package es.xunta.amtega.transversal.jdbc.logger;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Clase que actúa como proxy de la implementación de PreparedStatement a
 * ejecutar
 */
class PreparedStatementWrapper extends StatementWrapper implements PreparedStatement {

    private final String sql;
    private final Map<Integer,Object> params = new TreeMap<>();

    /**
     * Constructor
     *
     * @param statement Statement
     * @param sql SQL
     */
    public PreparedStatementWrapper(PreparedStatement statement, String sql) {
        super(statement);
        this.sql = sql;
    }

    /**
     * Añade un parámetro
     *
     * @param parameterIndex Índice
     * @param value Valor
     */
    protected void addParam(int parameterIndex, Object value) {
        params.put(parameterIndex, String.valueOf(value));
    }
    
    /**
     * Obtiene los valores de los parámetros como lista
     * 
     * @return Valores de parámetros
     */
    protected List<Object> getParamValues() {
        return new ArrayList<Object>(params.values());
    }

    /**
     * Escribe en el log
     */
    protected void log() {
        SQLLogger.log(getCurrentConnection(), sql, this.getParamValues(), startup);
    }

    /**
     * Escribe en el log de error
     *
     * @param error Código de error
     */
    protected void logError(String error) {
        SQLLogger.log(getCurrentConnection(), sql, this.getParamValues(), startup, error);
    }

    /**
     * Obtiene el prepared statement
     *
     * @return Statement
     */
    private PreparedStatement getPreparedStatement() {
        return (PreparedStatement) statement;
    }

    @Override
    public boolean execute() throws SQLException {
        startup = System.currentTimeMillis();
        try {
            boolean result = getPreparedStatement().execute();
            log();
            return result;
        } catch (SQLException e) {
            logError(e.getSQLState());
            throw e;
        }
    }

    @Override
    public ResultSet executeQuery() throws SQLException {
        startup = System.currentTimeMillis();
        try {
            ResultSet result = getPreparedStatement().executeQuery();
            log();
            return result;
        } catch (SQLException e) {
            logError(e.getSQLState());
            throw e;
        }
    }

    @Override
    public int executeUpdate() throws SQLException {
        startup = System.currentTimeMillis();
        try {
            int result = getPreparedStatement().executeUpdate();
            log();
            return result;
        } catch (SQLException e) {
            logError(e.getSQLState());
            throw e;
        }
    }

    @Override
    public void setNull(int parameterIndex, int sqlType) throws SQLException {
        addParam(parameterIndex, null);
        getPreparedStatement().setNull(parameterIndex, sqlType);
    }

    @Override
    public void setBoolean(int parameterIndex, boolean x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBoolean(parameterIndex, x);
    }

    @Override
    public void setByte(int parameterIndex, byte x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setByte(parameterIndex, x);
    }

    @Override
    public void setShort(int parameterIndex, short x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setShort(parameterIndex, x);
    }

    @Override
    public void setInt(int parameterIndex, int x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setInt(parameterIndex, x);
    }

    @Override
    public void setLong(int parameterIndex, long x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setLong(parameterIndex, x);
    }

    @Override
    public void setFloat(int parameterIndex, float x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setFloat(parameterIndex, x);
    }

    @Override
    public void setDouble(int parameterIndex, double x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setDouble(parameterIndex, x);
    }

    @Override
    public void setBigDecimal(int parameterIndex, BigDecimal x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBigDecimal(parameterIndex, x);
    }

    @Override
    public void setString(int parameterIndex, String x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setString(parameterIndex, x);
    }

    @Override
    public void setBytes(int parameterIndex, byte[] x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBytes(parameterIndex, x);
    }

    @Override
    public void setDate(int parameterIndex, Date x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setDate(parameterIndex, x);
    }

    @Override
    public void setTime(int parameterIndex, Time x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setTime(parameterIndex, x);
    }

    @Override
    public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setTimestamp(parameterIndex, x);
    }

    @Override
    public void setAsciiStream(int parameterIndex, InputStream x, int length) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setAsciiStream(parameterIndex, x, length);
    }

    @SuppressWarnings("deprecation")
    @Override
    public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws SQLException {
        getPreparedStatement().setUnicodeStream(parameterIndex, x, length);
    }

    @Override
    public void setBinaryStream(int parameterIndex, InputStream x, int length) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBinaryStream(parameterIndex, x, length);
    }

    @Override
    public void clearParameters() throws SQLException {
        getPreparedStatement().clearParameters();
    }

    @Override
    public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setObject(parameterIndex, x, targetSqlType);
    }

    @Override
    public void setObject(int parameterIndex, Object x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setObject(parameterIndex, x);
    }

    @Override
    public void addBatch() throws SQLException {
        getPreparedStatement().addBatch();
    }

    @Override
    public void setCharacterStream(int parameterIndex, Reader reader, int length) throws SQLException {
        getPreparedStatement().setCharacterStream(parameterIndex, reader, length);
    }

    @Override
    public void setRef(int parameterIndex, Ref x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setRef(parameterIndex, x);
    }

    @Override
    public void setBlob(int parameterIndex, Blob x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBlob(parameterIndex, x);
    }

    @Override
    public void setClob(int parameterIndex, Clob x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setClob(parameterIndex, x);
    }

    @Override
    public void setArray(int parameterIndex, Array x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setArray(parameterIndex, x);
    }

    @Override
    public ResultSetMetaData getMetaData() throws SQLException {
        return getPreparedStatement().getMetaData();
    }

    @Override
    public void setDate(int parameterIndex, Date x, Calendar cal) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setDate(parameterIndex, x, cal);
    }

    @Override
    public void setTime(int parameterIndex, Time x, Calendar cal) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setTime(parameterIndex, x, cal);
    }

    @Override
    public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setTimestamp(parameterIndex, x, cal);
    }

    @Override
    public void setNull(int parameterIndex, int sqlType, String typeName) throws SQLException {
        addParam(parameterIndex, null);
        getPreparedStatement().setNull(parameterIndex, sqlType, typeName);
    }

    @Override
    public void setURL(int parameterIndex, URL x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setURL(parameterIndex, x);
    }

    @Override
    public ParameterMetaData getParameterMetaData() throws SQLException {
        return getPreparedStatement().getParameterMetaData();
    }

    @Override
    public void setRowId(int parameterIndex, RowId x) throws SQLException {
        getPreparedStatement().setRowId(parameterIndex, x);
    }

    @Override
    public void setNString(int parameterIndex, String value) throws SQLException {
        addParam(parameterIndex, value);
        getPreparedStatement().setNString(parameterIndex, value);
    }

    @Override
    public void setNCharacterStream(int parameterIndex, Reader value, long length) throws SQLException {
        addParam(parameterIndex, value);
        getPreparedStatement().setNCharacterStream(parameterIndex, value, length);
    }

    @Override
    public void setNClob(int parameterIndex, NClob value) throws SQLException {
        addParam(parameterIndex, value);
        getPreparedStatement().setNClob(parameterIndex, value);
    }

    @Override
    public void setClob(int parameterIndex, Reader reader, long length) throws SQLException {
        addParam(parameterIndex, reader);
        getPreparedStatement().setClob(parameterIndex, reader, length);
    }

    @Override
    public void setBlob(int parameterIndex, InputStream inputStream, long length) throws SQLException {
        addParam(parameterIndex, inputStream);
        getPreparedStatement().setBlob(parameterIndex, inputStream, length);
    }

    @Override
    public void setNClob(int parameterIndex, Reader reader, long length) throws SQLException {
        addParam(parameterIndex, reader);
        getPreparedStatement().setNClob(parameterIndex, reader, length);
    }

    @Override
    public void setSQLXML(int parameterIndex, SQLXML xmlObject) throws SQLException {
        addParam(parameterIndex, xmlObject);
        getPreparedStatement().setSQLXML(parameterIndex, xmlObject);
    }

    @Override
    public void setObject(int parameterIndex, Object x, int targetSqlType, int scaleOrLength) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setObject(parameterIndex, x, targetSqlType, scaleOrLength);
    }

    @Override
    public void setAsciiStream(int parameterIndex, InputStream x, long length) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setAsciiStream(parameterIndex, x, length);
    }

    @Override
    public void setBinaryStream(int parameterIndex, InputStream x, long length) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBinaryStream(parameterIndex, x, length);
    }

    @Override
    public void setCharacterStream(int parameterIndex, Reader reader, long length) throws SQLException {
        addParam(parameterIndex, reader);
        getPreparedStatement().setCharacterStream(parameterIndex, reader, length);
    }

    @Override
    public void setAsciiStream(int parameterIndex, InputStream x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setAsciiStream(parameterIndex, x);
    }

    @Override
    public void setBinaryStream(int parameterIndex, InputStream x) throws SQLException {
        addParam(parameterIndex, x);
        getPreparedStatement().setBinaryStream(parameterIndex, x);
    }

    @Override
    public void setCharacterStream(int parameterIndex, Reader reader) throws SQLException {
        addParam(parameterIndex, reader);
        getPreparedStatement().setCharacterStream(parameterIndex, reader);
    }

    @Override
    public void setNCharacterStream(int parameterIndex, Reader value) throws SQLException {
        addParam(parameterIndex, value);
        getPreparedStatement().setNCharacterStream(parameterIndex, value);
    }

    @Override
    public void setClob(int parameterIndex, Reader reader) throws SQLException {
        addParam(parameterIndex, reader);
        getPreparedStatement().setClob(parameterIndex, reader);
    }

    @Override
    public void setBlob(int parameterIndex, InputStream inputStream) throws SQLException {
        addParam(parameterIndex, inputStream);
        getPreparedStatement().setBlob(parameterIndex, inputStream);
    }

    @Override
    public void setNClob(int parameterIndex, Reader reader) throws SQLException {
        addParam(parameterIndex, reader);
        getPreparedStatement().setNClob(parameterIndex, reader);
    }

}
