/* amtega-common-logging-sql: Amtega Common Logging SQL
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-logging-sql".
*
* "amtega-common-logging-sql" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-logging-sql" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.jdbc.logger;

import static es.xunta.amtega.transversal.jdbc.logger.Constants.BLANK;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.COMMA;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.DATE_PATTERN;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.DATE_ROW_SIZE;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.EMPTY;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.NA;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.MarkerManager;

import es.xunta.amtega.transversal.logging.exception.ResourceNotFoundException;

/**
 * Clase de utilidad para generar registros de log de SQL
 */
final class SQLLogger {

    private static final String MARKER_NAME = "SQL_LOGGER";
    private static final String PROPERTIES = "/amtega-common-logging-sql.properties";
    private static final String REPLACE_PARAMS_KEY = "replace.params";
    private static final String QUERY_EXECUTION_TIME = "query.execution.time";
    private static final String NO_ERROR = "0";
    private static final String INCLUDES_KEY = "includes";
    private static final String EXCLUDES_KEY = "excludes";
    private static final Logger LOGGER = LogManager.getLogger(SQLLogger.class);
    private static boolean replaceParams;
    private static List<String> includes = new ArrayList<String>();
    private static List<String> excludes = new ArrayList<String>();
    private static long maxExecutionTime;
    private static boolean isDebugExecutionTimeActive;

    /**
     * Constructor privado
     */
    private SQLLogger() {
        // Vacio
    }

    static {
        try {
            LOGGER.debug("Inicializando librería de log SQL");
            final Properties properties = loadConfiguration();

            final String value = properties.getProperty(REPLACE_PARAMS_KEY, Boolean.FALSE.toString());
            replaceParams = Boolean.parseBoolean(value);

            // Valores a incluir
            final String includeValue = properties.getProperty(INCLUDES_KEY, EMPTY).replace(BLANK, EMPTY);
            if (!includeValue.isEmpty()) {
                includes.addAll(Arrays.asList(includeValue.split(COMMA)));
            }

            // Métodos a excluir
            final String excludeValue = properties.getProperty(EXCLUDES_KEY, EMPTY).replace(BLANK, EMPTY);
            if (!excludeValue.isEmpty()) {
                excludes.addAll(Arrays.asList(excludeValue.split(COMMA)));
            }

            maxExecutionTime = Long.parseLong(properties.getProperty(QUERY_EXECUTION_TIME, "-1"));
            isDebugExecutionTimeActive = maxExecutionTime > -1;
        } catch (Exception e) {
            LOGGER.warn("Error cargando fichero de configuración, se utiliza configuración por defecto", e);
        }
    }

    /**
     * Escribe query en el log
     * @param connection Conexión a BD
     * @param sql Query
     * @param params Lista de parámetros de la query
     * @param startup Momento de inicio de la query
     */
    public static void log(Connection connection, String sql, List<Object> params, long startup) {
        log(connection, sql, params, startup, NO_ERROR);
    }

    /**
     * Escribe query en el log
     * @param connection Conexión a BD
     * @param sql Query
     * @param params Lista de parámetros de la query
     * @param startup Momento de inicio de la query
     * @param error Código de error
     */
    public static void log(Connection connection, String sql, List<Object> params, long startup, String error) {

        if (isLoggable()) {

            // Información de tiempo de ejecución
            final String fechaInicio = createDateInfo(startup);
            final String fechaFin = createDateInfo(System.currentTimeMillis());

            // Sql sin comentarios e info de tipo de consulta
            String cleanSql = SQLUtils.clean(sql);

            // Parámetros formateados para log
            final List<String> sqlParams = SQLUtils.createSQLParams(params);
            if (replaceParams) {
                try {
                    cleanSql = SQLUtils.replaceSQLParams(cleanSql, sqlParams);
                } catch (Exception e) {
                    LOGGER.error("Error reemplazando parámetros de consulta SQL: ", e);
                }
            }

            // Información de fila de log
            final String tipoSql = SQLUtils.getDMLType(cleanSql);

            // Mensaje de log con datos de consulta
            final AuditableMessageImpl msg = new AuditableMessageImpl();
            msg.put(AuditableMessageImpl.SQL, cleanSql);
            msg.put(AuditableMessageImpl.FECHA_INICIO, fechaInicio);
            msg.put(AuditableMessageImpl.FECHA_FIN, fechaFin);
            msg.put(AuditableMessageImpl.TIPO_SQL, tipoSql);
            msg.put(AuditableMessageImpl.PARAMETROS, sqlParams.toString());
            msg.put(AuditableMessageImpl.ERROR, error);

            try {
                String baseDatos = connection.getCatalog();
                if (baseDatos == null) {
                    final String url = connection.getMetaData().getURL();
                    baseDatos = databaseFromUrl(url);
                }
                msg.put(AuditableMessageImpl.BASE_DATOS, baseDatos);
            } catch (SQLException e) {
                LOGGER.warn("Información de base de datos no disponible", e);
                msg.put(AuditableMessageImpl.BASE_DATOS, NA);
            }

            final Marker marker = MarkerManager.getMarker(MARKER_NAME);
            LOGGER.info(marker, msg);

            long executionTime = System.currentTimeMillis() - startup;
            if (isDebugExecutionTimeActive && executionTime > maxExecutionTime) {
                LOGGER.debug(marker, "QUERY TIME {}ms: {} {}", executionTime, cleanSql, sqlParams.toString());
            }
        }
    }

    /**
     * Carga la configuración
     * @return Propiedades de configuración
     */
    private static Properties loadConfiguration() {
        final Properties properties = new Properties();
        try (InputStream in = SQLLogger.class.getResourceAsStream(PROPERTIES)) {
            LOGGER.debug("Cargando propiedades desde fichero: {}", PROPERTIES);
            properties.load(in);
        } catch (Exception e) {
            throw new ResourceNotFoundException("No se puede cargar " + PROPERTIES, e);
        }
        return properties;
    }

    /**
     * Comprueba si es necesario hacer log del método en función de la
     * configuración de include y exclude
     * 
     * @return true si es necesario hacer log de la sentencia
     */
    private static boolean isLoggable() {

        // Información de traza de ejecución
        StackTraceElement[] st = Thread.currentThread().getStackTrace();
        String trace = Arrays.toString(st);

        // Comprobar si está excluido
        for (String exclude : excludes) {
            Pattern p = Pattern.compile(exclude, Pattern.CASE_INSENSITIVE);
            Matcher m = p.matcher(trace);
            if (m.find()) {
                return false;
            }
        }

        // No hay valores de inclusión definido, se hace log de todos los
        // métodos
        if (includes.isEmpty()) {
            return true;
        }

        // Comprobar si está en includes
        for (String include : includes) {
            Pattern p = Pattern.compile(include, Pattern.CASE_INSENSITIVE);
            Matcher m = p.matcher(trace);
            if (m.find()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Formatea la información de período de tiempo para el log
     * 
     * @param timestamp Timestamp
     * @return Timestamp en el formato indicado
     */
    protected static String createDateInfo(long timestamp) {
        final DateFormat df = new SimpleDateFormat(DATE_PATTERN);
        final String startedInfo = df.format(new Date(timestamp));
        return startedInfo.substring(0, DATE_ROW_SIZE);
    }

    /**
     * Intenta obtener el nombre de base de datos desde la url de conexión
     * 
     * @param url
     *            url de conexión
     * @return nombre de base de datos
     */
    private static String databaseFromUrl(String url) {
        try {
            final String info = url.indexOf('@') > -1 ? url.split("@")[1] : url;
            String[] values = info.split(":");

            return values[values.length - 1];
        } catch (Exception e) {
            LOGGER.warn("No se ha podido recuperar la información de conexión a base de datos", e);
            return NA;
        }
    }

}
