/* amtega-common-logging-sql: Amtega Common Logging SQL
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-logging-sql".
*
* "amtega-common-logging-sql" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-logging-sql" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.jdbc.logger;

import static es.xunta.amtega.transversal.jdbc.logger.Constants.BLANK;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.DATE_PARAM_FORMAT;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.DML_UNKNOW;
import static es.xunta.amtega.transversal.jdbc.logger.Constants.EMPTY;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Scanner;
import java.util.StringTokenizer;

import es.xunta.amtega.transversal.jdbc.logger.exception.InvalidNumberOfParamsException;
import es.xunta.amtega.transversal.logging.StringPool;

/**
 * Clase de utilidades de formateo de logs
 */
final class SQLUtils {

    private static final int INDEX_NOT_FOUND = -1;
    private static final String CHAR_NEWLINE = "\n";
    private static final String CHAR_SLASH = "/";
    private static final String CHAR_BACKSLASH = "\\";
    private static final String CHAR_QUOTE = "\"";
    private static final String CHAR_ASTERISK = "*";

    private enum State {
        OUTSIDE_COMMENT, INSIDE_LINE_COMMENT, INSIDE_BLOCK_COMMENT, INSIDE_BLOCK_COMMENT_NO_NEW_LINE_YET, INSIDE_STRING
    }

    /**
     * Constructor
     */
    private SQLUtils() {
        // Vacio
    }

    /**
     * Elimina información de comentarios que añaden los ORM (Hibernate)
     *
     * @param sql sentencia a ejecutar
     * @return consulta sin comentarios
     */
    public static String clean(String sql) {
        return removeComments(sql).trim();
    }

    /**
     * Elimina los comentarios
     * @param code Código
     * @return Código sin comentarios
     */
    private static String removeComments(String code) {
        State state = State.OUTSIDE_COMMENT;
        StringBuilder result = new StringBuilder();
        Scanner s = new Scanner(code);
        s.useDelimiter(StringPool.EMPTY);
        while (s.hasNext()) {
            String c = s.next();
            if(state.equals(State.OUTSIDE_COMMENT)) {
                state = removeCommentsProcessOutsideComment(s, c, result);
            } else if(state.equals(State.INSIDE_STRING)) {
                state = removeCommentsProcessInsideString(s, c, result);
            } else if(state.equals(State.INSIDE_LINE_COMMENT)) {
                state = removeCommentsProcessInsideLineComment(c, result);
            } else if(state.equals(State.INSIDE_BLOCK_COMMENT_NO_NEW_LINE_YET)) {
                state = removeCommentsProcessInsideBlockCommentNoNewLine(s, c, result);
            } else if(state.equals(State.INSIDE_BLOCK_COMMENT)) {
                state = removeCommentsProcessInsideBlockComment(s, c);
            }
        }
        s.close();
        return result.toString();
    }
    
    /**
     * Proceso de borrado de comentarios, estado "fuera de comentarios" 
     * @param s Escaner de caracteres
     * @param c Caracter actual
     * @param result Resultado
     * @return Estado siguiente
     */
    private static State removeCommentsProcessOutsideComment(Scanner s, String c, StringBuilder result) {
        if (CHAR_SLASH.equals(c) && s.hasNext()) {
            // Se detecta un /
            String c2 = s.next();
            if (CHAR_SLASH.equals(c2)) {
                //  Detección de patrón // (comentario inline)
                return State.INSIDE_LINE_COMMENT;
            } else if (CHAR_ASTERISK.equals(c2)) {
                // Detección de patrón /* (bloque de comentario)
                return State.INSIDE_BLOCK_COMMENT_NO_NEW_LINE_YET;
            } else {
                // Otro caracter
                result.append(c).append(c2);
            }
        } else {
            // Cualquier caracter
            result.append(c);
            if (CHAR_QUOTE.equals(c)) {
                // Es un ", entramos en un strin (no hay comentarios)
                return State.INSIDE_STRING;
            }
        }
        return State.OUTSIDE_COMMENT;
    }
    
    /**
     * Proceso de borrado de comentarios, estado "dentro de un string" 
     * @param s Escaner de caracteres
     * @param c Caracter actual
     * @param result Resultado
     * @return Estado siguiente
     */
    private static State removeCommentsProcessInsideString(Scanner s, String c, StringBuilder result) {
        // Añadimos de cualquier forma el caracter
        result.append(c);
        if (CHAR_QUOTE.equals(c)) {
            // Si es ", se termina el string y volvemos al estado normal
            return State.OUTSIDE_COMMENT;
        } else if (CHAR_BACKSLASH.equals(c) && s.hasNext()) {
            // Si es un \, es que se está escapando el siguiente caracter, lo añadimos tal cual y seguimos
            result.append(s.next());
        }
        return State.INSIDE_STRING;
    }
    
    /**
     * Proceso de borrado de comentarios, estado "comentario inline" 
     * @param c Caracter actual
     * @param result Resultado
     * @return Estado siguiente
     */
    private static State removeCommentsProcessInsideLineComment(String c, StringBuilder result) {
        // Omitimos toda la línea hasta llegar al salto de línea, que volvemos a estado normal
        if (CHAR_NEWLINE.equals(c)) {
            result.append(CHAR_NEWLINE);
            return State.OUTSIDE_COMMENT;
        }
        return State.INSIDE_LINE_COMMENT;
    }
    
    /**
     * Proceso de borrado de comentarios, estado "comentario en bloque, sin nueva línea" 
     * @param s Escaner de caracteres
     * @param c Caracter actual
     * @param result Resultado
     * @return Estado siguiente
     */
    private static State removeCommentsProcessInsideBlockCommentNoNewLine(Scanner s, String c, StringBuilder result) {
        if (CHAR_NEWLINE.equals(c)) {
            // Si encontramos un salto de línea, lo añadimos y pasamos al estado "dentro del comentario en bloque"
            result.append(CHAR_NEWLINE);
            return State.INSIDE_BLOCK_COMMENT;
        }
        // Procesamos la hilera de "***"
        State state = removeCommentsProcessInsideBlockComment(s, c);
        if(state==State.OUTSIDE_COMMENT) {
            return State.OUTSIDE_COMMENT;
        }
        return State.INSIDE_BLOCK_COMMENT_NO_NEW_LINE_YET;
    }
    
    /**
     * Proceso de borrado de comentarios, estado "comentario en bloque" 
     * @param s Escaner de caracteres
     * @param c Caracter actual
     * @return Estado siguiente
     */
    private static State removeCommentsProcessInsideBlockComment(Scanner s, String c) {
        // Procesamos la hilera de "***". Si econtramos una /, salimos
        while (CHAR_ASTERISK.equals(c) && s.hasNext()) {
            String c2 = s.next();
            if (CHAR_SLASH.equals(c2)) {
                return State.OUTSIDE_COMMENT;
            }
        }
        return State.INSIDE_BLOCK_COMMENT;
    }

    /**
     * Obtiene el tipo de consulta que se va a ejecutar
     *
     * @param sql consulta en SQL
     * @return tipo de consulta
     */
    public static String getDMLType(String sql) {

        if (sql == null || EMPTY.equals(sql)) {
            return DML_UNKNOW;
        }

        int index = sql.indexOf(BLANK);
        return index == -1 ? DML_UNKNOW : sql.substring(0, index).toUpperCase();
    }

    /**
     * Formatea los objetos pasados como parámetro para mostrar en log
     *
     * @param params Parámetros objeto
     * @return Lista de parámetros en string
     */
    public static List<String> createSQLParams(List<Object> params) {

        if (params == null || params.isEmpty()) {
            return Collections.emptyList();
        }

        final List<String> formattedParams = new ArrayList<String>();
        for (Object param : params) {
            final String value = formatParam(param);
            formattedParams.add(value);
        }

        return formattedParams;
    }

    /**
     * Formatea un parámetro de tipo Object para mostrar en log
     *
     * @param param Parámetro (objeto)
     * @return Parámetro (string)
     */
    public static String formatParam(Object param) {

        if (param == null) {
            return "null";
        }

        if (param instanceof Date) {
            final DateFormat df = new SimpleDateFormat(DATE_PARAM_FORMAT);
            return new StringBuilder("'").append(df.format(param)).append("'").toString();
        }

        return new StringBuilder("'").append(param).append("'").toString();
    }

    /**
     * Realiza el reemplazo de los parámetros en la consulta sql
     *
     * @param sql Query
     * @param params Parámetros
     * @return Query con los parámetros sustituídos
     * @throws InvalidNumberOfParamsException En caso de que no estén cubiertos todos los parámetros
     */
    public static String replaceSQLParams(String sql, List<String> params) throws InvalidNumberOfParamsException {

        if (params == null || params.isEmpty()) {
            return sql;
        }

        final int paramsNumber = countMatches(sql, "?");
        if (params.size() != paramsNumber) {
            throw new InvalidNumberOfParamsException(String.format("El número de parámetros esperado (%s) es diferente al indicado (%s)", params.size(), paramsNumber));
        }

        final StringBuilder builder = new StringBuilder();
        final StringTokenizer tk = new StringTokenizer(sql, "?");
        final int tokensNumber = tk.countTokens();
        for (int i = 0; i < tokensNumber; i++) {
            String fragment = (String) tk.nextElement();
            builder.append(fragment);
            if (i < params.size()) {
                String paramValue = params.get(i);
                builder.append(paramValue);
            }
        }

        return builder.toString();
    }

    /**
     * Cuenta el número de apariciones de una cadena en el String str
     *
     * @param str Cadena
     * @param sub Subcadena
     * @return Número de apariciones
     */
    public static int countMatches(String str, String sub) {
        int count = 0;
        int idx = 0;
        while ((idx = str.indexOf(sub, idx)) != INDEX_NOT_FOUND) {
            count++;
            idx += sub.length();
        }

        return count;
    }
}
