/* amtega-common-logging-sql: Amtega Common Logging SQL
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-logging-sql".
*
* "amtega-common-logging-sql" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-logging-sql" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.jdbc.logger;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Clase que actúa como proxy de consultas Statement
 */
class StatementWrapper implements Statement {
    private static final Logger LOGGER = LogManager.getLogger(StatementWrapper.class);
    protected final Statement statement;
    protected long startup;

    /**
     * Constructor
     * @param statement Statement
     */
    public StatementWrapper(Statement statement) {
        this.statement = statement;
    }
    
    /**
     * Escribe query en el log
     * @param sql Query
     */
    protected void log(String sql) {
        SQLLogger.log(getCurrentConnection(), sql, null, startup);
    }

    /**
     * Escribe query en el log de error
     * @param sql Query
     * @param error Código de error
     */
    protected void logError(String sql, String error) {
        SQLLogger.log(getCurrentConnection(), sql, null, startup, error);
    }

    @Override
    public <T> T unwrap(Class<T> iface) throws SQLException {
        return statement.unwrap(iface);
    }

    @Override
    public ResultSet executeQuery(String sql) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            ResultSet result = statement.executeQuery(sql);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        return statement.isWrapperFor(iface);
    }

    @Override
    public int executeUpdate(String sql) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            int result = statement.executeUpdate(sql);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public void close() throws SQLException {
        statement.close();
    }

    @Override
    public int getMaxFieldSize() throws SQLException {
        return statement.getMaxFieldSize();
    }

    @Override
    public void setMaxFieldSize(int max) throws SQLException {
        statement.setMaxFieldSize(max);
    }

    @Override
    public int getMaxRows() throws SQLException {
        return statement.getMaxRows();
    }

    @Override
    public void setMaxRows(int max) throws SQLException {
        statement.setMaxRows(max);
    }

    @Override
    public void setEscapeProcessing(boolean enable) throws SQLException {
        statement.setEscapeProcessing(enable);
    }

    @Override
    public int getQueryTimeout() throws SQLException {
        return statement.getQueryTimeout();
    }

    @Override
    public void setQueryTimeout(int seconds) throws SQLException {
        statement.setQueryTimeout(seconds);
    }

    @Override
    public void cancel() throws SQLException {
        statement.cancel();
    }

    @Override
    public SQLWarning getWarnings() throws SQLException {
        return statement.getWarnings();
    }

    @Override
    public void clearWarnings() throws SQLException {
        statement.clearWarnings();
    }

    @Override
    public void setCursorName(String name) throws SQLException {
        statement.setCursorName(name);
    }

    @Override
    public boolean execute(String sql) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            boolean result = statement.execute(sql);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public ResultSet getResultSet() throws SQLException {
        return statement.getResultSet();
    }

    @Override
    public int getUpdateCount() throws SQLException {
        return statement.getUpdateCount();
    }

    @Override
    public boolean getMoreResults() throws SQLException {
        return statement.getMoreResults();
    }

    @Override
    public void setFetchDirection(int direction) throws SQLException {
        statement.setFetchDirection(direction);
    }

    @Override
    public int getFetchDirection() throws SQLException {
        return statement.getFetchDirection();
    }

    @Override
    public void setFetchSize(int rows) throws SQLException {
        statement.setFetchSize(rows);
    }

    @Override
    public int getFetchSize() throws SQLException {
        return statement.getFetchSize();
    }

    @Override
    public int getResultSetConcurrency() throws SQLException {
        return statement.getResultSetConcurrency();
    }

    @Override
    public int getResultSetType() throws SQLException {
        return statement.getResultSetType();
    }

    @Override
    public void addBatch(String sql) throws SQLException {
        statement.addBatch(sql);
    }

    @Override
    public void clearBatch() throws SQLException {
        statement.clearBatch();
    }

    @Override
    public int[] executeBatch() throws SQLException {
        startup = System.currentTimeMillis();
        try {
            int[] result = statement.executeBatch();
            log(statement.toString());
            return result;
        } catch (SQLException e) {
            logError(statement.toString(), e.getSQLState());
            throw e;
        }
    }

    @Override
    public Connection getConnection() throws SQLException {
        return statement.getConnection();
    }

    @Override
    public boolean getMoreResults(int current) throws SQLException {
        return statement.getMoreResults(current);
    }

    @Override
    public ResultSet getGeneratedKeys() throws SQLException {
        return statement.getGeneratedKeys();
    }

    @Override
    public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            int result = statement.executeUpdate(sql, autoGeneratedKeys);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public int executeUpdate(String sql, int[] columnIndexes) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            int result = statement.executeUpdate(sql, columnIndexes);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public int executeUpdate(String sql, String[] columnNames) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            int result = statement.executeUpdate(sql, columnNames);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            boolean result = statement.execute(sql, autoGeneratedKeys);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public boolean execute(String sql, int[] columnIndexes) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            boolean result = statement.execute(sql, columnIndexes);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public boolean execute(String sql, String[] columnNames) throws SQLException {
        startup = System.currentTimeMillis();
        try {
            boolean result = statement.execute(sql, columnNames);
            log(sql);
            return result;
        } catch (SQLException e) {
            logError(sql, e.getSQLState());
            throw e;
        }
    }

    @Override
    public int getResultSetHoldability() throws SQLException {
        return statement.getResultSetHoldability();
    }

    @Override
    public boolean isClosed() throws SQLException {
        return statement.isClosed();
    }

    @Override
    public void setPoolable(boolean poolable) throws SQLException {
        statement.setPoolable(poolable);
    }

    @Override
    public boolean isPoolable() throws SQLException {
        return statement.isPoolable();
    }

    @Override
    public void closeOnCompletion() throws SQLException {
        statement.closeOnCompletion();
    }

    @Override
    public boolean isCloseOnCompletion() throws SQLException {
        return statement.isCloseOnCompletion();
    }

    /**
     * Recupera la conexión actual a la base de datos
     * @return conexión a base de datos o null en caso de no poder recuperar la referencia
     */
    protected Connection getCurrentConnection() {
        try {
            return statement.getConnection();
        } catch (SQLException e) {
            LOGGER.warn("La conexión a base de datos no está disponible", e);
            // Evita modificar el comportamiento del DataSource
            return null;
        }
    }
}
