/* amtega-common-logging-sql: Amtega Common Logging SQL
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-logging-sql".
*
* "amtega-common-logging-sql" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-logging-sql" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.jdbc.logger;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.LoggerContext;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.Configuration;
import org.apache.logging.log4j.message.Message;
import org.junit.Before;
import org.junit.Test;

public class PreparedStatementWrapperTest {
    
    private static final String TEST_SQL = "select * from table where param1 = ? and param2 = ?";
    private static final String APPENDER_NAME = "CaptureSql";
    private PreparedStatement mockPreparedStatement;
    private PreparedStatementWrapper instance;
    
    public PreparedStatementWrapperTest() {
        this.addAppender();
    }
    
    @Before
    public void setUp() throws SQLException {
        // Reseteamos appender
        this.getAppender().reset();
        // Mock conexion BD
        Connection mockConnection = mock(Connection.class);
        when(mockConnection.getCatalog()).thenReturn("testBD");
        // Mock PreparedStatement
        mockPreparedStatement = mock(PreparedStatement.class);
        when(mockPreparedStatement.getConnection()).thenReturn(mockConnection);
        // Wrapper
        instance = new PreparedStatementWrapper(mockPreparedStatement, TEST_SQL);
    }

    @Test
    public void logQuery() throws SQLException {
        // given: preparamos resultado esperado
        ResultSet expected = mock(ResultSet.class);
        when(mockPreparedStatement.executeQuery()).thenReturn(expected);
        // when: establecemos valores de los parámetros
        instance.setString(1, "value1");
        instance.setString(2, "value2");
        ResultSet rs = instance.executeQuery();
        // then: el resultado es el esperado
        assertThat(rs, is(expected));
        assertThat(this.getAppender().getNumMessages(), is(1));
    }

    @Test
    public void logQueryUnorderedParams() throws SQLException {
        // given: preparamos resultado esperado
        ResultSet expected = mock(ResultSet.class);
        when(mockPreparedStatement.executeQuery()).thenReturn(expected);
        // when: establecemos valores de los parámetros
        instance.setString(2, "value2");
        instance.setString(1, "value1");
        ResultSet rs = instance.executeQuery();
        // then: el resultado es el esperado
        assertThat(rs, is(expected));
        assertThat(this.getAppender().getNumMessages(), is(1));
    }
    
    private void addAppender() {
        // Appender interceptor de SQLs
        CaptureSqlAppender appender = new CaptureSqlAppender(APPENDER_NAME);
        final Configuration config = LoggerContext.getContext(false).getConfiguration();
        config.getRootLogger().addAppender(appender, null, null);
        appender.start();
    }
    
    private CaptureSqlAppender getAppender() {
        final Configuration config = LoggerContext.getContext(false).getConfiguration();
        return (CaptureSqlAppender)config.getRootLogger().getAppenders().get(APPENDER_NAME);
    }
    
    /**
     * Appender para capturar las SQLs logadas
     */
    private static class CaptureSqlAppender extends AbstractAppender {
        private List<Message> messages = new ArrayList<>();
        public CaptureSqlAppender(final String name) {
            super(name, null, null);
        }
        @Override
        public void append(LogEvent event) {
            if("SQL_LOGGER".equals(event.getMarker().getName())) {
                messages.add(event.getMessage());
            }
        }
        public void reset() {
            this.messages = new ArrayList<>();
        }
        public int getNumMessages() {
            return this.messages.size();
        }
    }

}
