/* amtega-common-test: Amtega Test Utils
*
* Copyright (C) 2018 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-test".
*
* "amtega-common-test" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.2 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-test" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.test.util;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * Clase de utilidades varias para testing
 */
public final class AmtegaTest {

    /**
     * Constructor privado
     */
    private AmtegaTest() {

    }
    
    /**
     * Obtiene la ruta base por defecto que se utiliza en la fase de test:
     * ${project.basedir}/target/test-classes/root/
     * @return Ruta base
     */
    public static String getRootBasePath() {
        return new StringBuilder()
                .append(System.getProperty("user.dir"))
                .append(File.separator)
                .append("target")
                .append(File.separator)
                .append("test-classes")
                .append(File.separator)
                .append("root")
                .append(File.separator)
                .toString();
    }

    /**
     * Establece las variables de entorno clásicas de la Amtega, para artefactos
     * que accedan directamente a ellas a través de System.getProperty() en
     * lugar de amtega-common-configuration (que tiene un mecanismo de mock)
     */
    public static void setTestEnvironmentVariables() {
        setTestEnvironmentVariables(getRootBasePath());
    }

    /**
     * Establece las variables de entorno clásicas de la Amtega, para artefactos
     * que accedan directamente a ellas a través de System.getProperty() en
     * lugar de amtega-common-configuration (que tiene un mecanismo de mock)
     * @see #setTestEnvironmentVariables()
     * @param rootPath Ruta base de los directorios Amtega
     */
    public static void setTestEnvironmentVariables(String rootPath) {
        String path = (rootPath.endsWith(File.separator)) ? rootPath : rootPath.concat(File.separator);
        System.setProperty("apps_base_path", path);
        System.setProperty("apps_env", AmtegaTestConstants.ENVIRONMENT);
        System.setProperty("jboss.server.name", AmtegaTestConstants.SERVER_NAME);
    }

    /**
     * Limpia las variables de entorno clásicas de la Amtega
     * @see #setTestEnvironmentVariables()
     */
    public static void cleanTestEnvironmentVariables() {
        System.setProperty("apps_base_path", "");
        System.setProperty("apps_env", "");
        System.setProperty("jboss.server.name", "");
    }

    /**
     * Fuerza la llamada a un constructor privado
     * @param clazz Clase
     * @param <T> Clase
     * @return Instancia
     */
    public static <T> T callPrivateConstructor(Class<T> clazz) {
        // Los catch de las excepciones van de una en una por compatibilidad con Java 6
        try {
            Constructor<T> constructor = getConstructorPublic(clazz);
            return constructor.newInstance(new Object[0]);
        } catch (InstantiationException e) {
            throw new UnsupportedOperationException(e);
        } catch (IllegalAccessException e) {
            throw new UnsupportedOperationException(e);
        } catch (IllegalArgumentException e) {
            throw new UnsupportedOperationException(e);
        } catch (InvocationTargetException e) {
            throw new UnsupportedOperationException(e);
        }
    }

    /**
     * Obtiene el constructor
     * @param clazz Clase
     * @return Instancia
     */
    private static <T> Constructor<T> getConstructorPublic(Class<T> clazz) {
        // Los catch de las excepciones van de una en una por compatibilidad con Java 6
        try {
            Constructor<T> constructor = clazz.getDeclaredConstructor(new Class[0]);
            constructor.setAccessible(true);
            return constructor;
        } catch (NoSuchMethodException e) {
            throw new UnsupportedOperationException(e);
        } catch (SecurityException e) {
            throw new UnsupportedOperationException(e);
        }
    }

}
