/* amtega-common-test: Amtega Test Utils
*
* Copyright (C) 2018 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-common-test".
*
* "amtega-common-test" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.2 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-common-test" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.test.util;

import java.util.Calendar;
import java.util.Date;

import javax.xml.datatype.XMLGregorianCalendar;

/**
 * Clase de utilidades para fechas
 */
public final class DateTestUtil {

    /**
     * Constructor privado
     */
    private DateTestUtil() {
        // Vacio
    }

    /**
     * Construye una fecha
     * @param year Año
     * @param month Mes
     * @param date Día
     * @return Fecha
     */
    public static Date createDate(int year, int month, int date) {
        return createDate(year, month, date, 0, 0, 0);
    }

    /**
     * Construye una fecha
     * @param year Año
     * @param month Mes
     * @param date Día
     * @param hour Hora (24h)
     * @param minute Minuto
     * @param second Segundo
     * @return Fecha
     */
    public static Date createDate(int year, int month, int date, int hour, int minute, int second) {
        return createDate(year, month, date, hour, minute, second, 0);
    }

    /**
     * Construye una fecha
     * @param year Año
     * @param month Mes
     * @param date Día
     * @param hour Hora (24h)
     * @param minute Minuto
     * @param second Segundo
     * @param milisecond Milisegundos
     * @return Fecha
     */
    public static Date createDate(int year, int month, int date, int hour, int minute, int second, int milisecond) {
        Calendar c = Calendar.getInstance();
        c.set(Calendar.AM_PM, Calendar.AM);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        c.set(Calendar.DATE, date);
        c.set(Calendar.HOUR, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, second);
        c.set(Calendar.MILLISECOND, milisecond);
        return c.getTime();
    }

    /**
     * Añade a una fecha dada
     * @param date Fecha
     * @param field Campo (según java.util.Calendar)
     * @param amount Cantidad a sumar
     * @return Fecha modificada
     */
    public static Date add(Date date, int field, int amount) {
        if(date==null) {
            return null;
        }
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(field, amount);
        return c.getTime();
    }

    /**
     * Redondea a las 23:59:59 o 00:00:00, dependiendo de cual sea más cercano
     * @param date Fecha
     * @return Fecha redondeada
     */
    public static Date roundTime(Date date) {
        if(date==null) {
            return null;
        }
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int hour = c.get(Calendar.HOUR_OF_DAY);
        boolean roundUp = hour >= 12;
        return roundTime(date, roundUp);
    }

    /**
     * Redondea una fecha
     * @param date Fecha
     * @param roundUp "true": redondea hacia arriba (23:59:59), "false": redondea hacia abajo (00:00:00)
     * @return Fecha redondeada
     */
    public static Date roundTime(Date date, boolean roundUp) {
        if(date==null) {
            return null;
        }
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        if(roundUp) {
            c.set(Calendar.HOUR_OF_DAY, 23);
            c.set(Calendar.MINUTE, 59);
            c.set(Calendar.SECOND, 59);
            c.set(Calendar.MILLISECOND, 999);
        } else {
            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            c.set(Calendar.MILLISECOND, 0);
        }
        return c.getTime();
    }

    /**
     * Comprueba que un XMLGregorianCalendar y un Date representan la misma fecha
     * @param xmlGregorianCalendar Fecha en formato XMLGregorianCalendar
     * @param date Fecha en formato Date
     * @return True si representan la misma fecha, o ambos son nulos
     */
    public static boolean equals(XMLGregorianCalendar xmlGregorianCalendar, Date date) {
        if(xmlGregorianCalendar==null && date==null) {
            return true;
        }
        if(xmlGregorianCalendar==null || date==null) {
            return false;
        }
        return xmlGregorianCalendar.toGregorianCalendar().getTimeInMillis() == date.getTime();
    }

}
