/* amtega-utils-facade: Biblioteca de utilidades comunes para aplicaciones Amtega
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "amtega-utils-facade”.
*
* "amtega-utils-facade" is free software: you can redistribute it and/or modify
* it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "amtega-utils-facade" is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.transversal.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import es.xunta.amtega.transversal.authentication.Account;
import es.xunta.amtega.transversal.authentication.AuthenticationManager;
import es.xunta.amtega.transversal.authentication.exception.AccountNotFoundException;
import es.xunta.amtega.transversal.authentication.exception.CommunicationException;
import es.xunta.amtega.transversal.authentication.exception.InvalidLoginCredentialsException;
import es.xunta.amtega.transversal.authentication.exception.MultipleAccountsMatchException;

/**
 * Clase de acceso a funcionalidades comunes, realiza la auto-configuración de las librerías de configuración y autenticación
 */
public final class BaseUtils {
    private static final Logger LOGGER = LogManager.getLogger(BaseUtils.class);
    private static final String PROPERTIES = "/amtega-common-configuration.properties";

    private BaseUtils() {
        // Vacío
    }

    static {
        LOGGER.debug("Inicializando librería de utilidades");
    }

    /**
     * Conjunto de funcionalidades para trabajar con propiedades de configuración
     */
    public static final class Configuration {

        private static es.xunta.amtega.transversal.configuration.Configuration config;

        private Configuration() {
            // Vacío
        }

        static {
            // Lee fichero de propiedades de librería y crea instancia de configuración
            final Properties properties = new Properties();

            InputStream in = null;
            try {
                in = BaseUtils.class.getResourceAsStream(PROPERTIES);
                properties.load(in);
            } catch (IOException e) {
                LOGGER.error("Error cargando fichero de configuración: '{}'", PROPERTIES, e);
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (Exception e) {
                        // ignore error
                        LOGGER.error("Error al leer el fichero de configuración", e);
                    }
                }
            }

            final String appCode = properties.getProperty("app.code");
            final String resources = properties.getProperty("configuration.file");

            config = es.xunta.amtega.transversal.configuration.Configuration.build(appCode, resources);
        }

        /**
         * Devuelve el código de aplicación definido
         * @return código de aplicación
         */
        public static String getApplicationCode() {
            return config.getApplicationCode();
        }

        /**
         * Devuelve todas las propiedades definidas en los ficheros de configuración
         * @return instancia de properties
         */
        public static Properties getProperties() {
            return config.getProperties();
        }

        /**
         * Recupera una propiedad identificada por key.
         * @param key clave
         * @return valor
         */
        public static String getProperty(String key) {
            return config.getProperty(key);
        }

        /**
         * Recupera una propiedad identificada por key.
         * @param key clave
         * @return valor
         */
        public static String getEncryptedProperty(String key) {
            throw new UnsupportedOperationException();
        }

        /**
         * Recupera una propiedad identificada por key
         * @see #getProperty
         * @param key clave
         * @param defaultValue valor por defecto
         * @return valor de la clave, si no existe devuelve el valor defaultValue
         */
        public static String getProperty(String key, String defaultValue) {
            return config.getProperty(key, defaultValue);
        }

        /**
         * Recupera una propiedad identificada por key
         * @see #getProperty
         * @param key clave
         * @param defaultValue valor por defecto
         * @return valor de la clave, si no existe devuelve el valor defaultValue
         */
        public static boolean getProperty(String key, boolean defaultValue) {
            return config.getProperty(key, defaultValue);
        }

        /**
         * Recupera una propiedad identificada por key
         * @see #getProperty
         * @param key clave
         * @param defaultValue valor por defecto
         * @return valor de la clave, si no existe devuelve el valor defaultValue
         */
        public static int getProperty(String key, int defaultValue) {
            return config.getProperty(key, defaultValue);
        }

        /**
         * Recupera una propiedad identificada por key
         * @see #getProperty
         * @param key clave
         * @param defaultValue valor por defecto
         * @return valor de la clave, si no existe devuelve el valor defaultValue
         */
        public static long getProperty(String key, long defaultValue) {
            return config.getProperty(key, defaultValue);
        }

        /**
         * Recupera una propiedad identificada por key
         * @see #getProperty
         * @param key clave
         * @param defaultValue valor por defecto
         * @return valor de la clave, si no existe devuelve el valor defaultValue
         */
        public static double getProperty(String key, double defaultValue) {
            return config.getProperty(key, defaultValue);
        }

        /**
         * Devuelve el usuario de la cuenta de servicio de la aplicación
         * @return Usuario
         */
        public static String getUsername() {
            return config.getUsername();
        }

        /**
         * Devuelve la contraseña de la cuenta de servicio de la aplicación
         * @return Contraseña
         */
        public static String getPassword() {
            return config.getPassword();
        }

        /**
         * Devuelve la ruta al directorio de configuración de la aplicación
         * @return ruta al directorio de configuración
         */
        public static String getConfigPath() {
            return config.getConfigPath();
        }

        /**
         * Devuelve la ruta al directorio de logs de la aplicación
         * @return ruta al directorio de logs
         */
        public static String getLogPath() {
            return config.getLogPath();
        }

        /**
         * Devuelve la ruta al directorio de temporales
         * @return rutal al directorio de temporales
         */
        public static String getTempPath() {
            return config.getTempPath();
        }

        /**
         * Devuelve la ruta al directorio de datos
         * @return ruta al directorio
         */
        public static String getDataPath() {
            return config.getDataPath();
        }

        /**
         * Devuelve la ruta al directorio de recursos
         * @return ruta al directorio de recursos
         */
        public static String getResourcesPath() {
            return config.getResourcesPath();
        }

        /**
         * Devuelve la ruta al directorio de certificados
         * @return ruta al directorio de certificados
         */
        public static String getCertPath() {
            return config.getCertPath();
        }

        @Override
        public String toString() {
            return config.toString();
        }

    }
    
    /**
     * Conjunto de funcionalidades de autenticación
     */
    public static final class Authentication {

        private static AuthenticationManager auth = new AuthenticationManager(Configuration.getProperties());

        private Authentication() {
            // Vacío
        }

        /**
         * Realiza la operación de login con los parámetros indicados
         * @param username el código del usuario a loguear
         * @param password la contraseñal del usuario a loguear
         * @throws InvalidLoginCredentialsException error de autenticación de usuario
         * @throws CommunicationException error de comunicación con el proveedor de autenticación
         */
        public static synchronized void login(String username, String password) throws InvalidLoginCredentialsException, CommunicationException {
            auth.login(username, password);
        }

        /**
         * Recupera una cuenta de usuario
         * @param username el código del usuario a buscar
         * @return instancia de cuenta de usuario
         * @throws AccountNotFoundException la cuenta no existe
         * @throws MultipleAccountsMatchException se han encontrado varias cuentas para el mismo usuario
         * @throws CommunicationException error de comunicación con el proveedor de autenticación
         */
        public static synchronized Account findAccount(String username) throws AccountNotFoundException, MultipleAccountsMatchException, CommunicationException {
            return auth.findAccount(username);
        }

        /**
         * Realiza una búsqueda en el proveedor de autenticación
         * @param query consulta a realizar en el proveedor de autenticación (ej: cn=usuario)
         * @return lista de cuentas encontradas
         * @throws CommunicationException error de comunicación con el proveedor de autenticación
         */
        public static synchronized List<Account> findAccounts(String query) throws CommunicationException {
            return auth.findAccounts(query);
        }
    }

}
