/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.prepost.PreAuthorize;

import es.xunta.amtega.codapp.documentos.exception.DocumentoNoEncontradoException;
import es.xunta.amtega.codapp.documentos.exception.DocumentoYaExisteException;

/**
 * Define o servizo de acceso á libraría de documentos.
 */
public interface BibliotecaDocumentosService {

    /**
     * Busca un <code>Documento</code> polo seu identificador interno.
     *
     * @param id Identificador interno do documento.
     * @return Datos do documento.
     * @throws DocumentoNoEncontradoException En caso de non atopar o documento.
     */
    Documento buscar(long id);

    /**
     * Busca un <code>Documento</code> a partir do seu UUID.
     *
     * @param uuid Identificador UUID do documento.
     * @return Datos do documento.
     * @throws DocumentoNoEncontradoException En caso de non atopar o documento.
     */
    Documento buscar(String uuid);

    /**
     * Busca documentos na biblioteca cuxos títulos conteñan un texto determinado.
     * A busca non é sensible a maiúsculas/minúsculas.
     * O resultado devólvese de xeito paxinado.
     *
     * @param query Texto a buscar nos títulos.
     * @param pageable Configuración da paxinación e ordeación.
     * @return Páxina cos resultados da busca.
     */
    Page<Documento> buscar(String query, Pageable pageable);

    /**
     * Busca documentos na biblioteca en base a un filtro determinado.
     * O resultado devólvese de xeito paxinado.
     *
     * @param filter Filtro a aplicar na busca.
     * @param pageable Configuración da paxinación e ordeación.
     * @return Páxina cos resultados da busca.
     */
    Page<Documento> buscar(DocumentoFilter filter, Pageable pageable);

    /**
     * Busca todos os documentos da biblioteca.
     * O resultado devólvese de xeito paxinado.
     *
     * @param pageable Configuración da paxinación e ordeación.
     * @return Páxina cos resultados da busca.
     */
    Page<Documento> buscar(Pageable pageable);

    /**
     * Actualiza un <code>Documento</code> na base de datos e
     * persiste o InputStream no sistema de ficheiros.
     *
     * @param documento Documento a gardar.
     * @throws DocumentoYaExisteException Se xa existe un documento co mesmo título.
     */
    @PreAuthorize("hasAuthority('MODIFICAR_DOCUMENTO')")
    void actualizar(Documento documento) throws DocumentoYaExisteException;

    /**
     * Crea e inicializa un novo <code>Documento</code>.
     *
     * @param documento Documento a gardar.
     * @throws DocumentoYaExisteException Se xa existe un documento co mesmo título.
     */
    @PreAuthorize("hasAuthority('ALTA_DOCUMENTO')")
    void crear(Documento documento) throws DocumentoYaExisteException;

    /**
     * Elimina un <code>Documento</code> a partir do seu identificador interno.
     *
     * @param id Identificador do documento.
     */
    @PreAuthorize("hasAuthority('BORRAR_DOCUMENTO')")
    void borrar(long id);

}
