/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos;

import java.util.UUID;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.modelmapper.Condition;
import org.modelmapper.ModelMapper;
import org.modelmapper.PropertyMap;
import org.modelmapper.spi.MappingContext;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import es.xunta.amtega.codapp.documentos.exception.DocumentoNoEncontradoException;
import es.xunta.amtega.codapp.documentos.exception.DocumentoYaExisteException;
import es.xunta.amtega.codapp.documentos.listener.DocumentoCreadoEvent;
import es.xunta.amtega.codapp.documentos.listener.DocumentoEliminadoEvent;
import es.xunta.amtega.codapp.documentos.listener.DocumentoModificadoEvent;
import es.xunta.amtega.transversal.archam.core.persistence.PredicateUtils;

/**
 * Implementación do servizo de acceso á libraría de documentos.
 */
@Service
@Transactional(readOnly = true)
class BibliotecaDocumentosServiceImpl implements BibliotecaDocumentosService {
    
    private static final Logger LOGGER = LogManager.getLogger(BibliotecaDocumentosService.class);
    private final DocumentoRepository documentoRepository;
    private final ApplicationEventPublisher publisher;

    /**
     * Constructor do servizo.
     *
     * @param documentoRepo Repositorio de documentos.
     * @param publisher Xestor de eventos.
     */
    public BibliotecaDocumentosServiceImpl(DocumentoRepository documentoRepo, ApplicationEventPublisher publisher) {
        this.documentoRepository = documentoRepo;
        this.publisher = publisher;
    }

    @Override
    @Transactional(readOnly = false)
    public void crear(Documento documento) throws DocumentoYaExisteException {
        // Comprobamos que non existe outro documento co mesmo título
        assertTituloValido(documento);
        
        // Comprobamos que ven co ID baleiro
        if (documento.getId() != null) {
            throw new IllegalArgumentException("O documento xa ten un ID asignado: debe empregarse o método 'actualizar'");
        }

        // Aseguramos que o documento terá un identificador para a súa descarga (UUID)
        documento.setUuid(UUID.randomUUID().toString());
        
        // Gardamos
        documentoRepository.save(documento);
        
        // Enviamos notificación do evento aos posibles "EventListener"
        publisher.publishEvent(new DocumentoCreadoEvent(documento));
    }

    @Override
    @Transactional(readOnly = false)
    @CacheEvict(cacheNames = "documento-cache", key = "#documento.uuid")
    public void actualizar(Documento documento) throws DocumentoYaExisteException {
        // Comprobamos que non existe outro documento co mesmo título
        assertTituloValido(documento);
        
        // Comprobamos que se especificou o ID do documento a actualizar
        if (documento.getId() == null) {
            throw new IllegalArgumentException("O documento non está inicializado: de querer crealo, debe empregarse o método 'crear'");
        }

        // Ao non gardar datos en sesión, o contido do documento só ten
        // os datos especificados no formulario. Necesitamos recuperar o rexistro actual
        // e modificar só os valores dos campos actualizables.
        Documento entidad = documentoRepository.findOne(documento.getId());
        map(documento, entidad);

        // Gardamos
        documentoRepository.save(entidad);
        
        // Enviamos notificación do evento aos posibles "EventListener"
        publisher.publishEvent(new DocumentoModificadoEvent(documento));
        
        // Establecemos o UUID no obxeto que se pasou por parámetro, xa que pode chegar baleiro,
        // e o precisamos para que funcione correctamente '@CacheEvict'.
        // Idealmente a busca no repositorio e na caché deberan rexisrse polo mesmo atributo (UUID ou ID interno).
        documento.setUuid(entidad.getUuid());

        LOGGER.info("Gardado documento correctamente: {}", entidad);
    }

    @Override
    public Documento buscar(long id) {
        Documento documento = documentoRepository.findOne(id);
        assertExists(documento, "O documento con id '" + id + "' non existe");
        return documento;
    }

    @Override
    @Cacheable(cacheNames = "documento-cache", key = "#uuid")
    public Documento buscar(String uuid) {
        Documento documento = documentoRepository.findOneByUuid(uuid);
        assertExists(documento, uuid);
        return documento;
    }

    @Override
    public Page<Documento> buscar(String query, Pageable pageable) {
        return documentoRepository.findByTituloLikeIgnoreCase(PredicateUtils.percentConcat(query), pageable);
    }

    @Override
    public Page<Documento> buscar(Pageable pageable) {
        return documentoRepository.findAll(pageable);
    }

    @Override
    public Page<Documento> buscar(DocumentoFilter filter, Pageable pageable) {
        return documentoRepository.findAll(DocumentoSpecifications.filterBy(filter), pageable);
    }

    @Override
    @Transactional(readOnly = false)
    @CacheEvict(cacheNames = "documento-cache", key = "#uuid")
    public void borrar(long id) {
        // Comprobamos que existe o documento
        Documento documento = documentoRepository.findOne(id);
        assertExists(documento, String.valueOf(id));
        
        // Borramos
        documentoRepository.delete(documento);
        String path = documento.getPath();

        // Publicamos evento
        publisher.publishEvent(new DocumentoEliminadoEvent(documento));

        LOGGER.info("Eliminouse o documento na ruta '{}'", path);
    }

    /**
     * Comprueba que o documento existe (!=null)
     *
     * @param documento Documento a validar
     * @param text Texto a amosar na excepción
     * @throws DocumentoNoEncontradoException Erro que se lanza en caso de que o documento sexa nulo
     */
    private void assertExists(Documento documento, String text) {
        if (documento == null) {
            throw new DocumentoNoEncontradoException(text);
        }
    }

    /**
     * Comproeba que o título do documento non exista xa en base de datos
     * asociado a outro documento.
     *
     * @param documento Documento validar
     * @throws DocumentoYaExisteException
     *      En caso de que xa exista outro documento con ese título.
     */
    private void assertTituloValido(Documento documento) throws DocumentoYaExisteException {
        // Comprobamos que se especificou un título
        if (documento.getTitulo() == null || "".equals(documento.getTitulo())) {
            throw new IllegalArgumentException("O título do documento non é válido");
        }
        
        Documento documentoBD = documentoRepository.findOneByTitulo(documento.getTitulo());
        if (documentoBD != null && (documento.getId() == null
                || (documento.getId() != null && documento.getId().compareTo(documentoBD.getId()) != 0))) {
            // Se se trata dun documento novo, ou un xa existente distinto ao que estamos validando.
            throw new DocumentoYaExisteException(documento.getTitulo());
        }
    }

    /**
     * Define como se realiza a copia de valores entre un documento orixe e destino
     * @param source Documento orixe
     * @param target Documento onde copiar (o da base de datos)
     */
    private void map(Documento source, Documento target) {
        ModelMapper mapper = new ModelMapper();
        //Declaramos un PropertyMap para documentos
        //skip() Marca campos no actualizables
        //con notNull solo se actualizan si se indican nuevos valores
        mapper.addMappings(new PropertyMap<Documento, Documento>() {
            @Override
            protected void configure() {
                skip().setUuid(null);
                skip().setFechaCreacion(null);
                skip().setUsuarioCreacion(null);
                skip().setUsuarioUltimaModificacion(null);
                skip().setFechaUltimaModificacion(null);
                Condition<String, String> notNull = new Condition<String, String>() {
                    @Override
                    public boolean applies(MappingContext<String, String> ctx) {
                        return ctx.getSource() != null;
                    }
                };
                when(notNull).map().setNombre(source.getNombre());
                when(notNull).map().setPath(source.getPath());
                when(notNull).map().setFormato(source.getFormato());
            }
        });
        mapper.map(source, target);
    }

}
