/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos.listener;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;
import org.springframework.transaction.event.TransactionPhase;
import org.springframework.transaction.event.TransactionalEventListener;

import es.xunta.amtega.codapp.documentos.Documento;
import es.xunta.amtega.transversal.archam.core.io.StorageManager;

/**
 * Exemplo de uso de "EventListener" de Spring. Esta clase recibe as
 * notificacións de eventos de creación, actualización e eliminación de
 * documentos. Estes eventos se disparan dende a capa de servizo da biblioteca
 * de documentos.
 *
 * O uso de eventos na aplicación permite engadir novos comportamentos na
 * aplicación únicamente engadindo novas clases. Estas reciben os eventos
 * desexados e realizan tarefas de forma desacoplada:
 * notificación por email, log, rexistro na base de datos, etc.
 *
 * Permite ademáis comunicar distintas partes da aplicación de forma
 * transparente, evitando dependencias fortes entre as distintas
 * funcionalidades. Esta clase pode empregarse nun paquete doutra
 * funcionalidade que requira recibir esta información para realizar algunha
 * tarefa relacionada sobre o rexistro afectado.
 *
 * A anotación @TransactionalEventListener asegura que únicamente se invocarán
 * estes métodos se a transacción finaliza cun resultado "success".
 * Este comportamento pódese modificarindicando a "TransactionPhase" desexada.
 */
@Component
class BibliotecaDocumentosEventListener {

    private static final Logger LOGGER = LogManager.getLogger(BibliotecaDocumentosEventListener.class);
    private final StorageManager storageManager;

    /**
     * Constructor
     *
     * @param storageManager Xestor de almacenamento de ficheiros
     */
    public BibliotecaDocumentosEventListener(StorageManager storageManager) {
        this.storageManager = storageManager;
    }

    /**
     * Manexa o evento de creación dun documento
     *
     * @param event Evento
     */
    @TransactionalEventListener
    public void handle(DocumentoCreadoEvent event) {
        // Neste caso simplemente deixamos constancia no log
        Documento documento = event.getSource();
        LOGGER.info("Creouse un novo documento con ID '{}'", documento.getId());
    }

    /**
     * Manexa o evento de actualización dun documento
     *
     * @param event Evento
     */
    @TransactionalEventListener
    public void handle(DocumentoModificadoEvent event) {
        // Neste caso simplemente deixamos constancia no log
        Documento documento = event.getSource();
        LOGGER.info("Actualizouse o documento con ID '{}'", documento.getId());
    }

    /**
     * Manexa o evento de eliminación dun documento,
     * cando este finaliza correctamente
     *
     * @param event Evento
     */
    @TransactionalEventListener
    public void handle(DocumentoEliminadoEvent event) {
        Documento documento = event.getSource();
        // Eliminamos fichero del repositorio de datos
        storageManager.delete(documento.getPath());
        LOGGER.info("Eliminouse o documento con ID '{}'", documento.getId());
    }

    /**
     * Manexa o evento de eliminación dun documento,
     * cando este NON finaliza correctamente
     *
     * @param event Evento
     */
    @TransactionalEventListener(phase = TransactionPhase.AFTER_ROLLBACK)
    public void handleFail(DocumentoEliminadoEvent event) {
        // Houbo un erro ao eliminar, deixamos constancia no log
        Documento documento = event.getSource();
        LOGGER.warn("Non se puido eliminar o documento con ID '{}'", documento.getId());
    }

}
