/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos.web;

import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.redirectTo;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.TypeMismatchException;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.SortDefault;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import es.xunta.amtega.codapp.documentos.BibliotecaDocumentosService;
import es.xunta.amtega.codapp.documentos.Documento;
import es.xunta.amtega.codapp.documentos.DocumentoFilter;
import es.xunta.amtega.codapp.documentos.exception.DocumentoNoEncontradoException;
import es.xunta.amtega.codapp.documentos.exception.DocumentoYaExisteException;
import es.xunta.amtega.transversal.archam.core.exception.InternalErrorException;
import es.xunta.amtega.transversal.archam.core.io.StorageManager;
import es.xunta.amtega.transversal.archam.core.web.mvc.MessageHelper;
import es.xunta.amtega.transversal.archam.core.web.util.MultipartFileUtils;

/**
 * Controlador para a xestión da biblioteca de documentos.
 */
@Controller
@RequestMapping("/documentos")
public class BibliotecaDocumentosController {

    private static final Logger LOGGER = LogManager.getLogger(BibliotecaDocumentosController.class);
    private static final String LIST_VIEW = "documentos/list";
    private static final String DETAILS_VIEW = "documentos/details";
    private static final String FIELD_TITULO = "titulo";
    private static final String FIELD_NOMBRE = "nombre";

    private final BibliotecaDocumentosService bibliotecaDocumentosService;
    private final StorageManager storageManager;

    /**
     * Constructor.
     * 
     * @param bibliotecaDocumentosService Servizo de documentos.
     * @param storageManager Xestor de documentos
     */
    public BibliotecaDocumentosController(BibliotecaDocumentosService bibliotecaDocumentosService, StorageManager storageManager) {
        this.bibliotecaDocumentosService = bibliotecaDocumentosService;
        this.storageManager = storageManager;
    }
    
    /**
     * Permite o mapeo de só determinados campos do do formulario de documento.
     * 
     * @param dataBinder O <code>WebDataBinder</code>.
     */
    @InitBinder("documento")
    public void initDocumentoBinding(WebDataBinder dataBinder) {
        // Só permitimos Binding destes atributos
        dataBinder.setAllowedFields(FIELD_TITULO, "descripcion", "version");
    }

    /**
     * Permite o mapeo de só determinados campos do do formulario de
     * busca avanzada.
     * 
     * @param dataBinder O <code>WebDataBinder</code>.
     */
    @InitBinder("documentoFilter")
    public void initDocumentoFilterBinding(WebDataBinder dataBinder) {
        // Só permitimos Binding destes atributos
        dataBinder.setAllowedFields(FIELD_TITULO, FIELD_NOMBRE, "formato");
    }

    /**
     * Carga o buscador de documentos.
     *
     * @param model Modelo coas variables da vista.
     * @param pageable Criterios de paxinación e ordeación.
     * @return Vista do buscador de documentos.
     */
    @GetMapping
    public String list(ModelMap model, @SortDefault(FIELD_TITULO) Pageable pageable) {

        LOGGER.info("Cargando a vista do buscador de documentos.");

        model.addAttribute("page", bibliotecaDocumentosService.buscar(pageable));
        model.addAttribute("filter", new DocumentoFilter());

        LOGGER.info("Finalizada a carga da vista do buscador de documentos.");

        return LIST_VIEW;
    }

    /**
     * Realiza unha busca sinxela de documentos.
     *
     * @param model Modelo coas variables da vista.
     * @param pageable Criterios de paxinación e ordeación.
     * @param query Texto a buscar entre os títulos dos documentos.
     * @return Vista para a lista de documentos.
     */
    @GetMapping("/search")
    public String search(ModelMap model, @SortDefault(FIELD_TITULO) Pageable pageable,
            @RequestParam(defaultValue = "") String query) {

        LOGGER.info("Realizando unha busca sencilla de documentos cos parámetros: {}", query);

        model.addAttribute("page", bibliotecaDocumentosService.buscar(query, pageable));

        LOGGER.info("Finalizada a busca sencilla de documentos");

        return LIST_VIEW;
    }

    /**
     * Realiza unha busca avanzada de documentos.
     *
     * @param model Modelo coas variables da vista.
     * @param pageable Criterios de paxinación e ordeación.
     * @param filter Criterios de busca.
     * @return Vista para a lista de documentos.
     */
    @GetMapping("/advanced-search")
    public String search(ModelMap model, @SortDefault(FIELD_TITULO) Pageable pageable,
            @ModelAttribute DocumentoFilter filter) {

        LOGGER.info("Realizando unha busca avanzada de documentos cos parámetros: {}", filter);

        model.addAttribute("page", bibliotecaDocumentosService.buscar(filter, pageable));

        LOGGER.info("Finalizada a busca avanzada de documentos");

        return LIST_VIEW;
    }

    /**
     * Amosa o formulario para dar de alta un novo documento.
     *
     * @param model Modelo coas variables da vista.
     * @return Vista do formulario de detalle.
     */
    @GetMapping("/add")
    public String add(ModelMap model) {
        // Engadimos á vista un obxecto 'Documento' baleiro, a cubrir polo usuario
        model.addAttribute(new Documento());
        return DETAILS_VIEW;
    }

    /**
     * Recibe o formulario enviado polo usuario para dar de alta un documento.
     *
     * @param documento Documento a crear.
     * @param result Resultado de las validaciones efectuadas.
     * @param ra Parámetros de redirección (para amosar erros de validación).
     * @param file Ficheiro asociado ao documento.
     * @return En caso de éxito, vista de detalle do documento. En caso de erro, voltamos amosar o formulario.
     * @throws InternalErrorException En caso de erro no procesado do ficheiro enviado
     */
    @PostMapping("/add")
    public String create(@Valid Documento documento, BindingResult result, RedirectAttributes ra, MultipartFile file) throws InternalErrorException {

        if (file.isEmpty()) {
            // Debe achegarse un ficheiro ao crear un novo documento
            result.rejectValue(FIELD_NOMBRE, "NotEmptyFile");
        } else if ( !MultipartFileUtils.isAllowedContentType(file) ) {
            // O ficheiro asociado ao documento debe ter un formato permitido.
            result.rejectValue(FIELD_NOMBRE, "InvalidContentType");
        }
        
        // Se hai erros, paramos e voltamos amosar o formulario
        if (result.hasErrors()) {
            return DETAILS_VIEW;
        }

        // Procesamos o ficheiro
        processMultipartFile(file, documento);

        try {
            // Gardamos o rexistro na base de datos
            bibliotecaDocumentosService.crear(documento);
            LOGGER.info("Documento creado correctamente cos valores: {}", documento);
            
            // Mensaxe de confirmación ao usuario, rediriximo ao detalle do documento
            MessageHelper.success(ra, "documentos.documento.creado");
            return redirectTo("/documentos/" + documento.getId());
        } catch (DocumentoYaExisteException ex) {
            // O documento xa existe: amosamos de novo o formulario, indicando o erro ao usuario
            LOGGER.error("Erro creando o documento debido a título duplicado: '{}'", documento.getTitulo(), ex);
            cleanUploadedFile(documento);
            result.rejectValue(FIELD_TITULO, "Unique");
            return DETAILS_VIEW;
        }
    }
    
    /**
     * Amosa o detalle dun <code>Documento</code> da biblioteca.
     *
     * @param id Identificador interno do documento.
     * @param model Modelo coas variables da vista.
     * @return Vista do detalle do documento.
     */
    @GetMapping("/{id}")
    public String edit(@PathVariable long id, ModelMap model) {

        LOGGER.info("Carga de la vista de detalle de un documento con el id: {}", id);
        model.addAttribute(bibliotecaDocumentosService.buscar(id));

        LOGGER.info("Finalizada la carga de la vista de detalle de un documento con el id: {}", id);

        return DETAILS_VIEW;
    }

    /**
     * Recibe o formulario enviado polo usuario para actualizar un documento.
     *
     * @param documento Documento a actualizar.
     * @param result Resultado de las validaciones efectuadas.
     * @param ra Parámetros de redirección (para amosar erros de validación).
     * @param file Ficheiro asociado ao documento.
     * @param id Identificador interno do documento.
     * @return En caso de éxito, vista de detalle do documento. En caso de erro, voltamos amosar o formulario.
     * @throws InternalErrorException En caso de erro no procesado do ficheiro enviado
     */
    @PostMapping("/{id}")
    public String save(@Valid Documento documento, BindingResult result, RedirectAttributes ra, MultipartFile file, @PathVariable long id) throws InternalErrorException {
        
        // Asignamos ID para redirixir ben do detalle en caso de erro
        documento.setId(id);
        
        LOGGER.info("Actualizando documento {} cos valores {}", id, documento);

        // Se consta un novo ficheiro asociado, comprobamos que no estexa baleiro e que teña un formato permitido.
        if (isFileSelected(file)) {
            if (file.isEmpty()) {
                result.rejectValue(FIELD_NOMBRE, "NotEmptyFile");
            } else if ( !MultipartFileUtils.isAllowedContentType(file) ) {
                result.rejectValue(FIELD_NOMBRE, "InvalidContentType");
            }
        }
        
        // En caso de erro, voltamos a amosar o formulario de edición
        if (result.hasErrors()) {
            Documento doc = bibliotecaDocumentosService.buscar(id);
            documento.setNombre(doc.getNombre());
            return DETAILS_VIEW;
        }

        // Se consta un novo ficheiro, procesámolo.
        if (isFileSelected(file)) {
            processMultipartFile(file, documento);
        }
        
        try {
            // Actualizamos rexistro na BD
            bibliotecaDocumentosService.actualizar(documento);
            LOGGER.info("Documento {} actualizado correctamente cos valores {}", id, documento);

            // Mensaxe de confirmación ao usuario, rediriximo ao detalle do documento
            MessageHelper.success(ra, "documentos.documento.actualizado");
            return redirectTo("/documentos/{id}");
        } catch (DocumentoYaExisteException ex) {
            // O documento xa existe: amosamos de novo o formulario, indicando o erro ao usuario
            LOGGER.error("Error actualizando el documento debido a titulo duplicado: {}", documento.getTitulo(), ex);
            cleanUploadedFile(documento);
            result.rejectValue(FIELD_TITULO, "Unique");
            Documento doc = bibliotecaDocumentosService.buscar(id);
            documento.setNombre(doc.getNombre());
            return DETAILS_VIEW;
        }
    }

    /**
     * Descarga un ficheiro correspondiente a un <code>Documento</code>.
     *
     * @param uuid Identificador do documento.
     * @param response Resposta HTTP onde se escribe o contido do ficheiro.
     * @param request Petición HTTP do usuario.
     * @param ra Parámetros de redirección (para amosar erros de validación).
     * @return En caso de éxito, 'null'. En caso de erro, vista a amosar.
     */
    @GetMapping("/{uuid}/download")
    public String download(@PathVariable String uuid, HttpServletResponse response, HttpServletRequest request, RedirectAttributes ra) {
        LOGGER.info("Descargando documento co UUID '{}'", uuid);
        
        // Buscamos o ficheiro asociado ao documento
        Documento documento = bibliotecaDocumentosService.buscar(uuid);
        File file = storageManager.load(documento.getPath());
        try(InputStream in = new FileInputStream(file)) {
            // Asegurarse de amostrar o dialogo de descarga no navegador
            response.setHeader("Content-disposition", "attachment; filename=\"" + documento.getNombre() + "\"");
            response.setContentType(documento.getFormato());
            IOUtils.copy(in, response.getOutputStream());
            response.flushBuffer();
        } catch(IOException e) {
            // Erro ao obter o ficheiro asociado ao documento
            LOGGER.info("Erro ao descargar o documento con UUID '{}'", uuid, e);
            MessageHelper.error(ra, "documentos.documento.descargar.error");
            // Se hai un ID na petición, é que estamos no detalle, senón na lista.
            String id = request.getParameter("id");
            return id==null ? redirectTo("/documentos") : redirectTo("/documentos/"+id);
        }
        return null;
    }
    
    /**
     * Borra un <code>Documento</code> da biblioteca.
     * 
     * @param ra Parámetros de redirección (para amosar erros de validación).
     * @param id Identificador do documento.
     * @return Vista do listado de documentos.
     */
    @PostMapping("/{id}/delete")
    public String delete(RedirectAttributes ra, @PathVariable long id) {

        LOGGER.info("Eliminando documento co ID '{}'", id);

        try {
            // Eliminamos rexistro de BD
            bibliotecaDocumentosService.borrar(id);

            // Mensaxe de confirmación ao usuario
            MessageHelper.success(ra, "documentos.documento.borrado.exito");
            LOGGER.info("Documento con el id: {} eliminado correctamente", id);
        } catch (DocumentoNoEncontradoException e) {
            // Non se atopou o documento: amosamos mensaxe de erro
            LOGGER.error("Erro borrando o ficheiro con ID '{}'", id, new InternalErrorException(e.getMessage(), e));
            MessageHelper.error(ra, "documentos.documento.borrado.error");
        }
        
        return redirectTo("/documentos");
    }
    
    /**
     * Controla malas requests de Spring
     * 
     * @param exception Excepción producida
     * @return Vista a amosar
     */
    @ExceptionHandler(TypeMismatchException.class) 
    public String handleExceptions(Exception exception) {
        // O TypeMismatchException prodúcese, por exemplo, cando
        // se especifica manualmente un ID non númerico
        // Asumimos un erro 404 neste caso
        LOGGER.error("Erro de request", exception);
        return "error/404";
    }
    
    /**
     * Almacena o fichero recibido por Multipart e engade a información relacionada ao documento.
     * 
     * @param multipart Ficheiro subido polo usuario
     * @param documento Documento a gardar
     * @throws InternalErrorException Erro procesando o ficheiro adxunto
     */
    private void processMultipartFile(MultipartFile multipart, Documento documento) throws InternalErrorException {
        try {
            // Só verificamos se se enviou un ficheiro
            if (!multipart.isEmpty()) {
                // Gardamos en disco cubrimos no documento a información útil
                String path = storageManager.store(multipart.getInputStream());
                documento.setPath(path);
                documento.setFormato(multipart.getContentType());
                documento.setNombre(multipart.getOriginalFilename());
            }
        } catch (IOException e) {
            throw new InternalErrorException("Erro ao procesar o ficheiro adxunto", e);
        }
    }
    
    /**
     * Limpa o ficheiro achegado polo usuario ante un erro
     * @param documento Documento
     */
    private void cleanUploadedFile(Documento documento){
        storageManager.delete(documento.getPath());
    }

    /**
     * Comproba se o usuario achegou un ficheiro.
     * 
     * @param multipart Datos do ficheiro enviado polo usuario.
     * @return <code>true</code> se se achegou un ficheiro
     */
    public boolean isFileSelected(MultipartFile multipart) {
        return multipart != null && !"".equals(multipart.getOriginalFilename());
    }
    
}