/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.configuracion.web;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.forwardedUrl;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.view;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.MockitoAnnotations;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.test.web.servlet.setup.StandaloneMockMvcBuilder;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.ModelAndView;

import es.xunta.amtega.transversal.archam.core.exception.ResourceNotFoundException;

public class ConfiguracionControllerTest {

    private static final String INDEX_URL = "/configuracion";
    private static final String SISTEMA_URL = "/configuracion/sistema";

    private MockMvc mockMvc;
    
    @InjectMocks
    private ConfiguracionController configuracionController;

    @Before
    public void setup() {
        // Process mock annotations
        MockitoAnnotations.initMocks(this);
        this.mockMvc = this.initMockMvc(null);
    }
    
    @Test
    public void configuracionForGetShouldBeAllowed() throws Exception {
        //given: petición á pantalla de configuración por GET
        
        //when: realizamos a petición
        this.mockMvc.perform((get(INDEX_URL)))
        //then: vista correcta e estado OK (220)
            .andExpect(forwardedUrl("configuracion/index"))
            .andExpect(view().name("configuracion/index"))
            .andExpect(status().isOk())
            .andDo(print());
    }
    
    @Test
    public void configuracionForPostShouldBeDenied() throws Exception {
        //given: petición á pantalla de configuración por POST
        
        //when: realizamos a petición
        this.mockMvc.perform((post(INDEX_URL)))
        //then: non se permite acceso por POST
            .andExpect(status().isMethodNotAllowed())
            .andDo(print());
    }
    
    @Test
    public void sistemaForGet() throws Exception {
        //given: petición á pantalla de sistema por GET
        
        //when: realizamos la petición
        this.mockMvc.perform((get(SISTEMA_URL)))
        //then: vista correcta e estado OK
            .andExpect(forwardedUrl("configuracion/sistema"))
            .andExpect(view().name("configuracion/sistema"))
            .andExpect(status().isOk())
            .andDo(print());
    }
    
    @Test
    public void sistemaException() throws Exception {
        // given: Trucamos o valor do fichero de Release Notes a cargar (non existente), e rexistramos un "HandlerExceptionResolver" customizado
        configuracionController.setChangelogFile("not-found.txt");
        this.mockMvc = this.initMockMvc(new CustomHandlerExceptionResolver());
        
        //when: realizamos a petición
        this.mockMvc.perform((get(SISTEMA_URL)))
        //then: verificamos que salta o CustomHandlerExceptionResolver (devolve un erro 404)
            .andExpect(forwardedUrl("error/404"))
            .andExpect(status().isOk())
            .andDo(print());
        
        // Restauramos valor orixinal
        configuracionController.setChangelogFile("release-notes.txt");
    }
    
    /**
     * Inicializa o mock do MVC, opcionalmente cun HandlerExceptionResolver
     * @param exceptionHandler HandlerExceptionResolver
     * @return Mock
     */
    private MockMvc initMockMvc(HandlerExceptionResolver exceptionHandler) {
        StandaloneMockMvcBuilder builder = MockMvcBuilders.standaloneSetup(configuracionController);
        if(exceptionHandler!=null) {
            builder.setHandlerExceptionResolvers(exceptionHandler);
        }
        return builder.build();
    }
    
    /**
     * Exception Handler para probas
     */
    private static class CustomHandlerExceptionResolver implements HandlerExceptionResolver {
        @Override
        public ModelAndView resolveException(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) {
            if(ex instanceof ResourceNotFoundException) {
                return new ModelAndView("error/404");
            }
            return null;
        }
    }
    
}
