/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import es.xunta.amtega.codapp.documentos.exception.DocumentoNoEncontradoException;
import es.xunta.amtega.codapp.documentos.exception.DocumentoYaExisteException;
import org.springframework.data.jpa.domain.Specification;

public class BibliotecaDocumentosServiceImplTest {
    
    private static final String TITULO_TEST = "titulo";
    private static final String TITULO_TEST2 = "documento";

    private DocumentoRepository repo;
    private BibliotecaDocumentosService documentosService;
    
    @Rule
    public ExpectedException thrown = ExpectedException.none();

    @Before
    public void setup() {
        ApplicationEventPublisher publisher = mock(ApplicationEventPublisher.class);
        repo = mock(DocumentoRepository.class);
        documentosService = new BibliotecaDocumentosServiceImpl(repo, publisher);
    }

    @Test
    public void documentoSinTituloShouldThrowException() throws DocumentoYaExisteException {
        // given: o documento non ten título
        Documento doc = new Documento();

        // when: intentamos gardar o documento
        thrown.expect(IllegalArgumentException.class);
        documentosService.crear(doc);

        // then: o método debe lanzar unha IllegalArgumentException
        this.assertExpectedException();
    }

    @Test
    public void tituloDuplicadoShouldThrowException() throws DocumentoYaExisteException {
        // given: xa existe un documento co mesmo título. Configuramos o mock para que devolva un documento ao buscar
        String titulo = TITULO_TEST;
        when(repo.findOneByTitulo(titulo)).thenReturn(new Documento());

        // when: intentamos gardar un documento co mesmo título
        thrown.expect(DocumentoYaExisteException.class);
        Documento newDoc = this.buildDocumento(null, titulo);
        documentosService.crear(newDoc);

        // then: o método debe detectar que xa existe un
        this.assertExpectedException();
    }

    @Test
    public void buscarIdExisteShouldReturnDocumento() {
        // given: existe un documento co ID. Configuramos o mock para que devolva un documento ao buscar por ID.
        long id = 1;
        when(repo.findOne(id)).thenReturn(new Documento());

        // when: recuperamos o documento por ID
        Documento doc = documentosService.buscar(id);

        // then: o método debe devolver un documento
        assertThat(doc, is(notNullValue()));
    }

    @Test
    public void buscarIdNoExisteShouldThrowException() {
        // given: non existe un documento con este ID
        long id = 1;

        // when: recuperamos o documento por ID
        thrown.expect(DocumentoNoEncontradoException.class);
        documentosService.buscar(id);

        // then: o método debe indicar que non se atopou o documento
        this.assertExpectedException();
    }

    @Test
    public void buscarUuidExisteShouldReturnDocumento() {
        // given: existe un documento co UUID. Configuramos o mock para que devolva un documento.
        String uuid = "1";
        when(repo.findOneByUuid(uuid)).thenReturn(new Documento());

        // when: recuperamos o documento por uuid
        Documento doc = documentosService.buscar(uuid);

        // then: o método non debe devolver un documento
        assertThat(doc, is(notNullValue()));
    }

    @Test
    public void buscarUuidNoExisteShouldThrowException() {
        // given: non existe un documento con este UUID
        String uuid = "1";

        // when: recuperamos el documento por uuid
        thrown.expect(DocumentoNoEncontradoException.class);
        documentosService.buscar(uuid);

        // then: o método debe indicar que non se atopou o documento
        this.assertExpectedException();
    }

    @Test
    public void crearDocumentoConIdShouldThrowException() throws DocumentoYaExisteException {
        // given: xa existe un documento co mesmo ID. Configuramos o mock para que devolva un documento
        Long id = 1L;
        String titulo = "documento con id";
        Documento expected = this.buildDocumento(id, "other title");
        when(repo.findOneByTitulo(titulo)).thenReturn(expected);

        // when: intentamos gardar un documento con mesmo título
        thrown.expect(IllegalArgumentException.class);
        Documento newDoc = this.buildDocumento(id, titulo);
        documentosService.crear(newDoc);

        // then: debe lanzarse unha excepción
        this.assertExpectedException();
    }

    @Test
    public void crearDocumentoShouldCrearDocumento() throws DocumentoYaExisteException {
        // given: non existe un documento co mesmo titulo. Configuramos o mock para que devolva null
        String titulo = TITULO_TEST2;
        when(repo.findOneByTitulo(titulo)).thenReturn(null);

        // when: intentamos gardar un documento
        Documento newDoc = this.buildDocumento(null, titulo);
        documentosService.crear(newDoc);

        // then: execución correcta
        assertThat(newDoc, is(notNullValue()));
    }

    @Test
    public void actualizarDocumentoConIdShouldThrowException() throws DocumentoYaExisteException {
        // given: xa existe outro documento co mesmo título
        Long id = 1L;
        String titulo = TITULO_TEST2;
        Long otherId = 2L;
        // Configuramos mock para devolver un documento ao consultar por ID
        Documento expected = this.buildDocumento(id, titulo);
        when(repo.findOne(id)).thenReturn(expected);
        // Configuramos mock para devolver outro documento ao consultar por título
        Documento other = this.buildDocumento(otherId, titulo);
        when(repo.findOneByTitulo(titulo)).thenReturn(other);

        // when: intentamos actualizar un cun titulo existente noutro documento
        thrown.expect(DocumentoYaExisteException.class);
        Documento newDoc = this.buildDocumento(id, titulo);
        documentosService.actualizar(newDoc);

        // then: debe indicarse que xa existe un documento con ese titulo
        this.assertExpectedException();
    }

    @Test
    public void actualizarDocumentoShouldUpdateDocumento() throws DocumentoYaExisteException {
        // given: non existe outro documento co mesmo título: a busca por título devolve o mesmo
        Long id = 1L;
        String titulo = TITULO_TEST2;
        // Configuramos mock para devolver un documento ao consultar por ID, e mesmo ao consultar por título
        Documento expected = this.buildDocumento(id, titulo);
        when(repo.findOneByTitulo(titulo)).thenReturn(expected);
        when(repo.findOne(id)).thenReturn(expected);

        // when: intentamos actualizar o documento co mesmo título
        Documento newDoc = this.buildDocumento(id, titulo);
        documentosService.actualizar(newDoc);

        // then: ao ser o mesmo documento, a execución é correcta
        assertThat(newDoc, is(notNullValue()));
    }

    @Test
    public void buscarByQueryShouldReturnDocumentosPaginado() {
        // given: busca dun documento por título. Configuramos mock para devolver unha páxina de documentos.
        String query = "my title";
        Page<Documento> expectedPage = this.buildDocumentPage(2);
        when(repo.findByTituloLikeIgnoreCase(any(String.class), isA(Pageable.class))).thenReturn(expectedPage);

        // when: recuperamos o documento por UUID
        Page<Documento> result = documentosService.buscar(query, new PageRequest(1, 1));

        // then: devolve o resultado esperado
        assertThat(result, is(expectedPage));
    }

    @Test
    public void buscarShouldReturnDocumentosPaginado() {
        // given: listado de todos os documentos. Configuramos mock para devolver unha páxina de documentos.
        Page<Documento> expectedPage = this.buildDocumentPage(3);
        when(repo.findAll(isA(Pageable.class))).thenReturn(expectedPage);

        // when: recuperamos el documento por uuid
        Page<Documento> result = documentosService.buscar(new PageRequest(1, 1));

        // then: devolve o resultado esperado
        assertThat(result, is(expectedPage));
    }

    @Test
    @SuppressWarnings("unchecked")
    public void buscarByDocumentoFilterShouldReturnDocumentosPaginado() {
        // given: busca filtrada de documentos
        DocumentoFilter filter = new DocumentoFilter();
        filter.setTitulo(TITULO_TEST);
        // Configuramos mock
        Page<Documento> expectedPage = this.buildDocumentPage(1);
        when(repo.findAll(any(Specification.class), isA(Pageable.class))).thenReturn(expectedPage);

        // when: recuperamos el documento por uuid
        Page<Documento> result = documentosService.buscar(filter, new PageRequest(1, 1));

        // then: el servicio debe devolver un documento
        assertThat(result, is(expectedPage));
    }

    @Test
    public void borrarDocumentoByIdExistsShouldDeleteDocumento() {
        // given: existe documento co ID indicado
        Long id = 1L;
        String titulo = TITULO_TEST;
        Documento expected = this.buildDocumento(id, titulo);
        when(repo.findOne(id)).thenReturn(expected);

        // when: borramos el documento
        documentosService.borrar(id);
        
        // then: o documento bórrase correctamente
        assertThat(expected, is(notNullValue()));
    }

    /**
     * Constrúe unha páxina de resultados
     * @return
     */
    private Documento buildDocumento(Long id, String titulo) {
        Documento documento = new Documento();
        documento.setId(id);
        documento.setTitulo(titulo);
        return documento;
    }

    /**
     * Constrúe unha páxina de resultados
     * @return
     */
    private Page<Documento> buildDocumentPage(int numResultados) {
        List<Documento> expected = new ArrayList<>();
        for(long i=0; i<numResultados; i++) {
            expected.add(this.buildDocumento(i, TITULO_TEST+i));
        }
        return new PageImpl<>(expected);
    }
    
    /**
     * Lanza erro ao esperar unha excepción
     */
    private void assertExpectedException() {
        fail("Esperábase unha excepción");
    }
}
