/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos.web;

import static org.hamcrest.CoreMatchers.*;
import static org.hamcrest.Matchers.hasProperty;
import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.doThrow;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.fileUpload;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.flash;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.forwardedUrl;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.model;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.view;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Paths;
import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Matchers;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableHandlerMethodArgumentResolver;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMultipartHttpServletRequestBuilder;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import es.xunta.amtega.codapp.documentos.BibliotecaDocumentosService;
import es.xunta.amtega.codapp.documentos.Documento;
import es.xunta.amtega.codapp.documentos.DocumentoFilter;
import es.xunta.amtega.codapp.documentos.exception.DocumentoNoEncontradoException;
import es.xunta.amtega.codapp.documentos.exception.DocumentoYaExisteException;
import es.xunta.amtega.transversal.archam.core.io.StorageManager;
import es.xunta.amtega.transversal.archam.core.web.mvc.Message;
import es.xunta.amtega.transversal.utils.BaseUtils.Configuration;

@SuppressWarnings("deprecation")
public class BibliotecaDocumentosControllerTest {

    private static final String LIST_VIEW = "documentos/list";
    private static final String DETAILS_VIEW = "documentos/details";
    
    private static final String MODEL = "documento";
    private static final String MODEL_PAGE = "page";
    private static final String MODEL_FILTER = "filter";
    
    private static final String BASE_URL = "/documentos";
    private static final String SEARCH_URL = BASE_URL + "/search";
    private static final String ADVANCED_SEARCH_URL = BASE_URL + "/advanced-search";
    private static final String ADD_URL = BASE_URL + "/add";
    private static final String DETAIL_URL = BASE_URL + "/";
    private static final String DELETE_URL = "/delete";
    private static final String PARAM_TITULO = "titulo";
    private static final String PARAM_NOMBRE = "nombre";
    private static final String PARAM_MESSAGE = "message";

    private static final String TEST_FILE = "foo.txt";
    
    private static final String REDIRECT_TO = "redirect:";
    
    private static final Long TEST_DOC_ID = 1L;
    
    private MockMvc mockMvc;

    @Mock
    private BibliotecaDocumentosService bibliotecaDocumentosService;
    @Mock
    private StorageManager storageManager;

    @InjectMocks
    private BibliotecaDocumentosController bibliotecaDocumentosController;

    @Before
    public void setup() {
        // Process mock annotations
        MockitoAnnotations.initMocks(this);
        this.mockMvc = MockMvcBuilders.standaloneSetup(bibliotecaDocumentosController)
                .setCustomArgumentResolvers(new PageableHandlerMethodArgumentResolver()).build();
    }

    /**
     * Valida o acceso á pantalla de listado de documentos
     * @throws Exception excepción
     */
    @Test
    public void documentosForGet() throws Exception {
        //given: establecemos a páxina de resultado da busca
        Page<Documento> page = new PageImpl<>(new ArrayList<Documento>());
        given(bibliotecaDocumentosService.buscar(Matchers.<Pageable>anyObject())).willReturn(page);
        
        //when: petición ao listado de documentos
        this.mockMvc.perform((get(BASE_URL)))
        //then: Validamos a vista, variables e estado
                .andExpect(forwardedUrl(LIST_VIEW))
                .andExpect(view().name(LIST_VIEW))
                .andExpect(model().attributeExists(MODEL_PAGE))
                .andExpect(model().attribute(MODEL_PAGE, is(page)))
                .andExpect(model().attributeExists(MODEL_FILTER))
                .andExpect(status().isOk())
                .andDo(print());
    }

    /**
     * Valida que el método POST non está permitido
     * @throws Exception Excepción
     */
    @Test
    public void documentosForPost() throws Exception {
        //when: petición por POST
        this.mockMvc.perform((post(BASE_URL)))
        //then: método no permitido
            .andExpect(status().isMethodNotAllowed())
            .andDo(print());
    }

    /**
     * Valida a busca sinxela
     * @throws Exception Excepción
     */
    @Test
    public void searchForGet() throws Exception {
        //given: inicializamos mock para que devolva unha páxina de resultados concreta
        Page<Documento> page = new PageImpl<>(new ArrayList<Documento>());
        given(bibliotecaDocumentosService.buscar(Matchers.anyString(), Matchers.<Pageable>anyObject())).willReturn(page);
        
        // when: petición por GET á busca
        this.mockMvc.perform((get(SEARCH_URL)))
        // then: validamos vista, varialbes e estado
                .andExpect(forwardedUrl(LIST_VIEW))
                .andExpect(view().name(LIST_VIEW))
                .andExpect(model().attributeExists(MODEL_PAGE))
                .andExpect(model().attribute(MODEL_PAGE, is(page)))
                .andExpect(status().isOk())
                .andDo(print());
    }

    /**
     * Valida a busca avanzada
     * @throws Exception Excepción
     */
    @Test
    public void avancedSearchForGet() throws Exception {
        //given: inicializamos mock para que devolva unha páxina de resultados concreta
        Page<Documento> page = new PageImpl<>(new ArrayList<Documento>());
        given(bibliotecaDocumentosService.buscar(Matchers.<DocumentoFilter>anyObject(), Matchers.<Pageable>anyObject())).willReturn(page);
        
        // when: petición por GET á busca avanzada
        this.mockMvc.perform((get(ADVANCED_SEARCH_URL)))
        // then: validamos vista, varialbes e estado
                .andExpect(forwardedUrl(LIST_VIEW))
                .andExpect(view().name(LIST_VIEW))
                .andExpect(model().attributeExists(MODEL_PAGE))
                .andExpect(model().attribute(MODEL_PAGE, is(page)))
                .andExpect(status().isOk())
                .andDo(print());
    }

    /**
     * Valida o formulario para engadir un documento
     * @throws Exception Excepción
     */
    @Test
    public void addForGet() throws Exception {
        // when: petición a engadir documento
        this.mockMvc.perform((get(ADD_URL)))
        // then: validamos vista, varialbes e estado
            .andExpect(forwardedUrl(DETAILS_VIEW))
            .andExpect(view().name(DETAILS_VIEW))
            .andExpect(model().attributeExists(MODEL))
            .andExpect(status().isOk())
            .andDo(print());
    }
    
    /**
     * Valida a xestión de erros ao gardar un documento
     * @throws Exception Excepción
     */
    @Test
    public void createEmptyError() throws Exception {
        //given: gardado dun documento baleiro
        Documento documento = new Documento();
        documento.setTitulo("a");
        MockMultipartFile file = getEmptyMultipartFile();
        
        //when: envío de formulario
        this.mockMvc.perform((fileUpload(ADD_URL)
                    .file(file)
                    .param(PARAM_TITULO, documento.getTitulo())))
        //then: validamos a vista, erros e estado
                .andExpect(forwardedUrl(DETAILS_VIEW))
                .andExpect(view().name(DETAILS_VIEW))
                .andExpect(model().hasErrors())
                .andExpect(model().attributeHasFieldErrors(MODEL, PARAM_TITULO, PARAM_NOMBRE))
                .andExpect(status().isOk())
                .andDo(print());
    }
    
    /**
     * Valida o control do tipo de documento ao gardalo
     * @throws Exception Excepción
     */
    @Test
    public void createContentTypeError() throws Exception {
        //given: gardado dun documento con erros
        Documento documento = new Documento();
        documento.setTitulo("a");
        MockMultipartFile file = getWrongContentTypeMultipartFile();
        
        //when: envío de formulario
        this.mockMvc.perform((fileUpload(ADD_URL)
                    .file(file)
                    .param(PARAM_TITULO, documento.getTitulo())))
        //then: validamos a vista, erros e estado
                .andExpect(forwardedUrl(DETAILS_VIEW))
                .andExpect(view().name(DETAILS_VIEW))
                .andExpect(model().hasErrors())
                .andExpect(model().attributeHasFieldErrors(MODEL, PARAM_TITULO, PARAM_NOMBRE))
                .andExpect(status().isOk())
                .andDo(print());
    }
    
    
    /**
     * Valida o gardado sen erros dun documento
     * @throws Exception Excepción
     */
    @Test
    public void createWithoutErrors() throws Exception {
        //given: gardado dun documento
        Documento documento = new Documento();
        MockMultipartFile file = getNotEmptyMultipartFile();

        //when: envío de formulario
        this.mockMvc.perform((fileUpload(ADD_URL)
                    .file(file)
                    .param(PARAM_TITULO, "fichero1")))
        //then: validamos a vista, variables e estado
                .andExpect(forwardedUrl(null))
                .andExpect(view().name(REDIRECT_TO + DETAIL_URL + documento.getId()))
                .andExpect(model().hasNoErrors())
                .andExpect(flash().attributeExists(PARAM_MESSAGE))
                .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty(PARAM_MESSAGE, is("documentos.documento.creado"))))
                .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty("type", is(Message.Type.SUCCESS))))
                .andExpect(status().isFound())
                .andDo(print());
    }

    /**
     * Valida a carga do detalle dun documento
     * @throws Exception
     */
    @Test
    public void editForGet() throws Exception {
        //given: Declaramos un documento a devolver polo mock
        Long id = 1L;
        Documento documento = new Documento();
        documento.setId(id);
        documento.setTitulo("Titulo");
        documento.setDescripcion("Descripción documento");
        documento.setNombre("miNombre");
        documento.setFormato("formto");
        documento.setUuid("UUID");
        given(bibliotecaDocumentosService.buscar(id)).willReturn(documento);

        // when: petición
        this.mockMvc.perform((get(DETAIL_URL + id)))
        //then: validamos a vista, variables e estado
                .andExpect(forwardedUrl(DETAILS_VIEW))
                .andExpect(view().name(DETAILS_VIEW))
                .andExpect(model().attributeExists(MODEL))
                .andExpect(model().attribute(MODEL, hasProperty(PARAM_TITULO, is(documento.getTitulo()))))
                .andExpect(model().attribute(MODEL, hasProperty("descripcion", is(documento.getDescripcion()))))
                .andExpect(model().attribute(MODEL, hasProperty(PARAM_NOMBRE, is(documento.getNombre()))))
                .andExpect(model().attribute(MODEL, hasProperty("uuid", is(documento.getUuid()))))
                .andExpect(status().isOk())
                .andDo(print());
    }

    /**
     * Valida a edición con erros
     * @throws Exception Excepción
     */
    @Test
    public void saveWithErrors() throws Exception {
        //given: configuramos mock para deolver o documento axeitado
        this.mockBuscaDocumento(TEST_DOC_ID);
        MockMultipartFile file = getNotEmptyMultipartFile();
        //when: petición
        this.performFileUploadAndExpectError(TEST_DOC_ID, file);
    }

    /**
     * Valida la edición con errores
     * @throws Exception Excepción
     */
    @Test
    public void saveWithErrorsNullFile() throws Exception {
        //given: configuramos mock para deolver o documento axeitado
        this.mockBuscaDocumento(TEST_DOC_ID);
        //when: petición
        this.performFileUploadAndExpectError(TEST_DOC_ID, null);
    }
    
    @Test
    public void saveWithErrorsEmptyOriginalFilename() throws Exception {
        //given: configuramos mock para deolver o documento axeitado
        this.mockBuscaDocumento(TEST_DOC_ID);
        MockMultipartFile file = getNotEmptyMultipartFileWithEmptyOriginalName();
        //when: petición
        this.performFileUploadAndExpectError(TEST_DOC_ID, file);
    }
    
    @Test
    public void saveWithErrorsEmptyFile() throws Exception {
        //given: configuramos mock para deolver o documento axeitado
        this.mockBuscaDocumento(TEST_DOC_ID);
        MockMultipartFile file = getEmptyMultipartFile();
        //when: petición
        this.performFileUploadAndExpectError(TEST_DOC_ID, file);
    }
    
    @Test
    public void saveWithErrorsWrongContentType() throws Exception {
        //given: configuramos mock para deolver o documento axeitado
        this.mockBuscaDocumento(TEST_DOC_ID);
        MockMultipartFile file = getWrongContentTypeMultipartFile();
        //when: petición
        this.performFileUploadAndExpectError(TEST_DOC_ID, file);
    }
    
    /**
     * Editar sin errores
     * @throws Exception
     */
    @Test
    public void saveWithoutErrors() throws Exception {
        //given: configuramos mock para deolver o documento axeitado
        Long id = 1L;
        MockMultipartFile file = getNotEmptyMultipartFile();
        Documento documento = new Documento();
        documento.setId(id);
        given(bibliotecaDocumentosService.buscar(id)).willReturn(documento);

        //when: petición
        this.mockMvc.perform((fileUpload(DETAIL_URL + id)
                    .file(file)))
        //then: validamos que garda
            .andExpect(status().isFound())
            .andDo(print());
    }

    /**
     * Validamos el guardado de un documento que ya existe
     * @throws Exception Excepción
     */
    @Test
    public void saveWithDocumentoYaExisteException() throws Exception {
        //given: configuramos mock para devolver excepción
        MockMultipartFile file = getNotEmptyMultipartFile();
        Documento documento = new Documento();
        Long id = 1L;
        given(bibliotecaDocumentosService.buscar(id)).willReturn(documento);
        doThrow(new DocumentoYaExisteException("El documento ya existe")).when(bibliotecaDocumentosService).actualizar(Matchers.<Documento>anyObject());
        
        //when: petición
        this.mockMvc.perform((fileUpload(DETAIL_URL + id)
                .file(file)))
        //then: validamos erro
            .andExpect(forwardedUrl(DETAILS_VIEW))
            .andExpect(view().name(DETAILS_VIEW))
            .andExpect(model().hasErrors())
            .andExpect(model().attributeHasFieldErrors(MODEL, PARAM_TITULO))
            .andExpect(status().isOk())
            .andDo(print());
    }
    
    /**
     * Descargamos un documento por GET
     * @throws Exception Excepción
     */
    @Test
    public void downloadForGet() throws Exception {
        //given: configuramos mock para devolver documento e ruta onde se gardou o documento
        Documento documento = new Documento();
        documento.setUuid("123456");
        documento.setNombre(TEST_FILE);
        documento.setPath("#");
        MockMultipartFile file = getNotEmptyMultipartFile();
        given(bibliotecaDocumentosService.buscar(Matchers.anyString())).willReturn(documento);
        given(storageManager.load("#")).willReturn(new File(Configuration.getTempPath(), file.getOriginalFilename()));
        
        //when: petición de descarga
        this.mockMvc.perform((get(DETAIL_URL + documento.getUuid() + "/download")))
        //then: validamos estado
            .andExpect(forwardedUrl(null))
            .andExpect(status().isOk())
            .andDo(print());
    }
    
    
    /**
     * Validamos que non se pode facer descarga por POST
     * @throws Exception Excepción
     */
    @Test
    public void downloadForPost() throws Exception {
        //given: Inicializamos el documento
        Documento documento = new Documento();
        documento.setUuid("123456");
        documento.setNombre(TEST_FILE);

        //when: petición de descarga
        this.mockMvc.perform((post(DETAIL_URL + documento.getUuid() + "/download")))
        //then: validamos que el método no está permitido
            .andExpect(forwardedUrl(null))
            .andExpect(status().isMethodNotAllowed())
            .andDo(print());
    }
    
    /**
     * Validamos o borrado por POST
     * @throws Exception Excepción
     */
    @Test
    public void deleteForPost() throws Exception {
        //given: declaramos un documento
        Documento documento = new Documento();
        documento.setId(1L);
        
        //when: petición
        this.mockMvc.perform((post(DETAIL_URL + documento.getId() + DELETE_URL)))
        //then: validamos que se redirixe á URL correcta e se obtén mensaxe de éxito
            .andExpect(forwardedUrl(null))
            .andExpect(view().name(REDIRECT_TO + BASE_URL))
            .andExpect(status().isFound())
            .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty(PARAM_MESSAGE, is("documentos.documento.borrado.exito"))))
            .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty("type", is(Message.Type.SUCCESS))))
            .andDo(print());
    }

    /**
     * Validamos que se lanza excepción no borrado
     * @throws Exception Excepcción
     */
    @Test
    public void deleteForPostWithException() throws Exception {
        //given: creamos un documento e indicamos ao mock que ten que lanzar unha excepción
        Documento documento = new Documento();
        documento.setId(1L);
        doThrow(new DocumentoNoEncontradoException("Error al borrar el documento")).when(bibliotecaDocumentosService).borrar(documento.getId());
        
        //when: petición
        this.mockMvc.perform((post(DETAIL_URL + documento.getId() + DELETE_URL)))
        //then: validamos excepción
            .andExpect(view().name(REDIRECT_TO + BASE_URL))
            .andExpect(status().isFound())
            .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty(PARAM_MESSAGE, is("documentos.documento.borrado.error"))))
            .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty("type", is(Message.Type.DANGER))))
            .andDo(print());
    }
    
    /**
     * Validamos que non se poden borrar ficheiros por GET
     * @throws Exception Excepción
     */
    @Test
    public void deleteForGet() throws Exception {
        //given: declaramos un documento
        Documento documento = new Documento();
        documento.setId(1L);
        
        //when: petición de borrado por GET
        this.mockMvc.perform((get(DETAIL_URL + documento.getId() + DELETE_URL)))
        //then: non está permitido
            .andExpect(forwardedUrl(null))
            .andExpect(status().isMethodNotAllowed())
            .andDo(print());
    }
    
    
    
    
    
    /**
     * Crea un ficheiro Mock non baleiro
     * 
     * @return Multipart
     * @throws IOException Excepcion
     */
    private MockMultipartFile getNotEmptyMultipartFile() throws IOException {
        File createdFile = Paths.get(Configuration.getTempPath(), TEST_FILE).toFile();
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(createdFile))) {
            writer.write("Hello world!");
        }
        InputStream in = new FileInputStream(createdFile);
        return new MockMultipartFile("file", TEST_FILE, "text", in);
    }
    
    /**
     * Crea un ficheiro co nome baleiro
     * @return Multipart
     * @throws IOException Excepción
     */
    private MockMultipartFile getNotEmptyMultipartFileWithEmptyOriginalName() throws IOException {
        File createdFile = Paths.get(Configuration.getTempPath(), TEST_FILE).toFile();
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(createdFile))) {
            writer.write("Hello world!");
        }
        InputStream in = new FileInputStream(createdFile);
        return new MockMultipartFile("file", "", "text", in);
    }
    
    /**
     * Crea un ficheiro multipart baleiro
     * @return Multipart
     * @throws IOException Excepción
     */
    private MockMultipartFile getEmptyMultipartFile() throws IOException {
        return new MockMultipartFile("file", TEST_FILE, "text", new byte[]{});
    }
    
    /**
     * Crea un ficheiro multipart cun tipo inválido
     * @return Multipart
     * @throws IOException Excepción
     */
    private MockMultipartFile getWrongContentTypeMultipartFile() throws IOException {
        File file = new File(Configuration.getDataPath() + "/invalid-content.png");
        InputStream in = new FileInputStream(file);
        return new MockMultipartFile("file", "invalid-content.png", "image/png", in);
    }
    
    /**
     * Configura o mock para devolver un documento cando se busque por un ID
     * @param id ID
     * @return Documento
     */
    private Documento mockBuscaDocumento(Long id) {
        Documento documento = new Documento();
        given(bibliotecaDocumentosService.buscar(id)).willReturn(documento);
        return documento;
    }
    
    /**
     * Realiza a petición de subida de documento, esperando erros
     * @param id ID do documento
     * @param file Ficheiro achegado
     * @throws Exception Erro
     */
    private void performFileUploadAndExpectError(Long id, MockMultipartFile file) throws Exception {
        MockMultipartHttpServletRequestBuilder builder = fileUpload(DETAIL_URL + id);
        if(file!=null) {
            builder.file(file);
        }
        // when: petición
        this.mockMvc.perform(builder.param(PARAM_TITULO, "a"))
        // then: valida que hai errores
                .andExpect(model().hasErrors())
                .andExpect(model().attributeHasFieldErrors(MODEL, PARAM_TITULO))
                .andExpect(status().isOk())
                .andDo(print());
    }
    
}
