/* Amtega Archetype for Spring-Web Applications
*
* Copyright (C) 2017 AMTEGA - Xunta de Galicia <componentes-transversais.amtega@xunta.gal>
*
* This file is part of "Amtega Archetype for Spring-Web Applications".
*
* "Amtega Archetype for Spring-Web Applications" is free software: you can redistribute it
* and/or modify it under the terms of:
* Apache License, Version 2.0 (the "License");
* or the European Union Public License, either Version 1.1 or – as soon
* they will be approved by the European Commission - subsequent versions of
* the EUPL;
*
* "Amtega Archetype for Spring-Web Applications" is distributed in the hope that
* it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* Apache Software License or European Union Public License for more details.
*
* You may obtain a copy of the Apache License, Version 2.0 at:
* http://www.apache.org/licenses/LICENSE-2.0
*
* You may obtain a copy of the European Union Public Licence at:
* http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*/

package es.xunta.amtega.codapp.documentos.web;

import static es.xunta.amtega.transversal.archam.core.web.util.HttpUtils.*;
import static org.hamcrest.CoreMatchers.*;
import static org.hamcrest.Matchers.hasProperty;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.user;
import static org.springframework.security.test.web.servlet.setup.SecurityMockMvcConfigurers.springSecurity;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.fileUpload;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.flash;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.header;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.model;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.view;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;
import javax.servlet.ServletException;

import org.junit.Before;
import org.junit.Test;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;

import es.xunta.amtega.codapp.documentos.BibliotecaDocumentosService;
import es.xunta.amtega.codapp.documentos.Documento;
import es.xunta.amtega.codapp.documentos.exception.DocumentoNoEncontradoException;
import es.xunta.amtega.codapp.test.AbstractIntegrationTest;
import es.xunta.amtega.transversal.archam.core.web.filter.ExecutionContextFilter;
import es.xunta.amtega.transversal.archam.core.web.filter.RefererManagerFilter;
import es.xunta.amtega.transversal.archam.core.web.mvc.Message;
import es.xunta.amtega.transversal.utils.BaseUtils;
import es.xunta.amtega.transversal.utils.BaseUtils.Configuration;
import java.util.Date;

/**
 * Test de integración das funcionalidades da biblioteca de
 * documentos, instanciando o contexto de Spring para validar que todos os
 * beans funcionan correctamente e sobreescribendo a configuración de base de
 * datos a H2 en memoria.
 * Para executalo, indicar goal "verify"
 */
public class BibliotecaDocumentosIT extends AbstractIntegrationTest {

    private static final String LIST_VIEW = "documentos/list";
    private static final String DETAILS_VIEW = "documentos/details";
    private static final String ERROR_VIEW = "error/500";

    private static final String MODEL_FORM = "documento";
    private static final String MODEL_PAGE = "page";
    private static final String MODEL_FILTER = "filter";

    private static final String BASE_URL = "/documentos";
    private static final String SEARCH_URL = BASE_URL + "/search";
    private static final String ADVANCED_SEARCH_URL = BASE_URL + "/advanced-search";
    private static final String ADD_URL = BASE_URL + "/add";
    private static final String DETAIL_URL = BASE_URL + "/";
    private static final String DELETE_URL = "/delete";
    private static final String DOWNLOAD_URL = "/download";

    private static final String PARAM_TITULO = "titulo";
    private static final String PARAM_DESCRIPCION = "descripcion";
    private static final String PARAM_FECHACREACION = "fechacreacion";
    private static final String PARAM_MESSAGE = "message";

    private static final String ROLE_USER = "USER";
    private static final String ROLE_ADMIN = "ADMIN";
    private static final String PERM_ALTA_DOCUMENTO = "ALTA_DOCUMENTO";
    private static final String PERM_MODIFICAR_DOCUMENTO = "MODIFICAR_DOCUMENTO";

    private static final String TEST_TITULO = "Título documento";
    private static final String TEST_DESCRIPCION = "Descripción documento";

    private static final String REDIRECT_TO = "redirect:";

    @Inject
    BibliotecaDocumentosService documentos;

    @Inject
    private WebApplicationContext wac;

    private MockMvc mockMvc;

    @Before
    public void setUp() throws ServletException {
        ExecutionContextFilter executionContextFilter = new ExecutionContextFilter();
        executionContextFilter.init(null);

        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.wac).addFilter(executionContextFilter, "/*")
                .addFilter(new RefererManagerFilter(), "/*").apply(springSecurity()).build();
    }

    /**
     * Valídase login dummy configuado en codapp.properties
     * @throws Exception Excepción
     */
    @Test
    public void login() throws Exception {
        // given: declaramos usuario e contrasinal
        String user = BaseUtils.Configuration.getProperty("test.user");
        String pass = BaseUtils.Configuration.getProperty("test.pass");
        
        // when: petición
        this.mockMvc
                .perform((post("/login").param("username", user).param("password", pass).param(REFERER_PARAM,
                        REFERER_PARAM.concat(EQ) + "/")).with(csrf()))
        // then: petición OK
                .andExpect(status().isFound()).andDo(print());
    }

    /**
     * Valida petición de documentos por GET
     * @throws Exception Excepción
     */
    @Test
    public void documentosForGet() throws Exception {
        // given: nada
        // when: petión ao listado de documentos
        this.mockMvc.perform((get(BASE_URL).with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN))))
        // then: atributos e vista OK
                .andExpect(view().name(LIST_VIEW)).andExpect(model().attributeExists(MODEL_PAGE))
                .andExpect(model().attributeExists(MODEL_FILTER)).andExpect(status().isOk()).andDo(print());
    }

    /**
     * Valida la petición de documentos por POST
     * @throws Exception Excepción
     */
    @Test
    public void documentosForPost() throws Exception {
        // given: nada
        // when: petión ao listado de documentos por POST
        MvcResult result = this.mockMvc
                .perform((post(BASE_URL).with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN))
                        .with(csrf())))
                // then: rediríxese á vista do erro 500
                // Estatus ok porque salta un isOk porque redirige a error/500
                .andExpect(view().name(ERROR_VIEW)).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(HttpRequestMethodNotSupportedException.class));
    }

    /**
     * Busca de documentos por GET
     * @throws Exception Excepción
     */
    @Test
    public void searchForGet() throws Exception {
        // given: nada
        // when: petición GET
        this.mockMvc
                .perform((get(SEARCH_URL).param("query", "search"))
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN)))
        // then: petición OK
                .andExpect(view().name(LIST_VIEW)).andExpect(model().attributeExists(MODEL_PAGE))
                .andExpect(model().attribute(MODEL_PAGE, hasProperty("totalElements", is(0L))))
                .andExpect(status().isOk()).andDo(print());
    }

    /**
     * Busca de documentos por POST
     * @throws Exception Excepción
     */
    @Test
    public void searchForPost() throws Exception {
        // given: nada
        // when: petición POST
        MvcResult result = this.mockMvc
                .perform((post(SEARCH_URL)).with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN))
                        .with(csrf()))
        // then: prodúcese un erro
                .andExpect(view().name(ERROR_VIEW)).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(MethodArgumentTypeMismatchException.class));
    }

    /**
     * Busca avanzada por GET
     * @throws Exception Excepción
     */
    @Test
    public void avancedSearchForGet() throws Exception {
        // given: nada
        // when: petición GET
        this.mockMvc
                .perform((get(ADVANCED_SEARCH_URL).param("nombre", "Nombre documento").param(PARAM_TITULO, "")
                        .param("formato", "txt")).with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN)))
        // then: petición OK
                .andExpect(view().name(LIST_VIEW)).andExpect(model().attributeExists(MODEL_PAGE))
                .andExpect(model().attribute(MODEL_PAGE, hasProperty("totalElements", is(0L))))
                .andExpect(status().isOk()).andDo(print());
    }

    /**
     * Busca avanzada por POST
     * @throws Exception Excepción
     */
    @Test
    public void avancedSearchForPost() throws Exception {
        // given: nada
        // when: petición POST
        MvcResult result = this.mockMvc
                .perform((post(ADVANCED_SEARCH_URL))
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN)).with(csrf()))
        // then: prodúcese un erro
                .andExpect(view().name(ERROR_VIEW)).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(MethodArgumentTypeMismatchException.class));
    }

    /**
     * Vista de novo documento
     * @throws Exception Excepción
     */
    @Test
    public void addForGet() throws Exception {
        // given: nada
        // when: petión url con los parámetros
        this.mockMvc.perform((get(ADD_URL).with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN))))
                // then: valida la vista cargada y sus variables
                .andExpect(view().name(DETAILS_VIEW)).andExpect(model().attributeExists(MODEL_FORM)).andDo(print());
    }

    /**
     * Gardado de novo documento con erros
     * @throws Exception Excepción
     */
    @Test
    public void saveNewWithErrors() throws Exception {

        // given: Inicialízase un ficheiro
        MockMultipartFile file = getNotEmptyMultipartFile();

        // when: petión url con los parámetros
        this.mockMvc
                .perform((fileUpload(ADD_URL).file(file).param(PARAM_TITULO, "a"))
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN)).with(csrf()))
                // then: Se valida la vista cargada, las variables de la vista y
                // los errores del formulario
                .andExpect(view().name(DETAILS_VIEW)).andExpect(model().hasErrors())
                // Validar campos
                .andExpect(model().attributeHasFieldErrors(MODEL_FORM, PARAM_TITULO)).andExpect(status().isOk())
                .andDo(print());
    }

    /**
     * Guardar nuevo sin permisos
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void saveNewWithoutErrorsAndAccessDeniedException() throws Exception {

        // given: Inicialízase un ficheiro
        MockMultipartFile file = getNotEmptyMultipartFile();

        // when: petión url con los parámetros
        MvcResult result = this.mockMvc
                .perform((fileUpload(ADD_URL).file(file).param(PARAM_TITULO, "Título").param("nombre", "Nombre")
                        .param(PARAM_DESCRIPCION, "Descripción").param(PARAM_FECHACREACION, new Date().toString()))
                                .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN)).with(csrf()))
                // then: se valida la vista de error y el tipo de excepción
                .andExpect(view().name("error/403")).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(AccessDeniedException.class));
    }

    /**
     * Guarda un nuevo documento sin errores
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void saveNewWithoutErrors() throws Exception {

        // given: Inicialízase un ficheiro + permisos necesarios
        MockMultipartFile file = getNotEmptyMultipartFile();
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;

        // when: petión url con los parámetros
        MvcResult result = this.mockMvc.perform(
                (fileUpload(ADD_URL).file(file).param(PARAM_TITULO, titulo).param(PARAM_DESCRIPCION, descripcion).param(PARAM_FECHACREACION, new Date().toString()))
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))
                        .with(csrf()))
                // then: validamos el resultado y que el documento se ha
                // guardado en la bbdd a través del servicio
                .andExpect(status().isFound()).andDo(print()).andReturn();

        // Sacamos el id del nombre de la vista
        String viewName = result.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // Validar si existe
        Documento documento = documentos.buscar(Long.parseLong(documentoId));
        assertThat(documento, is(notNullValue()));
        assertThat(documento.getId(), is(Long.parseLong(documentoId)));
        assertThat(documento.getNombre(), is(file.getOriginalFilename()));
        assertThat(documento.getTitulo(), is(titulo));
        assertThat(documento.getFormato(), is(file.getContentType()));
        assertThat(documento.getDescripcion(), is(descripcion));
    }

    /**
     * Guarda un documento con un título que ya existe
     * 
     * @throws Exception
     */
    @Test
    public void saveNewWithDocumentoYaExisteException() throws Exception {

        // given: Inicialízase un ficheiro + permisos
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        saveDocument(file, titulo, descripcion, authorities);

        // when: petión url con los parámetros
        this.mockMvc.perform(
                (fileUpload(ADD_URL).file(file).param(PARAM_TITULO, titulo).param(PARAM_DESCRIPCION, descripcion).param(PARAM_FECHACREACION, new Date().toString()))
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))
                        .with(csrf()))
                // then: validamos el error de título duplicado
                .andExpect(model().hasErrors()).andExpect(model().attributeHasFieldErrors(MODEL_FORM, PARAM_TITULO))
                .andExpect(status().isOk()).andDo(print()).andReturn();
    }

    /**
     * Edita un documento con errores
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void saveEditWithErrors() throws Exception {
        // given: Inicialízase un ficheiro + permisos
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // when: realizamos la petición con datos erróneos
        this.mockMvc
                .perform((fileUpload(DETAIL_URL + documentoId).file(file).param(PARAM_TITULO, "A")
                        .param(PARAM_DESCRIPCION, descripcion).param(PARAM_FECHACREACION, new Date().toString())).with(
                                user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))
                                .with(csrf()))
                // then: validamos que hay errores
                .andExpect(model().hasErrors()).andExpect(model().attributeHasFieldErrors(MODEL_FORM, PARAM_TITULO))
                .andExpect(status().isOk()).andDo(print()).andReturn();
    }

    /**
     * Edición de un documento que ya existe con título que ya existe
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void saveEditWithDocumentoYaExisteException() throws Exception {

        // given: Inicialízase un ficheiro + permisos
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String titulo2 = "Título documento 2";
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        saveDocument(file, titulo, descripcion, authorities);
        MvcResult saveResult = saveDocument(file, titulo2, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // when: Realizamos la petición para cambiar el título del documento 2
        // (titulo1)
        // por titulo2 lo que produce la excepción
        this.mockMvc
                .perform((fileUpload(DETAIL_URL + documentoId).file(file).param(PARAM_TITULO, titulo)
                        .param(PARAM_DESCRIPCION, descripcion).param(PARAM_FECHACREACION, new Date().toString())).with(
                                user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))
                                .with(csrf()))
                // then: validamos que hay errores en el título
                .andExpect(model().hasErrors()).andExpect(model().attributeHasFieldErrors(MODEL_FORM, PARAM_TITULO))
                .andExpect(status().isOk()).andDo(print()).andReturn();
    }

    /**
     * Edita un documento existente. Primero guardamos un documento nuevo,
     * recuperamos el id del documento guardado. Lo editamos y comprobamos que
     * se ha realizado la edición correctamente.
     * 
     * @throws Exception
     *             Excepción
     */
    @Transactional
    @Test
    public void saveEdit() throws Exception {
        // given: Inicializa las variables y guarda un documento
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);
        String otroTitulo = "Otro título";
        String otraDescripcion = "Otra descripción";

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // when: Editamos el documento correctamente
        this.mockMvc
                .perform((fileUpload(DETAIL_URL + documentoId).file(file).param(PARAM_TITULO, otroTitulo)
                        .param(PARAM_DESCRIPCION, otraDescripcion).param(PARAM_FECHACREACION, new Date().toString())).with(
                                user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))
                                .with(csrf()))
                // then:Validamos que no hay errores
                .andExpect(model().hasNoErrors()).andExpect(status().isFound()).andDo(print()).andReturn();

        // Validamos que se ha editado correctamente
        Documento documento = documentos.buscar(Long.parseLong(documentoId));
        assertThat(documento, is(notNullValue()));
        assertThat(documento.getId(), is(Long.parseLong(documentoId)));
        assertThat(documento.getNombre(), is(file.getOriginalFilename()));
        assertThat(documento.getTitulo(), is(otroTitulo));
        assertThat(documento.getFormato(), is(file.getContentType()));
        assertThat(documento.getDescripcion(), is(otraDescripcion));
    }

    /**
     * Intenta descargar un fichero que no existe
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void downloadForGetWithDocumentoNoEncontradoException() throws Exception {
        // given: guardamos un documento e inicializamos las variables
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        // Doucmento que no existe
        String documentoId = "56445";

        // when: realizamos la petición para descargar und documento que no
        // existe
        MvcResult result = this.mockMvc
                .perform((get(DETAIL_URL + documentoId + DOWNLOAD_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))))
                // then: Se valida el estado y la excepción producida
                .andExpect(status().isNotFound()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(DocumentoNoEncontradoException.class));
    }

    /**
     * Descarga un documento
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void downloadForGet() throws Exception {
        // given: inicializamos variables y guardamos un documento
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // Lo buscamos directamente en el servicio para obtener el uuid
        Documento documento = documentos.buscar(Long.parseLong(documentoId));

        // when: Lo descargamos
        this.mockMvc
                .perform((get(DETAIL_URL + documento.getUuid() + DOWNLOAD_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN))))
                // then: Valiadamos que se descarga correctamente
                .andExpect(status().isOk())
                .andExpect(header().string("Content-Type", file.getContentType())).andExpect(header()
                        .string("Content-disposition", "attachment; filename=\"" + file.getOriginalFilename() + "\""))
                .andDo(print());
    }

    /**
     * Descargar documento por POST
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void downloadForPost() throws Exception {
        // given: inicializamos variables y guardamos un documento
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // Lo buscamos directamente en el servicio para obtener el uuid
        Documento documento = documentos.buscar(Long.parseLong(documentoId));

        // when: Lo intentamos descargar
        MvcResult result = this.mockMvc
                .perform((post(DETAIL_URL + documento.getUuid() + DOWNLOAD_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN))).with(csrf()))
                // then: validamos el error de método no soportado, el tipo de
                // excepción y la vista de error
                .andExpect(view().name(ERROR_VIEW)).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(HttpRequestMethodNotSupportedException.class));
    }

    /**
     * Borramos un fichero por post
     * 
     * @throws Exception
     */
    @Test
    public void deleteForPost() throws Exception {
        // given: inicializamos las variables y guardamos un documento
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO), new SimpleGrantedAuthority("BORRAR_DOCUMENTO"));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // when: Lo eliminamos mediante una petición post
        this.mockMvc
                .perform((post(DETAIL_URL + documentoId + DELETE_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities)))
                                .with(csrf()))
                // then:validamos la redirección, los mensajes y que se ha
                // borrado de bbdd a través del servicio
                .andExpect(view().name(REDIRECT_TO + BASE_URL)).andExpect(flash().attributeExists(PARAM_MESSAGE))
                .andExpect(flash().attribute(PARAM_MESSAGE,
                        hasProperty(PARAM_MESSAGE, is("documentos.documento.borrado.exito"))))
                .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty("type", is(Message.Type.SUCCESS))))
                .andExpect(status().isFound()).andDo(print());

        // Buscamos por id con el servicio directamente
        try {
            documentos.buscar(Long.parseLong(documentoId));
            fail("Se ha encontrado un documento que se debería haber borrado");
        } catch (DocumentoNoEncontradoException e) {
            assertThat(e, is(notNullValue()));
        }
    }

    /**
     * Borrado sin permisos
     * 
     * @throws Exception
     *             Excepcion
     */
    @Test
    public void deleteForPostAccessDeniedException() throws Exception {
        // given: inicializamos variables y guardamos un documento
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // when: Lo intentamos eliminar sin el permiso para eliminar documentos
        MvcResult result = this.mockMvc
                .perform((post(DETAIL_URL + documentoId + DELETE_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities)))
                                .with(csrf()))
                // then: validamos el error de permisos
                .andExpect(view().name("error/403")).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(AccessDeniedException.class));
    }

    /**
     * Intentamos eliminar un documento que no existe
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void deleteForPostDocumentoNoEncontradoException() throws Exception {

        // given: declaramos un id que no existe y los permisos
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO), new SimpleGrantedAuthority("BORRAR_DOCUMENTO"));

        Long documentoId = 23423L;

        // when: elimimanos fichero que no existe
        this.mockMvc
                .perform((post(DETAIL_URL + documentoId + DELETE_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities)))
                                .with(csrf()))
                // then: validamos el error al intentar eliminar
                .andExpect(view().name(REDIRECT_TO + BASE_URL)).andExpect(status().isFound())
                .andExpect(flash().attributeExists(PARAM_MESSAGE))
                .andExpect(flash().attribute(PARAM_MESSAGE,
                        hasProperty(PARAM_MESSAGE, is("documentos.documento.borrado.error"))))
                .andExpect(flash().attribute(PARAM_MESSAGE, hasProperty("type", is(Message.Type.DANGER))))
                .andDo(print());
    }

    /**
     * Eliminar un documento por GET
     * 
     * @throws Exception
     *             Excepción
     */
    @Test
    public void deleteForGet() throws Exception {

        // given: guardamos un documento y inicializamos los permisos
        List<SimpleGrantedAuthority> authorities = Arrays.asList(new SimpleGrantedAuthority(PERM_ALTA_DOCUMENTO),
                new SimpleGrantedAuthority(PERM_MODIFICAR_DOCUMENTO));
        String titulo = TEST_TITULO;
        String descripcion = TEST_DESCRIPCION;
        MockMultipartFile file = getNotEmptyMultipartFile();
        MvcResult saveResult = saveDocument(file, titulo, descripcion, authorities);

        // Sacamos el id del nombre de la vista
        String viewName = saveResult.getModelAndView().getViewName();
        String[] tokens = viewName.split("/");
        String documentoId = tokens[tokens.length - 1];

        // when: eliminamos el documento enviado una petición GET
        MvcResult result = this.mockMvc
                .perform((get(DETAIL_URL + documentoId + DELETE_URL)
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities)))
                                .with(csrf()))
                // then: validamos el error de método no permitido
                .andExpect(view().name(ERROR_VIEW)).andExpect(status().isOk()).andDo(print()).andReturn();
        assertThat(result.getResolvedException(), instanceOf(HttpRequestMethodNotSupportedException.class));
    }

    /**
     * Guarda un documento a través del método del controller
     * 
     * @param file
     *            Fichero físico
     * @param titulo
     *            Título del fichero
     * @param descripcion
     *            Descripción
     * @param authorities
     *            Permisos
     * @return Resultado
     * @throws Exception
     *             Excepción
     */
    private MvcResult saveDocument(MockMultipartFile file, String titulo, String descripcion,
            List<SimpleGrantedAuthority> authorities) throws Exception {

        // Insertamos el documento
        return this.mockMvc.perform(
                (fileUpload(ADD_URL).file(file).param(PARAM_TITULO, titulo).param(PARAM_DESCRIPCION, descripcion).param(PARAM_FECHACREACION, new Date().toString()))
                        .with(user(ROLE_ADMIN).password("pass").roles(ROLE_USER, ROLE_ADMIN).authorities(authorities))
                        .with(csrf()))
                // then
                .andExpect(status().isFound()).andDo(print()).andReturn();
    }

    /**
     * Crea un fichero Mock no vacio
     * 
     * @return Fichero multipart
     * @throws IOException
     *             Excepcion
     */
    private MockMultipartFile getNotEmptyMultipartFile() throws IOException {
        File createdFile = new File(Configuration.getTempPath() + "/foo.txt");
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(createdFile))) {
            writer.write("Hello world!");
        }
        InputStream in = new FileInputStream(createdFile);
        return new MockMultipartFile("file", "foo.txt", "text", in);
    }
}
